#!/bin/sh
#
# $Id: bin/appManager.sh 1.42 2007/09/07 09:55:47MDT Jim Boyd (jboyd) dev  $
##

## SP3 (320G.GS00600) this will need to be changes for every release of the firmware.
VALID_DOWN_GRADE_VERSION=320

applicationPid=$$
fileBuffer="/tmp/.${applicationPid}.fileNameToUseAsBufferForCommandOutput.${applicationPid}"

## Path to application home
applicationHome="/home/embedded/library"
## DEBUG applicationHome="/home/jboyd/shaggy/Code/Applications/Scalar101/S101Components"
## Source include file.
. ${applicationHome}/AppManager/bin/include.sh

## Application Manager scripts and images.
appManagerFullpathName="${applicationHome}/AppManager/bin/appManager.sh"
firmwareManagerScriptName="${applicationHome}/AppManager/bin/fmrManager.sh"
applicationIncludeName="${applicationHome}/AppManager/bin/include.sh"
imagesDirectory="${applicationHome}/AppManager/images"

## DEBUG applicationHome="/home/embedded/library"

## Status of 0 is NOT running, 1 is running
APPLICATION_STATUS=0

## Corrective Action flag
installCorrectiveAction="false"

## Files used to communicate information over restarts of this script.
forceRestartUpgradeNeeded="${applicationHome}/AppManager/forceRestartUpgradeNeeded"
forceRestartRecoveryNeeded="${applicationHome}/AppManager/forceRestartRecoveryNeeded"
forceRestartRestoreNeeded="${applicationHome}/AppManager/forceRestartRestoreNeeded"
upgradeCompletedCheckRestart="${applicationHome}/images/upgradeCompletedCheckRestart"

## Application names.
s101coreBaseName="S101Core.exe"
consoleApplication="${applicationHome}/ConsoleService/bin/ConsoleMgmtApplication.exe"
consoleApplicationBaseName=`basename ${consoleApplication}`
apacheApplicationControl="${applicationHome}/HTTPService/apache/bin/apachectl"
snmpApplication="${applicationHome}/SNMPService/sbin/snmpd"
snmpApplicationBaseName=`basename ${snmpApplication}`
smisApplicationName="owcimomd"
smisApplicationControl="${applicationHome}/SMI-SService/bin/smis_control"
snmpTrapApplication="${applicationHome}/SNMPService/sbin/snmptrapd"
snmpTrapApplicationBaseName=`basename ${snmpTrapApplication}`

## Application status indicators
SMISenabled="/etc/SMISenabled"

#########################################
# Signal handler
# Inputs: 
#     1. The signal that was caught by the application
#
#########################################
ProcessSignal()
{
   local caughtSignal="$1"
   LogTrace "Got signal ${caughtSignal}, exit appManager" 
   exit 0
}

#########################################
# S101Core application control, used to start, stop and get status of
# application
# Inputs: 
#     1. The state transition, "start", "stop" or "status"
# Outputs:
#     1. The "status" option sets the global variable APPLICATION_STATUS
#
#########################################
CoreApplicationControl()
{
   local state="$1"
   local command=""
   local output=""
   local timeout=0

   if [ "${state}" = "start" ]; then
      LogTrace "Starting Core application" 
      if [ ! -d "${applicationHome}/CoreService/dat" ]; then
         command="mkdir ${applicationHome}/CoreService/dat"
         RunCommand "$command" 
      fi

      if [ "${sendCoreOutputToConsole}" = "f" ]; then
         eval "cd ${applicationHome}/CoreService/dat && ../bin/${s101coreBaseName} 2>&1 &"
      else
         eval "cd ${applicationHome}/CoreService/dat && ../bin/${s101coreBaseName} >/dev/null 2>&1 &"
      fi
      sleep 2
      CheckApplicationsStatus "${s101coreBaseName}"
      if [ ${APPLICATION_STATUS} -ne 1 ]; then
         LogFatal "Failed to start Core application" 
         exit 1
      else
         LogTrace "Core started successfully" 
      fi
   fi

   if [ "${state}" = "stop" ]; then
      ## Lets check if old AppMgr is running, if so we kill it.
      output=`ps -o pid --no-heading -C AppMgr.exe`
      if [ $? -eq 0 ]; then
         command="kill -9 $output"
         RunCommand "$command"
      fi
      TerminateApplication "${s101coreBaseName}" "S101Core" "15"
      ## Give it one minute to die, then kill it with signal 9
      while [ 1 ]; do
         sleep 1
         CheckApplicationsStatus "${s101coreBaseName}"
         if [ ${APPLICATION_STATUS} -eq 1 ]; then
            timeout=`expr $timeout + 1`
            if [ ${timeout} -gt 60 ]; then
               LogTrace "Need to terminate Core with signal 9" 
               TerminateApplication "${s101coreBaseName}" "S101Core" "9"
               break
            fi
         else
            break
         fi
      done
      CheckApplicationsStatus "${s101coreBaseName}"
      if [ ${APPLICATION_STATUS} -eq 0 ]; then
         LogTrace "Core application stopped" 
      else
         LogError "Core application failed to terminate" 
      fi
   fi

   if [ "${state}" = "status" ]; then
      CheckApplicationsStatus "${s101coreBaseName}"
   fi
}

#########################################
# Console application control, used to start, stop and get status of
# application
# Inputs: 
#     1. The state transition, "start", "stop" or "status"
# Outputs:
#     1. The "status" option sets the global variable APPLICATION_STATUS
#
#########################################
ConsoleApplicationControl()
{
   local state="$1"
   local personality=""

   if [ "${state}" = "start" ]; then
      LogTrace "Starting Console application" 
      eval "${consoleApplication} >/dev/null 2>&1 &"
      sleep 2
      CheckApplicationsStatus "${consoleApplicationBaseName}"
      if [ ${APPLICATION_STATUS} -eq 0 ]; then
         LogError "Failed to start Console application"
      else
         LogTrace "Console started successfully" 
      fi
      ## if [ $? != 0 ]; then
         ## LogError "Failed to start Console application"
      ## else
         ## LogTrace "Console started successfully" 
      ## fi
   fi

   if [ "${state}" = "stop" ]; then
      if [ -f "${userChangeingLibraryPersonality}" ]; then
         sleep 2
         personality=`grep ADICLIBRARY /etc/profile | awk -F= '{print $2}'`
         export ADICLIBRARY_BRAND="${personality}"
         LogTrace "Set personality to ${personality}"
         RunCommand "rm ${userChangeingLibraryPersonality}"
      fi
      ## We need to do this twice, somehow peg is catching the signal and not dieing.
      ## TODO: Figure out why signal 15 is not killing console
      TerminateApplication "${consoleApplicationBaseName}" "Console" "15"
      CheckApplicationsStatus "${consoleApplicationBaseName}"
      if [ ${APPLICATION_STATUS} -eq 0 ]; then
         LogTrace "Console application stopped" 
         if [ -f "/tmp/.rebootInProgress" ]; then
            RunCommand "cp /home/embedded/library/AppManager/images/rebootImage /dev/fb0"
            RunCommand "rm /tmp/.rebootInProgress"
         fi
         if [ -f "/tmp/.shutdownInProgress" ]; then
            RunCommand "cp /home/embedded/library/AppManager/images/shutdownImage /dev/fb0"
            RunCommand "rm /tmp/.shutdownInProgress"
         fi
      else
         LogError "Console application failed to terminate with SIGTERM, sending it signal 9" 
         TerminateApplication "${consoleApplicationBaseName}" "Console" "9"
         if [ -f "/tmp/.rebootInProgress" ]; then
            RunCommand "cp /home/embedded/library/AppManager/images/rebootImage /dev/fb0"
            RunCommand "rm /tmp/.rebootInProgress"
         fi
         if [ -f "/tmp/.shutdownInProgress" ]; then
            RunCommand "cp /home/embedded/library/AppManager/images/shutdownImage /dev/fb0"
            RunCommand "rm /tmp/.shutdownInProgress"
         fi
      fi
   fi

   if [ "${state}" = "status" ]; then
      CheckApplicationsStatus "${consoleApplicationBaseName}"
   fi
}

#########################################
# HTTP application control, used to start, stop and get status of
# application
# Inputs: 
#     1. The state transition, "start", "stop" or "status"
# Outputs:
#     1. The "status" option sets the global variable APPLICATION_STATUS
#
#########################################
HTTPapplicationControl()
{
   local state="$1"
   local pid=""
   local isSSLenabled=0

   isSSLenabled=`cat /etc/extaccess.conf | grep ssl_gui_flag | awk -F= '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`

   if [ "${state}" = "start" ]; then
      LogTrace "Start HTTP application" 
      ## Lets clean up, stop application if its already running
      `pidof httpd httpsd >/dev/null`
      if [ $? = 0 ]; then
         RunCommand "kill -9 `pidof httpd httpsd` >/dev/null 2>&1"
      fi
      `pidof gcache >/dev/null`
      if [ $? = 0 ]; then
         RunCommand "kill -9 `pidof gcache` >/dev/null 2>&1"
      fi
      ## Remove the pid file
      if [ -f "${applicationHome}/HTTPService/apache/logs/httpd.pid" ]; then
         RunCommand "rm ${applicationHome}/HTTPService/apache/logs/httpd.pid"
      fi

      if [ ${isSSLenabled} -eq 0 ]; then
         eval "$apacheApplicationControl start >/dev/null 2>&1"
         if [ $? != 0 ]; then
            LogError "Failed to start HTTP application" 
         else
            LogTrace "HTTP started successfully" 
         fi
      else
         LogTrace "Start HTTP in secure mode (SSL)" 
         if [ -f "${applicationHome}/HTTPService/apache/apache/bin/setssl.sh" ]; then
            RunCommand "sh ${applicationHome}/HTTPService/apache/apache/bin/setssl.sh enable"
         fi
         eval "$apacheApplicationControl -DSSL -k start >/dev/null 2>&1"
         if [ $? != 0 ]; then
            LogError "Failed to start HTTP application" 
         else
            LogTrace "HTTP started successfully" 
         fi
      fi
   fi

   if [ "${state}" = "stop" ]; then
      LogTrace "Stopping HTTP application" 
      if [ -f "${applicationHome}/HTTPService/apache/logs/httpd.pid" ]; then
         pid=`cat ${applicationHome}/HTTPService/apache/logs/httpd.pid`
      fi
      if [ "$pid" != "" ]; then
         eval "$apacheApplicationControl stop >/dev/null 2>&1"
         if [ $? != 0 ]; then
            LogError "Failed to stop HTTP application, lets use kill"
            RunCommand "kill -9 `pidof httpd` `pidof gcache` >/dev/null 2>&1"
         fi
      else
         local pidsToKill=""
         pidsToKill=`pidof httpd httpsd 2>&1`
         if [ $? = 0 ]; then
            RunCommand "kill -9 ${pidsToKill} >/dev/null 2>&1"
         fi
         pidsToKill=`pidof gcache 2>&1`
         if [ $? = 0 ]; then
            RunCommand "kill -9 ${pidsToKill} >/dev/null 2>&1"
         fi
      fi
      sleep 2
      HTTPapplicationControl "status"
      if [ ${APPLICATION_STATUS} -eq 0 ]; then
         LogTrace "HTTP application stopped" 
      else
         LogError "HTTP failed to terminate" 
      fi
   fi

   if [ "${state}" = "status" ]; then
      if [ -f "${applicationHome}/HTTPService/apache/logs/httpd.pid" ]; then
         pid=`cat /home/embedded/library/HTTPService/apache/logs/httpd.pid`
         eval "ps -ef | grep -v grep | grep $pid >/dev/null"
         if [ $? -eq 0 ]; then
            APPLICATION_STATUS=1
         else
            pidsToKill=`ps -ef | egrep -e httpd -e httpsd >/dev/null 2>&1`
            if [ $? -eq 0 ]; then
               APPLICATION_STATUS=1
            else
               APPLICATION_STATUS=0
            fi
         fi
      else
         pid=`ps -ef | egrep -e httpd -e httpsd | grep -v grep >/dev/null 2>&1`
         if [ $? -eq 0 ]; then
            APPLICATION_STATUS=1
         else
            APPLICATION_STATUS=0
         fi
      fi
   fi
}

#########################################
# SNMP application control, used to start, stop and get status of
# application
# Inputs: 
#     1. The state transition, "start", "stop" or "status"
# Outputs:
#     1. The "status" option sets the global variable APPLICATION_STATUS
#
#########################################
SNMPapplicationControl()
{
   local state="$1"

   if [ "${state}" = "start" ]; then
      LogTrace "Starting SNMP application" 

      # Determine which ports for SNMP to listen on.
      local SNMPListenPorts="udp:161"
      if [ "${IPv6}" = "true" ]; then
          SNMPListenPorts="${SNMPListenPorts} udp6:161"
      fi

      local applicationArguments="-p /var/run/snmpd.pid -A -Lf /home/embedded/library/SNMPService/dat/snmpd.log ${SNMPListenPorts}"

      eval "${snmpApplication} ${applicationArguments}"
      if [ $? != 0 ]; then
         LogError "Failed to start SNMP application"
      else
         LogTrace "SNMP started successfully" 
      fi
   fi

   if [ "${state}" = "stop" ]; then
      TerminateApplication "${snmpApplicationBaseName}" "SNMP" "9"
      sleep 2
      CheckApplicationsStatus "${snmpApplicationBaseName}"
      if [ ${APPLICATION_STATUS} -eq 0 ]; then
         LogTrace "SNMP application stopped" 
      else
         LogError "SNMP application failed to terminate" 
      fi
   fi

   if [ "${state}" = "status" ]; then
      CheckApplicationsStatus "${snmpApplicationBaseName}"
   fi
}

#########################################
# SNMP TRAP daemon application control, used to start, stop and get status of
# application
# Inputs: 
#     1. The state transition, "start", "stop" or "status"
# Outputs:
#     1. The "status" option sets the global variable APPLICATION_STATUS
#
#########################################
SNMPTrapApplicationControl()
{
   local state="$1"

   if [ "${state}" = "start" ]; then
      LogTrace "Starting SNMP Trap daemon application" 
      local applicationArguments="-p /var/run/snmptrapd.pid -n -Os -Lf /home/embedded/library/SNMPService/dat/snmptrapd.log"
      if [ -f "${snmpTrapApplication}" ]; then
         eval "${snmpTrapApplication} ${applicationArguments}"
         if [ $? != 0 ]; then
            LogError "Failed to start SNMP Trap daemon application" 
         else
            LogTrace "SNMP Trap daemon started successfully" 
         fi
      fi
   fi

   if [ "${state}" = "stop" ]; then
      if [ -f "${snmpTrapApplication}" ]; then
         TerminateApplication "${snmpTrapApplicationBaseName}" "SNMP Trap daemon" "15"
      fi
      sleep 2
      CheckApplicationsStatus "${snmpTrapApplicationBaseName}"
      if [ ${APPLICATION_STATUS} -eq 0 ]; then
         LogTrace "SNMP Trap daemon stopped" 
      else
         LogError "SNMP Trap daemon failed to terminate" 
      fi
   fi

   if [ "${state}" = "status" ]; then
      if [ -f "${snmpTrapApplication}" ]; then
         CheckApplicationsStatus "${snmpTrapApplicationBaseName}"
      else
         APPLICATION_STATUS=1
      fi
   fi
}

#########################################
# SMIS application control, used to start, stop and get status of
# application
# Inputs: 
#     1. The state transition, "start", "stop" or "status"
# Outputs:
#     1. The "status" option sets the global variable APPLICATION_STATUS
#
#########################################
SMISapplicationControl()
{
   local state="$1"

   if [ ! -f "$SMISenabled" ]; then
      LogTrace "SMI-S is DISABLED."
      state="disabled"
   fi

   if [ "${state}" = "start" ]; then
      LogTrace "Starting SMI-S application" 
      eval "${smisApplicationControl} start >/dev/null 2>&1"
      CheckApplicationsStatus "${smisApplicationName}"
      if [ ${APPLICATION_STATUS} -eq 1 ]; then
         LogTrace "SMI-S started successfully" 
      else
         LogError "SMI-S failed to start" 
      fi
   fi

   if [ "${state}" = "stop" ]; then
      LogTrace "Stopping SMI-S application" 
      eval "$smisApplicationControl stop >/dev/null 2>&1"
      if [ $? != 0 ]; then
         local pidsToKill=""
         pidsToKill=`pidof owcimomd 2>&1`
         if [ $? = 0 ]; then
            RunCommand "kill -9 ${pidsToKill} >/dev/null 2>&1"
         fi
         pidsToKill=`pidof slpd 2>&1`
         if [ $? = 0 ]; then
            RunCommand "kill -9 ${pidsToKill} >/dev/null 2>&1"
         fi
      fi
      sleep 2
      CheckApplicationsStatus "${smisApplicationName}"
      if [ ${APPLICATION_STATUS} -eq 0 ]; then
         LogTrace "SMI-S stopped" 
      else
         LogError "SMI-S failed to terminate" 
      fi
   fi

   if [ "${state}" = "status" ]; then
      CheckApplicationsStatus "${smisApplicationName}"
   fi
}


#########################################
# Terminates a process
# Inputs: 
#     1. The process name
#     2. The process name used for logging
#     3. The SIGNAL used to kill the process
#
#########################################
TerminateApplication()
{
   local processName="$1"
   local name="$2"
   local signal="$3"

   LogTrace "Stopping ${name} application, with signal ${signal}" 
   output=`ps -o pid --no-heading -C ${processName} | grep -v defunct`
   if [ $? -eq 0 ]; then
      command="killall -e -q -${signal} ${processName}"
      RunCommand "${command}"
   else
      LogTrace "${name} is not running" 
   fi
}

#########################################
# Check if an application is running
# Inputs: 
#     1. The process name
# Outputs:
#     1. The global variable APPLICATION_STATUS is set to 0 (not running) or #     1 (is running)
#
#########################################
CheckApplicationsStatus()
{
   local processName="$1"
   local output=""
   output=`ps --no-heading -C ${processName} | grep -v defunct`
   if [ $? -eq 0 ]; then
      APPLICATION_STATUS=1
   else
      APPLICATION_STATUS=0
   fi
}

#########################################
# Start/Stop the application sowtware
# Inputs: 
#     1. The state requested, "start", "stop" or "status"
#     2. Optional requested status
#
#########################################
ApplicationControl()
{
   local state="$1"
   local requestedStatus="$2"

   if [ "${requestedStatus}" = "" ]; then
      requestedStatus="1"
   fi

   if [ "${state}" = "start" ]; then 
      RunCommand "/etc/init.d/ntpd restart"
      SNMPTrapApplicationControl "${state}"
      if [ "${dontRestartCoreIfAlreadyRunning}" = "i" ]; then
         CoreApplicationControl "status"
      else
         CoreApplicationControl "${state}"
         CoreApplicationControl "status"
      fi
      if [ ${APPLICATION_STATUS} -eq 1 ]; then
         ConsoleApplicationControl "${state}"
         ConsoleApplicationControl "status"
         if [ ${APPLICATION_STATUS} -eq 1 ]; then
            HTTPapplicationControl "${state}"
            HTTPapplicationControl "status"
            if [ ${APPLICATION_STATUS} -eq 1 ]; then
               SNMPapplicationControl "${state}"
               SNMPapplicationControl "status"
               if [ ${APPLICATION_STATUS} -eq 1 ]; then
                  SMISapplicationControl "${state}"
               fi
            fi
         fi
      fi
   fi

   if [ "${state}" = "stop" ]; then
      TerminateApplication "${snmpApplicationBaseName}" ", sending SIGUSR2 SNMP" "12"
      ConsoleApplicationControl "${state}"
      CoreApplicationControl "${state}"
      HTTPapplicationControl "${state}"
      SNMPapplicationControl "${state}"
      SNMPTrapApplicationControl "${state}"
      SMISapplicationControl "${state}"
   fi

   if [ "${state}" = "status" ]; then
      CoreApplicationControl "${state}"
      if [ ${APPLICATION_STATUS} -eq ${requestedStatus} ]; then
         ConsoleApplicationControl "${state}"
         if [ ${APPLICATION_STATUS} -eq ${requestedStatus} ]; then
            HTTPapplicationControl "${state}"
            if [ ${APPLICATION_STATUS} -eq ${requestedStatus} ]; then
               SNMPapplicationControl "${state}"
               if [ ${APPLICATION_STATUS} -eq ${requestedStatus} ]; then
                  SNMPTrapApplicationControl "${state}"
                  if [ ${APPLICATION_STATUS} -eq ${requestedStatus} ]; then
                     if [ ! -f "$SMISenabled" ]; then
                        APPLICATION_STATUS=${requestedStatus}
                     else
                        SMISapplicationControl "${state}"
                     fi
                  fi
               fi
            fi
         fi
      fi
   fi
}

#########################################
# Start i500 application sowtware, if its already running just exit
# Inputs: 
#     1. Force restart if application is already running (optional)
#
#########################################
StartAppManager()
{
   local gotAMatch=""
   local line=""
   local forceRestart="$1"
   local name=""

   LogTrace "Starting applications ..."
   name=`basename ${appManagerFullpathName}`
   if [ "$forceRestart" != "" ]; then
      sleep 2  ## Give existing script to exit gracefully
      eval "ps -no-headers -ef | grep ${name} | grep -v grep | grep -v ' ${applicationPid} ' | grep -v gvim >$fileBuffer" 
      while [ 1 ]
      do
         read line || break
         gotAMatch="true"
         LogTrace "Application already running will force restart" 
         kill -9 `echo $line | awk '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
      done < $fileBuffer
      if [ "${gotAMatch}" != "" ]; then ## A force restart was requested need to stop all applications
         APPLICATION_STATUS=0
         ApplicationControl "status" "0"
         if [ ${APPLICATION_STATUS} -ne 0 ]; then
            ApplicationControl "stop"
         fi
      fi
   else
      eval "ps -no-headers -ef | grep ${name} | grep -v grep | grep -v ' ${applicationPid} ' | grep -v gvim >$fileBuffer" 
      while [ 1 ]
      do
         read line || break
         LogTrace "Application is already running" 
         rm "$fileBuffer"
         exit
      done < $fileBuffer
   fi
   rm "$fileBuffer"
}

#########################################
# Stop i500 application sowtware
# Inputs: 
#
#########################################
StopAppManager()
{
   local gotAMatch=""
   local line=""
   local name=""

   ## Power down blades.
   RunCommand "/usr/bin/bmmcommands -D"

   name=`basename ${appManagerFullpathName}`
   LogTrace "Stopping i500 Application Firmware ...................." 
   eval "ps -no-headers -ef | grep ${name} | grep -v grep | grep -v ' ${applicationPid} ' | grep -v gvim >$fileBuffer"
   while [ 1 ]
   do
      read line || break
      gotAMatch="true"
      kill -9 `echo "$line" | awk '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
   done < $fileBuffer
   if [ "$gotAMatch" = "" ]; then
      LogTrace "$name was not running?" 
   fi

   ## Check if the rest of the application porocess are running
   APPLICATION_STATUS=0
   ApplicationControl "status" "0"
   if [ ${APPLICATION_STATUS} -ne 0 ]; then
      ApplicationControl "stop"
   fi
   rm "$fileBuffer"
}

#########################################
# Install firmware initialization process. We first extract new/old appManager.sh,
# fmrManager.sh and include files from distribution, touch a file to indicate
# upgrade, restore or recovery and restart appManager.
#
# Inputs: 
#
#########################################
InitalizeLibraryFirmware()
{
   local whatToDo="$1"
   local command=""
   local output=""
   local name=""
   local firmwareDistrubutionName=""

   RunCommand "/usr/bin/initlcd --wakeup"
   if [ "${whatToDo}" = "upgrade" ]; then

      LogTrace "Prepare for library firmware upgrade" 

      RunCommand "cp ${imagesDirectory}/upgradeImage /dev/fb0"

      CreateRAMdisk
      firmwareDistrubutionName="${upgradeFirmwareFileName}"
      ValidateDistribution "upgrade" "${firmwareDistrubutionName}"
      if [ ${COMMAND_EXIT_STATUS} -ne 0 ]; then
         LogError "Failed to validate firmware distribution" 
         if [ -f "${firmwareDistrubutionName}" ]; then
            RunCommand "rm ${firmwareDistrubutionName}"
         fi
         UnmountRAMdisk
         RestartApplicationManager
      fi

      if [ ! -f "${forceRestartUpgradeNeeded}" ]; then
         command="touch ${forceRestartUpgradeNeeded}"
         RunCommand "$command" 
      fi

      if [ "X${installCorrectiveAction}" != "Xtrue" ]; then
         LogTrace "Extract new app Manager code" 
         command="tar xzf ${firmwareDistrubutionName} -C ${ramDiskMountPoint} ./AppManager/bin/*"
         RunCommand "${command}" 
         ExtractApplicationManagerScripts

         ## Check if there were any errors
         if [ ${COMMAND_EXIT_STATUS} -ne 0 ]; then
            LogTrace "Failed to extract new application manager scripts" 
            ## TODO: Do we need to abandon the upgrade.
            rm ${forceRestartUpgradeNeeded}
         else
            if [ -f "${bladeDistributionDirectory}" ]; then
               RunCommand "rm ${bladeDistributionDirectory}/${bladeImage}"
               RunCommand "rm ${bladeDistributionDirectory}/${bladeDistribution}"
            fi
            LogTrace "Restart with new appManager" 
         fi
      fi
      UnmountRAMdisk
      RestartApplicationManager
   fi

   if [ "${whatToDo}" = "recover" -o "${whatToDo}" = "restore" ]; then

      local filenameFirmwareUpdateNeeded=""
      local fileIndicatorName=""
      local message=""

      if [ "${whatToDo}" = "recover" ]; then
         RunCommand "cp ${imagesDirectory}/downgradeImage /dev/fb0"
         LogTrace "Prepare for library firmware recovery" 
         firmwareDistrubutionName="${systemBackupFile}"
         fileIndicatorName="${forceRestartRecoveryNeeded}"
         message="recovery"
      else
         LogTrace "Prepare for library firmware restore" 
         RunCommand "cp ${imagesDirectory}/restoreImage /dev/fb0"
         firmwareDistrubutionName="${restoreFilename}"
         fileIndicatorName="${forceRestartRestoreNeeded}"
         message="restore"
      fi

      CreateRAMdisk
      ValidateDistribution "restore" "${firmwareDistrubutionName}"
      if [ ${COMMAND_EXIT_STATUS} -ne 0 ]; then
         LogError "Failed to validate firmware distribution" 
         if [ -f "${firmwareDistrubutionName}" ]; then
            RunCommand "rm ${firmwareDistrubutionName}"
         fi
         RestartApplicationManager
      fi

      if [ ! -f "${fileIndicatorName}" ]; then
         command="touch ${fileIndicatorName}"
         RunCommand "$command" 
      fi

      if [ ! -f "${firmwareDistrubutionName}" ]; then
         LogError "No $message file [${firmwareDistrubutionName}] exists, were done, call service." 
         exit 1
      fi

      command="tar xf ${firmwareDistrubutionName} -C ${ramDiskMountPoint}"
      RunCommand "${command}" 

      ## Now lets look for library.tgz
      output=`find ${ramDiskMountPoint} -name library.tgz`
      if [ "${output}" != "" ]; then
         command="tar xf ${output} -C ${ramDiskMountPoint} ./AppManager/bin/*"
         RunCommand "$command" 
         if [ ${COMMAND_EXIT_STATUS} -ne 0 ]; then
            LogTrace "Does not have anything in ./AppManager/bin" 
         fi
      else
         ## This should never happen ValidateDistribution should have caught this
         LogFatal "Could not find $message image." 
         COMMAND_EXIT_STATUS=1
      fi
      ExtractApplicationManagerScripts

      if [ ${COMMAND_EXIT_STATUS} -ne 0 ]; then
         ## abandon the restore/recover
         command="rm ${fileIndicatorName}"
         RunCommand "$command"
      else
         if [ -f "${bladeDistributionDirectory}" ]; then
            RunCommand "rm ${bladeDistributionDirectory}/${bladeImage}"
            RunCommand "rm ${bladeDistributionDirectory}/${bladeDistribution}"
         fi
      fi
      UnmountRAMdisk
      RestartApplicationManager
   fi
}

#########################################
# We need to validate the distrubition
# Inputs: 
#     1. Are we dowing a upgrade, restore or recover
#     2. Firmware version name
#
#########################################
ValidateDistribution()
{
   local distType="$1"
   local firmwareDistrubutionName="$2"
   local command=""
   local newVersion=""
   local currentVersion=""
   local tmp1=""
   local tmp2=""
   local tmpfilename=""
   local new=""
   local old=""

   command="tar xzf /home/embedded/library/upload/library.tgz -C /var/tmp etc/MANIFEST.ca 2>/dev/null"
   eval "$command"
   if [ $? -eq 0 ]; then
      installCorrectiveAction="true"
   else
      LogTrace "Validating firmware distrubition ${firmwareDistrubutionName}" 
      if [ "${distType}" = "upgrade" ]; then
         ## We need to check if we are going to down grade
         command="tar xzf ${firmwareDistrubutionName} -C ${ramDiskMountPoint} ./S101CoreVersion.cpp"
         RunCommand "$command" 
         if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
            if [ -f "${ramDiskMountPoint}/S101CoreVersion.cpp" ]; then
               newVersion=`cat ${ramDiskMountPoint}/S101CoreVersion.cpp  | grep FirmwareImageVersion | grep 'const char' | awk -F\" '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
            fi
            if [ -f "${applicationHome}/S101CoreVersion.cpp" ]; then
               currentVersion=`cat ${applicationHome}/S101CoreVersion.cpp  | grep FirmwareImageVersion | grep 'const char' | awk -F\" '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
            fi

            LogTrace "Current version: ${currentVersion}, new version: ${newVersion}" 
            if [ "${newVersion}" = "" -o "${currentVersion}" = "" ]; then
               LogTrace "Could not validate firmware versions" 
               RunCommand "touch ${statusFirmwareCouldNotValidateVersion}"
               APPLICATION_STATUS=1
            else
               LogTrace "Checking for valid upgrade version" 
               old=`echo $currentVersion | cut -c1,2,3`
               new=`echo $newVersion | cut -c1,2,3`
               if [ ${old} -gt ${new} ]; then
                  LogTrace "The upgrade distribution is older than the current version." 
                  ## Validate we can downgrade filmware.
                  ## TODO: Do we need to do this?
                  ## if [ $new -lt $VALID_DOWN_GRADE_VERSION ]; then
                  ## LogTrace "The upgrade distribution is not valid, its too old, exit upgrade."
                  ## COMMAND_EXIT_STATUS=1
                  ## else
                  ## PrepareForDownGrade "downgrade"
                  ## fi
                  RunCommand "cp ${imagesDirectory}/downgradeImage /dev/fb0"
                  RunCommand "rm /tftpboot/*"
                  if [ -f "/etc/disableStatelessAutoConfiguration" ]; then
                     RunCommand "rm /etc/disableStatelessAutoConfiguration"
                  fi
                  PrepareForDownGrade "downgrade"
                  if [ ${new} -lt 410 ]; then
                     LogTrace "Downgrading to version ${new}, lets remove IPv6 static address if it exists."
                     RunCommand "/bin/sh /home/embedded/library/AppManager/bin/ipv6Manager.sh -c -s delete"
                  fi
               fi
            fi


            ## TODO: Do we need to check the personality
            old=`echo $currentVersion | cut -c4`
            new=`echo $newVersion | cut -c4`
         fi
      fi
   fi

   if [ "${distType}" = "restore" ]; then
      command="tar xf ${firmwareDistrubutionName} -C ${ramDiskMountPoint}"
      RunCommand "$command" 

      if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
         LogTrace "Retrive $message persistent files" 
         tmpfilename=`find ${ramDiskMountPoint} -name ${persistentTarballBaseName}.gz`
         if [ "${tmpfilename}" != "" ]; then
            command="gzip -d ${tmpfilename}"
            RunCommand "${command}"
         else
            LogInfo "No ${persistentTarballBaseName}.gz, restore must be pre 300G.GS0015"
            restoreIsPerVersion300G="true"
            ## RunCommand "touch ${statusFirmwareRestoreFileCorrupted}"
            ## COMMAND_EXIT_STATUS=1
         fi
      fi

      if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
         if [ "${restoreIsPerVersion300G}" = "false" ]; then
            tmpfilename=`echo ${tmpfilename} | sed 's/\.gz$//'`
            ## Lets check that the restore file is compatable, check serial numbers are the same
            command="tar xf ${tmpfilename} -C ${ramDiskMountPoint}"
            RunCommand "${command}" 
         fi
         tmp1=`cat /etc/serial.num`
         if [ -f "${ramDiskMountPoint}/etc/serial.num" ]; then
            tmp2=`cat ${ramDiskMountPoint}/etc/serial.num`
            if [ "${tmp1}" != "${tmp2}" ]; then
               LogError "Serial numbers do not match, current: $tmp1, distribution: $tmp2"
               RunCommand "touch ${statusFirmwareRestoreSerialNumberMismatch}"
               COMMAND_EXIT_STATUS=1
            fi
         else
            if [ -f "${ramDiskMountPoint}/SERIAL_NUM" ]; then
               tmp2=`cat ${ramDiskMountPoint}/SERIAL_NUM`
               if [ "${tmp1}" != "${tmp2}" ]; then
                  LogError "Serial numbers do not match, current: $tmp1, distribution: $tmp2"
                  RunCommand "touch ${statusFirmwareRestoreSerialNumberMismatch}"
                  COMMAND_EXIT_STATUS=1
               fi
            fi
         fi
      fi

      if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
         ## Lets report versions
         tmpfilename=`find ${ramDiskMountPoint} -name ${applicationDistributionBasename}`
         if [ "${tmpfilename}" != "" ]; then
            command="tar xf ${tmpfilename} -C ${ramDiskMountPoint} ./S101CoreVersion.cpp"
            RunCommand "$command"
            if [ -f "${ramDiskMountPoint}/S101CoreVersion.cpp" ]; then
               newVersion=`cat ${ramDiskMountPoint}/S101CoreVersion.cpp  | grep FirmwareImageVersion | grep 'const char' | awk -F\" '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
            fi
            if [ -f "${applicationHome}/S101CoreVersion.cpp" ]; then
               currentVersion=`cat ${applicationHome}/S101CoreVersion.cpp  | grep FirmwareImageVersion | grep 'const char' | awk -F\" '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
            fi
            LogTrace "Current version: ${currentVersion}, restore version: ${newVersion}" 

            LogTrace "Checking for valid restore version" 
            old=`echo $currentVersion | cut -c1,2,3`
            new=`echo $newVersion | cut -c1,2,3`
            ## Validate we can restore filmware.
            if [ $new -gt $old ]; then
               LogTrace "The restore distribution is not valid, restore version $new is more recent that current $old." 
               RunCommand "touch ${statusFirmwareRestoreVersionGreaterThanOld}"
               COMMAND_EXIT_STATUS=1
            else
               if [ $new -lt 400 ]; then
                  RunCommand "rm /tftpboot/*"
               fi
            fi
         fi
         if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
            PrepareForDownGrade
         fi
      fi
   fi

   if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
      ## Remve files from RAM Disk
      for file in `ls ${ramDiskMountPoint} | grep -v 'lost+found'`
      do
         command="rm -fr ${ramDiskMountPoint}/${file}"
         RunCommand "$command"
      done
   fi
}

#########################################
# Remove persistence, we doing a downgrade
# Inputs: 
#     1. Is this a downgrade
#
#########################################
PrepareForDownGrade()
{
   local downgrade="$1"
   
   ## If this is a downgrade lets backup current persistence
   if [ "${downgrade}" != "" ]; then
      if [ ! -d "${applicationHome}/downgradeBackup" ]; then
         RunCommand "mkdir ${applicationHome}/downgradeBackup"
      fi
      RunCommand "tar czf ${applicationHome}/downgradeBackup/downGradePersistenceBackup.tgz ${applicationHome}/CoreService/dat/[CcDPRSTmsuv]*"
   fi

   ## Remove Core persistence
   LogTrace "Remove current persistence" 
   RunCommand "rm ${applicationHome}/CoreService/dat/[CcDPRSTmsuv]*" 

   ## Reset /etc/extaccess.conf to original (default)
   LogTrace "Reset extaccess.conf" 
   RunCommand "cp /etc/extaccess.conf.orig /etc/extaccess.conf" 

   COMMAND_EXIT_STATUS=0
}

#########################################
# Extrace app manager files
# Inputs: 
#
#########################################
ExtractApplicationManagerScripts()
{
   local output=""
   local name=""
   local command=""

   if [ ${COMMAND_EXIT_STATUS} -eq 0 ]; then
      output=`find ${ramDiskMountPoint} -name AppMgr.exe`
      if [ "${output}" != "" ]; then
         LogTrace "Extrace AppMgr.exe" 
         command="mv ${output} ${applicationHome}/AppManager/bin"
         RunCommand "${command}" 
      fi

      name=`basename  ${appManagerFullpathName}`
      output=`find ${ramDiskMountPoint} -name ${name}`
      if [ "${output}" != "" ]; then
         LogTrace "Extrace ${name}" 
         command="mv ${output} ${applicationHome}/AppManager/bin"
         RunCommand "${command}" 
      fi

      name=`basename  ${firmwareManagerScriptName}`
      output=`find ${ramDiskMountPoint} -name ${name}`
      if [ "${output}" != "" ]; then
         LogTrace "Extrace ${name}" 
         command="mv ${output} ${applicationHome}/AppManager/bin"
         RunCommand "${command}" 
      fi

      name=`basename  ${applicationIncludeName}`
      output=`find ${ramDiskMountPoint} -name ${name}`
      if [ "${output}" != "" ]; then
         LogTrace "Extrace ${name}" 
         command="mv ${output} ${applicationHome}/AppManager/bin"
         RunCommand "${command}" 
      fi
      RunCommand "chmod 755 /home/embedded/library/AppManager/bin/*"
   fi
}

#########################################
# Force a restart of this script
# Inputs: 
#
#########################################
RestartApplicationManager()
{
   LogTrace " "
   LogTrace "Restart application manager ....."
   exec sh ${appManagerFullpathName} -s start -f &
   if [ $? != 0 ]; then
      LogError "Failed to exec sh ${appManagerFullpathName} -s start -f &" 
      exit 1
   fi
   exit 0
}

#########################################
# Setup library enviroment (personality, core location etc..)
# Inputs: 
#
#########################################
InitializeLibrarySettings()
{
   local personality=""
   local command=""
   local output=""
   LogTrace "Starting i500 Application Firmware, Initialize Environment ...................." 

   if [ -f "${applicationHome}/branding/LCS_Personality.ADIC" ]; then
      personality="adic"
   fi
   if [ -f "${applicationHome}/branding/LCS_Personality.DELL" ]; then
      personality="dell"
   fi
   if [ -f "${applicationHome}/branding/LCS_Personality.IBM" ]; then
      personality="ibm"
   fi
   if [ -f "${applicationHome}/branding/LCS_Personality.QUANTUM" ]; then
      personality="quantum"
   fi
   if [ -f "${applicationHome}/branding/LCS_Personality.TANDBERG" ]; then
      personality="tandberg"
   fi

   if [ "${personality}" != "" ]; then
      export ADICLIBRARY_BRAND="${personality}"
      LogTrace "Library Personality ${personality}" 
      LogTrace "Modify library /etc/profile" 
      output=`grep ADICLIBRARY_BRAND /etc/profile`
      if [ $? -eq 0 ]; then
         command="sed -i -r \"s/export ADICLIBRARY_BRAND=.*/export ADICLIBRARY_BRAND=${personality}/\" /etc/profile"
         RunCommand "${command}" 
      else
         echo "export ADICLIBRARY_BRAND=${personality}" >> /etc/profile
      fi
      ## Create htdocs link
      if [ ! -L "${applicationHome}/HTTPService/apache/htdocs" -a -d "${applicationHome}/HTTPService/apache" ]; then
         LogTrace "Create HTTP docs link" 
         if [ "${personality}" = "ibm" ]; then
            command="ln -s ${applicationHome}/HTTPService/apache/htdocs.ibm ${applicationHome}/HTTPService/apache/htdocs"
            RunCommand "${command}" 
         else
            command="ln -s ${applicationHome}/HTTPService/apache/htdocs.adic ${applicationHome}/HTTPService/apache/htdocs"
            RunCommand "${command}" 
         fi
      fi
   else
      LogError "No Personality Found" 
      ConsoleApplicationControl "start"
      exit
   fi

   if [ ! -d "${applicationHome}/AppManager/status" ]; then
      RunCommand "mkdir ${applicationHome}/AppManager/status"
   fi

   ## Set library path.
   export LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${applicationHome}/lib

   ## Set core file location and ulimit
   LogTrace "Set core file location and ulimit" 
   command="echo /home/embedded/library/AppManager/dat/core.%e.%p > /proc/sys/kernel/core_pattern"
   RunCommand "${command}"
   ulimit -c 20000

   ## Miscellaneous stuff here.
   ##

   ## Make IBM txt file writable.
   if [ "${personality}" = "ibm" ]; then
      if [ -f "${applicationHome}/HTTPService/apache/htdocs.ibm/session_timeout.txt" ]; then
         RunCommand "chmod 766 ${applicationHome}/HTTPService/apache/htdocs.ibm/session_timeout.txt"
      fi
   fi
}

#########################################
# Cleanup duplicate core files for each process
#
#########################################
CleanupCores()
{
   local file=""
   local uniqueFile=""
   local count=0
   local coreDirectory="/home/embedded/library/AppManager/dat"

   for uniqueFile in `ls -1 ${coreDirectory}/core* 2>/dev/null | cut -f0-2 -d'.' | awk '{print $NF}' | uniq`
   do
      count=0
      for file in `ls -1t ${uniqueFile}* | awk '{print $NF}'`
      do
         if [ $count -ne 0 ]; then
            LogTrace "Remove duplicate core file $file" 
            RunCommand "rm $file"
         fi
         count=`expr $count + 1`
      done
   done
}

#########################################
# Check log file size have not grown past there max size limit.
#
#########################################
checkLogfileSizes()
{
   local appManager="/var/log/appManager.log"
   local appManagerSize=524288
   local apache="${applicationHome}/HTTPService/apache/logs/error_log"
   local apacheSize=2097152
   local console="${applicationHome}/ConsoleService/dat/console.log"
   local consoleSize=524288
   local smis="${applicationHome}/SMI-SService/dat/owcimomd.log"
   local smisSize=65536
   local snmp="${applicationHome}/SNMPService/dat/snmpd.log"
   local snmpSize=65536
   local snmptrapd="${applicationHome}/SNMPService/dat/snmptrapd.log"
   local snmptrapdSize=24640
   local msalog="${applicationHome}/CoreService/dat/MediaSecurityEventLog.csv"
   local msalogSize=2097152
   ## NOTE: if you add files to the list above you need to add file name valiabe to the following list.
   local logfiles="appManager apache console smis snmp snmptrapd msalog"

   local maxFilesize=0
   local currentFileSize=0
   local maxFilesizePlusSome=0
   local file=0
   local padding=2048

   for logfile in `echo $logfiles`
   do
      eval file="\$${logfile}"
      eval maxFilesize="\$${logfile}Size"
      maxFilesizePlusSome=`expr $maxFilesize + $padding`

      if [ -f "${file}.bak" ]; then
         LogError "Found non zipped backup file $file, removing it."
         RunCommand "rm -f ${file}.bak"
      fi

      if [ -f "$file" ]; then
         currentFileSize=`ls -l ${file} | awk '{print $5}'`
         if [ $currentFileSize -gt ${maxFilesizePlusSome} ]; then
            LogError "Log file $file has size: ${currentFileSize}, allowed size: ${maxFilesize}, removing it."
            RunCommand "rm -f ${file}"
            RunCommand "rm -f ${file}.bak.gz"
         fi
      fi
   done
}

#########################################
# Clean up files we know should not be on filesystem.
#
#########################################
cleanDiskSpace()
{
   local appHome="/home/embedded/library"
   local fileToClean=""
   local directoriesToClean=""
   local file=""
   local directory=""
   local apacheBinDir=""
   local smisLibDir=""
   
   fileToClean="/root/CalibrationTool "
   fileToClean="${fileToClean} /usr/bin/gdb* "

   fileToClean="${fileToClean} ${appHome}/ConsoleService/bin/CalibrationTool.exe "
   fileToClean="${fileToClean} ${appHome}/UDS/drive_dumps/* "
   fileToClean="${fileToClean} ${appHome}/UDS/drive_firmware/*"

   apacheBinDir="/home/embedded/library/HTTPService/apache/bin"
   fileToClean="${fileToClean} "
   fileToClean="${fileToClean} ${apacheBinDir}/ab "
   fileToClean="${fileToClean} ${apacheBinDir}/httpsd "
   fileToClean="${fileToClean} ${apacheBinDir}/apxs "
   fileToClean="${fileToClean} ${apacheBinDir}/checkgid "
   fileToClean="${fileToClean} ${apacheBinDir}/gcache "
   fileToClean="${fileToClean} ${apacheBinDir}/setssl.sh "

   smisLibDir="/home/embedded/library/SMI-SService/lib"
   fileToClean="${fileToClean} ${smisLibDir}/requesthandlers/libowrequesthandlerbinary.so "
   fileToClean="${fileToClean} ${smisLibDir}/cmpiproviders/libcppindicationcommon.so "
   fileToClean="${fileToClean} ${smisLibDir}/cmpiproviders/libcmpifcbcomputersystem.so "

   LogTrace "Remove unwanted files"
   for file in `echo $fileToClean`
   do
      RunCommand "rm -f $file"
   done

   directoriesToClean="${appHome}/home /var/tmp/* ${appHome}/Extractor"

   LogTrace "Remove unwanted directories"
   for directory in `echo $directoriesToClean`
   do
      if [ -d "$directory" ]; then
         RunCommand "rm -fr $directory"
      fi
   done
}

#########################################
# Display usage message
# Inputs: 
#
#########################################
Usage()
{
   echo "Usage: $0 [ \"stop\" or \"start\" [\"force\"] ] | [ -c ] [ -d ] [ -e \"option\" ] [ -f ] ] [ -h ] [ -s < 'stop' | 'start' > ]  

   -c indicates we are moving to new app manager.
   -d flag to indicate we do not want to remove core files.
   -e engineering mode, which takes the following options
      -e f \"send core stdout/stderr to console\"
      -e i \"If core is running ignore it and try to start all other applications\"
   -f force a restart
   -h show this message
   -s change application state, options are \"start\" or \"stop\"

   ";
   exit 1
}

#################################################################################
####                               M A I N                                   ####
#################################################################################

trap "ProcessSignal SIGINT; trap 2" 2 

requestedState=""
forceStart=""
dontRemoveCoreFiles=""
sendCoreOutputToConsole=""
dontRestartCoreIfAlreadyRunning=""

if [ $# -gt 0 ]; then
   ## Parce arguments
   while getopts "cde:fhs:" opt
   do
      case "$opt" in
         c) converToNewApplicationManager="true"
         ;;
         d) dontRemoveCoreFiles="true"
         ;;
         e) 
         if [ "$OPTARG" = "f" ]; then
            sendCoreOutputToConsole="f"
         fi
         if [ "$OPTARG" = "i" ]; then
            dontRestartCoreIfAlreadyRunning="i"
         fi
         if [ "${OPTARG}" != "f" -a  "${OPTARG}" != "i" ]; then
            echo "ERROR: not a valid argument to -e option"
            Usage
         fi
         ;;
         f) forceStart="true"
         ;;
         h)Usage
         ;;
         s) requestedState="$OPTARG"
         if [ "${requestedState}" != "stop" -a  "${requestedState}" != "start" ]; then
            echo "ERROR: not a valid argument to -s option"
            Usage
         fi
         ;;
         *)Usage
         ;;
      esac
   done

   if [ "x$1" = "xstop" -o "x$1" = "xstart" ]; then
      requestedState="$1"
   fi
fi

if [ $# -eq 0 ]; then
   ## default if no arguments
   requestedState="start"
fi

## Check log file sizes on startup
if [ "X${requestedState}" = "Xstart" ]; then
   cleanDiskSpace
   checkLogfileSizes
fi

## Check for last update firmware file.
if [ ! -f "${lastFirmwareUpdateFile}" ]; then
   LogTrace "Did not find \"${lastFirmwareUpdateFile}\" lets create it."
   RunCommand "date '+%m %e %Y %H %M %S' > ${lastFirmwareUpdateFile}"
fi

InitLogFile "/var/log/appManager.log"
if [ "X$requestedState" = "Xstart" ]; then
   InitializeLibrarySettings
fi

## This should only be call when we move from old AppMgr.exe to new appManager.sh
if [ "${converToNewApplicationManager}" = "true" ]; then
   LogTrace "Moving to New Application Manager" 
   RunCommand "cp ${imagesDirectory}/upgradeImage /dev/fb0"
   ApplicationControl "status" "0"
   if [ ${APPLICATION_STATUS} -ne 0 ]; then
      ApplicationControl "stop"
   fi
   ## Let's assume that the upgrade will fail, this covers the case where someone powers down library during upgrade.
   RunCommand "touch ${forceRestartRecoveryNeeded}" 
   output=`sh ${firmwareManagerScriptName} -u "upgrade" -c`
   if [ $? -eq 0 ]; then
      LogTrace "Upgrade to new application manager completed successfully, restart applications" 
      RunCommand "rm ${forceRestartRecoveryNeeded}"
      RunCommand "touch ${upgradeCompletedCheckRestart}"
      RunCommand "date '+%m %e %Y %H %M %S' > ${lastFirmwareUpdateFile}"
      RestartApplicationManager
   else
      RunCommand "touch ${statusFirmwareUpgradeFailed}"
      if [ -f "${systemBackupFile}" ]; then
         RunCommand "touch ${forceRestartRecoveryNeeded}"
         RestartApplicationManager
      fi
   fi
fi

## Stop i500 application
if [ "$requestedState" = "stop" ]; then
   StopAppManager
   exit 0
fi

## Check if we need to restore from saved image
if [ -f "${restoreFilename}" ]; then
   if [ -f "${forceRestartRestoreNeeded}" ]; then
      command="rm ${forceRestartRestoreNeeded}"
      RunCommand "$command" 
      LogTrace "Restore firmware" 
      ## if the application software is running stop it.
      APPLICATION_STATUS=0
      ApplicationControl "status" "0"
      if [ ${APPLICATION_STATUS} -ne 0 ]; then
         ApplicationControl "stop"
      fi
      output=`sh ${firmwareManagerScriptName} -u "restore"`
      if [ $? -eq 0 ]; then
         ## We need to reboot the library, we also need to tell application when it restarts that an upgrade took place
         LogTrace "Restore of firmware completed successfully" 
         RunCommand "date '+%m %e %Y %H %M %S' > ${lastFirmwareUpdateFile}"
         if [ ! -f "${NO_REBOOT_BSP_NOT_INCLUDED}" ]; then
            LogTrace "Reboot the library" 
            command="/sbin/reboot"
            RunCommand "$command"
         else
            RunCommand "rm ${NO_REBOOT_BSP_NOT_INCLUDED}"
            LogTrace "No BSP, no need to reboot the library" 
            RestartApplicationManager
         fi
         exit 0
      else
         RunCommand "touch ${statusFirmwareRestoreFailed}"
         LogError "Failed to restore firmware, call service."
         ## TODO: We may need to display warning image on local op panel, do we exit, reboot or restart?
         LogTrace "Reboot the library"
         RunCommand "/sbin/reboot"
         ## RestartApplicationManager
      fi
   else
      LogTrace "Restore required" 
      InitalizeLibraryFirmware "restore"
      exit 0
   fi
fi

## Check if we need to recover form botched upgrade
if [ -f "${systemBackupFile}" ]; then
   if [ -f "${forceRestartRecoveryNeeded}" ]; then
      command="rm ${forceRestartRecoveryNeeded}"
      RunCommand "$command" 
      LogTrace "Recover firmware" 
      ## if the application software is running stop it.
      APPLICATION_STATUS=0
      ApplicationControl "status" "0"
      if [ ${APPLICATION_STATUS} -ne 0 ]; then
         ApplicationControl "stop"
      fi
      output=`sh ${firmwareManagerScriptName} -u "recover"`
      if [ $? -eq 0 ]; then
         ## We need to reboot the library, we also need to tell application when it restarts that an upgrade took place
         LogTrace "Recover firmware completed successfully" 
         if [ ! -f "${NO_REBOOT_BSP_NOT_INCLUDED}" ]; then
            LogTrace "Reboot the library" 
            command="/sbin/reboot"
            RunCommand "$command"
         else
            RunCommand "rm ${NO_REBOOT_BSP_NOT_INCLUDED}"
            LogTrace "No BSP, no need to reboot the library" 
            RestartApplicationManager
         fi
         exit 0
      else
         LogError "Failed to recover firmware, call service."
         RunCommand "touch ${statusFirmwareRecoverFailed}"
         ## TODO: We may need to display warning image on local op panel, do we exit, reboot or restart?
         LogTrace "Reboot the library"
         RunCommand "/sbin/reboot"
      fi
   fi
fi

## Check to see if we need to upgrade. 
if [ -f "${upgradeFirmwareFileName}" ]; then
   if [ -f "${forceRestartUpgradeNeeded}" ]; then
      command="rm ${forceRestartUpgradeNeeded}"
      RunCommand "$command" 
      LogTrace "Upgrade new firmware" 
      ## if the application software is running stop it.
      APPLICATION_STATUS=0
      ApplicationControl "status" "0"
      if [ ${APPLICATION_STATUS} -ne 0 ]; then
         ApplicationControl "stop"
      fi
      ## Let's assume that the upgrade will fail, this covers the case where someone powers down library during upgrade.
      RunCommand "touch ${forceRestartRecoveryNeeded}" 
      output=`sh ${firmwareManagerScriptName} -u "upgrade"`
      if [ $? -eq 0 ]; then
         ## We need to reboot the library, we also need to tell application when it restarts that an upgrade took place
         LogTrace "Upgrade new firmware completed successfully" 
         RunCommand "date '+%m %e %Y %H %M %S' > ${lastFirmwareUpdateFile}"
         RunCommand "rm ${forceRestartRecoveryNeeded}"
         command="touch ${upgradeCompletedCheckRestart}"
         RunCommand "$command"

         if [ ! -f "${NO_REBOOT_BSP_NOT_INCLUDED}" ]; then
            LogTrace "Lets do a reboot since BSP was update" 
            command="/sbin/reboot"
            RunCommand "$command"
         else
            LogTrace "BSP was not update, no need to reboot." 
            RunCommand "rm ${NO_REBOOT_BSP_NOT_INCLUDED}"
            RestartApplicationManager
         fi
         exit
      else
         if [ -f "${NO_REBOOT_BSP_NOT_INCLUDED}" ]; then
            RunCommand "rm ${NO_REBOOT_BSP_NOT_INCLUDED}"
         fi
         RunCommand "rm ${forceRestartRecoveryNeeded}"
         RunCommand "touch ${statusFirmwareUpgradeFailed}"
         LogTrace "Upgrade new firmware failed, lets try to recover" 
         if [ -f "${applicationHome}/upload/library.tgz" ]; then
            RunnCommand "rm ${applicationHome}/upload/library.tgz"
         fi
         InitalizeLibraryFirmware "recover"
      fi
   else
      LogTrace "Upgrade required" 
      InitalizeLibraryFirmware "upgrade"
      exit 0
   fi
fi

applicationStartRetries=0
numberOfStartRetries=3

## Start i500 application
if [ "$requestedState" = "start" ]; then
   StartAppManager "$forceStart"
   ApplicationControl "start"
   ## Check if the application started
   APPLICATION_STATUS=0
   ApplicationControl "status" "1"
   if [ ${APPLICATION_STATUS} -ne 1 ]; then
      LogError "Failed to start all the applications, lets do retries"
      while [ ${applicationStartRetries} -lt ${numberOfStartRetries} ]
      do
         ApplicationControl "stop"
         ApplicationControl "start"
         ## Check if the application started
         APPLICATION_STATUS=0
         ApplicationControl "status" "1"
         if [ ${APPLICATION_STATUS} -eq 1 ]; then
            break;
         fi
         applicationStartRetries=`expr ${applicationStartRetries} + 1`
         CleanupCores
      done
      if [ ${APPLICATION_STATUS} -ne 1 ]; then
         LogError "Failed to start all applications after ${applicationStartRetries} retries"

         LogTrace "Lets try to start Core, Console and HTTP"
         ApplicationControl "stop"
         ## Lets try to start Core, Console and HTTP
         CoreApplicationControl "start"
         CoreApplicationControl "status" 
         if [ ${APPLICATION_STATUS} -ne 1 ]; then
            LogFatal "Failed to start core, were done."
         else
            ConsoleApplicationControl "start"
            ConsoleApplicationControl "status"
            if [ ${APPLICATION_STATUS} -ne 1 ]; then
               LogFatal "Failed to start console, were done."
            else
               HTTPapplicationControl "start"
               HTTPapplicationControl "status"
               if [ ${APPLICATION_STATUS} -ne 1 ]; then
                  LogFatal "Failed to start HTTP, were done."
               fi
            fi
         fi
         ## Did we just do an upgrade
         if [ -f "${upgradeCompletedCheckRestart}" -a ${APPLICATION_STATUS} -ne 1 ]; then
            ## Lets try to recover back to old version
            RunCommand "rm ${upgradeCompletedCheckRestart}"
            InitalizeLibraryFirmware "recover"
         else
            if [ ${APPLICATION_STATUS} -ne 1 ]; then
               LogFatal "Could not start the application, call service."
               exit 1
            fi
         fi
      else
         ## Remove upgrade success indicator file.
         if [ -f "${upgradeCompletedCheckRestart}" ]; then
            RunCommand "rm ${upgradeCompletedCheckRestart}"
         fi
         if [ -f "${systemBackupFile}" ]; then
            LogTrace "Removing system backup file ${systemBackupFile}"
            RunCommand "rm ${systemBackupFile}"
         fi
      fi
   else
      ## Remove upgrade success indicator file.
      if [ -f "${upgradeCompletedCheckRestart}" ]; then
         RunCommand "rm ${upgradeCompletedCheckRestart}"
      fi
      if [ -f "${systemBackupFile}" ]; then
         LogTrace "Removing system backup file ${systemBackupFile}"
         RunCommand "rm ${systemBackupFile}"
      fi
   fi
fi

coreStoppedRetries=0
consoleStoppedRetries=0
httpStoppedRetries=0
snmpStoppedRetries=0
smisStoppedRetries=0
snmptrapdStoppedRetries=0
sleepTime=5

#### Main Process Loop ####
LogTrace "Entering main application loop" 
RunCommand "/etc/init.d/cron stop"
while [ 1 ]; do
   sleep ${sleepTime}
   ## Check Core Application status
   CoreApplicationControl "status" 
   if [ ${APPLICATION_STATUS} -ne 1 ]; then
      LogError "Core Application is not running"
      LogTrace "Stop All other applications and try to restart" 
      ## Now lets check the status
      APPLICATION_STATUS=0
      while [ ${APPLICATION_STATUS} -ne 1 -a ${coreStoppedRetries} -ne ${numberOfStartRetries} ]
      do
         ApplicationControl "stop"
         LogInfo ">>>>>>>>>>> RESTART RETRIES: ${coreStoppedRetries}"
         ApplicationControl "start"
         CoreApplicationControl "status"
         let coreStoppedRetries=`expr ${coreStoppedRetries} + 1`
         CleanupCores
      done
      if [ ${APPLICATION_STATUS} -eq 1 ]; then 
         ConsoleApplicationControl "status"
         if [ ${APPLICATION_STATUS} -eq 1 ]; then 
            HTTPapplicationControl "status"
            if [ ${APPLICATION_STATUS} -eq 1 ]; then 
               ApplicationControl "status"
               if [ ${APPLICATION_STATUS} -eq 1 ]; then 
                  LogTrace "Complete restart of all applications" 
               else
                  LogError "Some of the applications did not start." 
               fi
            else
               LogTrace "Failed to restart of HTTP, exit application." 
               exit 1
            fi
         else
            LogTrace "Failed to restart of Console, exit application." 
            exit 1
         fi
         coreStoppedRetries=0
      else
         LogFatal "Could not restart core." 
         exit 1
      fi
   else
      ## Check the rest of the applications
      ConsoleApplicationControl "status"
      if [ ${APPLICATION_STATUS} -ne 1 -a ${consoleStoppedRetries} -ne ${numberOfStartRetries} ]; then
         RunCommand "/etc/init.d/ntpd restart"
         LogError "Console not running need to restart, retries ${consoleStoppedRetries}"
         ConsoleApplicationControl "start"
         let consoleStoppedRetries=`expr ${consoleStoppedRetries} + 1`
      else
         if [ ${APPLICATION_STATUS} -eq 1 ]; then
            consoleStoppedRetries=0
         fi
      fi
      HTTPapplicationControl "status"
      if [ ${APPLICATION_STATUS} -ne 1 -a ${httpStoppedRetries} -ne ${numberOfStartRetries} ]; then
         RunCommand "/etc/init.d/ntpd restart"
         LogError "HTTP not running need to restart, retries ${httpStoppedRetries}"
         HTTPapplicationControl "start"
         let httpStoppedRetries=`expr ${httpStoppedRetries} + 1`
      else
         if [ ${APPLICATION_STATUS} -eq 1 ]; then
            httpStoppedRetries=0
         fi
      fi
      SNMPapplicationControl "status"
      if [ ${APPLICATION_STATUS} -ne 1 -a ${snmpStoppedRetries} -ne ${numberOfStartRetries} ]; then
         RunCommand "/etc/init.d/ntpd restart"
         LogError "SNMP not running need to restart, retries ${snmpStoppedRetries}"
         SNMPapplicationControl "start"
         let snmpStoppedRetries=`expr ${snmpStoppedRetries} + 1`
      else
         if [ ${APPLICATION_STATUS} -eq 1 ]; then
            snmpStoppedRetries=0
         fi
      fi
      if [ -f "$SMISenabled" ]; then
         SMISapplicationControl "status"
         if [ ${APPLICATION_STATUS} -ne 1 -a ${smisStoppedRetries} -ne ${numberOfStartRetries} ]; then
            RunCommand "/etc/init.d/ntpd restart"
            LogError "SMI-S not running need to restart, retries ${smisStoppedRetries}"
            SMISapplicationControl "start"
            let smisStoppedRetries=`expr ${smisStoppedRetries} + 1`
         else
            if [ ${APPLICATION_STATUS} -eq 1 ]; then
               smisStoppedRetries=0
            fi
         fi
      fi
      SNMPTrapApplicationControl "status"
      if [ ${APPLICATION_STATUS} -ne 1 -a ${snmptrapdStoppedRetries} -ne ${numberOfStartRetries} ]; then
         RunCommand "/etc/init.d/ntpd restart"
         LogError "SNMP Trap daemon not running need to restart, retries ${snmptrapdStoppedRetries}"
         SNMPTrapApplicationControl "start"
         let snmptrapdStoppedRetries=`expr ${snmptrapdStoppedRetries} + 1`
      else
         if [ ${APPLICATION_STATUS} -eq 1 ]; then
            snmptrapdStoppedRetries=0
         fi
      fi
   fi

   if [ "${dontRemoveCoreFiles}" != "true" ]; then
      CleanupCores
   fi

   if [ -f "${applicationHome}/upload/${applicationDistributionBasename}" ]; then
      LogTrace "Found new library firmware" 
      RunCommand "/usr/bin/bmmcommands -D" ## Power down blades
      ApplicationControl "stop"
      InitalizeLibraryFirmware "upgrade"
   fi

   if [ -f "${restoreFilename}" ]; then
      LogTrace "Found library restore firmware file" 
      RunCommand "/usr/bin/bmmcommands -D" ## Power down blades
      ApplicationControl "stop"
      InitalizeLibraryFirmware "restore"
   fi
done
