#!/bin/sh
#
# $Id: bin/captureSnapshot.sh 1.19 2007/11/05 09:10:03MST Bob Ackerman (backerman) dev  $
##

## Path to application home
applicationHome="/home/embedded/library"
applicationIncludeName="${applicationHome}/AppManager/bin/include.sh"

. ${applicationIncludeName}

systemInfoFilename="/var/log/sysinfo.txt"



#########################################
# Create the temp directories needed to build the distribution list of files.
#
#########################################
CreateTmpDirectories()
{
   local appHomeDir="home/embedded/library"
   local directories=""
   local directory=""

   directories="${appHomeDir}/AppManager/dat ${appHomeDir}/ConsoleService/dat ${appHomeDir}/CoreService/dat ${appHomeDir}/HTTPService/apache/logs "
   directories="${directories} ${appHomeDir}/SMI-SService/dat ${appHomeDir}/SMI-SService/etc/openwbem ${appHomeDir}/SMI-SService/repository "
   directories="${directories} ${appHomeDir}/SNMPService/dat ${appHomeDir}/SNMPService/share/snmp ${appHomeDir}/SNMPService/var/net-snmp ${appHomeDir}/branding"
   directories="${directories} ${appHomeDir}/SNMPService/share/snmp/mibs etc var/log etc/network tmp ${appHomeDir}/AppManager/bin"

   for directory in `echo ${directories}`
   do
      RunCommand "mkdir -p ${tmpSnapshotDirector}/${directory}"
   done
}

#########################################
# Create the S101core persistence file list.
#
#########################################
ProcessCorePersistence()
{
   local coreFiles=""
   local file=""
   local tmp=""
   local rasAgentFiles=""
   local datDirectory="/home/embedded/library/CoreService/dat"

   if [ "$appendToSnapshotFile" != "true" ]; then
      coreFiles="RASLog.Hot RASLog.Warm RASLog.Cool Discovery.Associations Discovery.Associations.ext Discovery.Elements Discovery.Elements.ext "
      coreFiles="${coreFiles} Tickets Tickets.ext Snapshot.RASAgents Snapshot.RASAgents.ext"

      for file in `echo ${coreFiles}`
      do
         if [ -f "${datDirectory}/${file}" ]; then
            RunCommand "gzip -c ${datDirectory}/${file} > ${tmpSnapshotDirector}/${file}.gz"
         else
            LogTrace "snapshot file [${datDirectory}/${file}] does not exist" 
         fi
      done
   fi

   coreFiles="uds_sn_spoofing uds_sd_servers uds_sd_servers.ext bmsbids  PMCPersistConfig PMCPersistConfig.ext RASEmailAccts "
   coreFiles="${coreFiles} RASEmailAccts.ext securityFile securityFile.ext securityConfFile securityConfFile.ext sl_LibInfo sl_LibInfo.ext "
   coreFiles="${coreFiles} sl_LibSettings sl_LibSettings.ext sl_LibCleanSlots sl_LibCleanSlots.ext LicKeys LicKeys.ext service service.ext "
   coreFiles="${coreFiles} DevMaps DevMaps.ext CmdPaths CmdPaths.ext PMCPersistModules PMCPersistModules.ext PMCPersistMagDrv PMCPersistMagDrv.ext "
   coreFiles="${coreFiles} PMCCellPersistance PMCCellPersistance.ext PMCCellMedia PMCCellMedia.ext PersistanceLMCS PersistanceLMCS.ext "
   coreFiles="${coreFiles} mc_mediumchanger mc_mediumchanger.ext PersistanceSMCDS PersistanceSMCDS.ext SNMPConfig SNMPConfig.ext"
   coreFiles="${coreFiles} IVTLog.txt IVTStatus.txt uds_ekm uds_ekm.ext akm_servers akm_servers.ext ekm_data ekm_data.ext QkmImport.log"

   rasAgentFiles=`ls /home/embedded/library/CoreService/dat/RASAgent.*.ActiveStrategies*`
   for file in `echo ${rasAgentFiles}`
   do
      tmp=`basename ${file}`
      coreFiles="${coreFiles} ${tmp}"
   done

   for file in `echo ${coreFiles}`
   do
      if [ -f "${datDirectory}/${file}" ]; then
         RunCommand "gzip -c ${datDirectory}/${file} > ${tmpSnapshotDirector}/${datDirectory}/${file}.gz"
      else
         LogTrace "snapshot file [${datDirectory}/${file}] does not exist" 
      fi
   done
}

#########################################
# Build the Applications persistence file list. (Console, HTTP, SNMP etc...)
#
#########################################
ProcessApplicationFiles()
{
   local snmpDir="/home/embedded/library/SNMPService"
   local smisDir="/home/embedded/library/SMI-SService"

   local snapFiles=""
   local file=""

   snapFiles="${snmpDir}/dat/snmpd.log ${snmpDir}/dat/snmpd.log.bak ${snmpDir}/dat/snmptrapd.log ${snmpDir}/share/snmp/snmpd.conf "
   snapFiles="${snapFiles} ${snmpDir}/var/net-snmp/snmpd.conf ${snmpDir}/share/snmp/mibs/ADIC_TAPE_LIBRARY_MIB.mib "
   snapFiles="${snapFiles} ${smisDir}/dat/owcimomd.log ${smisDir}/etc/openwbem/hostkey+cert.pem ${smisDir}/etc/openwbem/openwbem.conf "
   snapFiles="${snapFiles} ${smisDir}/etc/openwbem/slp.conf ${smisDir}/repository/classassociation.dat ${smisDir}/repository/instances.dat " 
   snapFiles="${snapFiles} ${smisDir}/repository/instassociation.dat ${smisDir}/repository/namespaces.dat ${smisDir}/repository/schema.dat "
   snapFiles="${snapFiles} ${smisDir}/repository/classassociation.ndx ${smisDir}/repository/instances.ndx ${smisDir}/repository/instassociation.ndx "
   snapFiles="${snapFiles} ${smisDir}/repository/namespaces.ndx ${smisDir}/repository/schema.ndx ${smisDir}/repository/instances.lock "
   snapFiles="${snapFiles} ${smisDir}/repository/namespaces.lock ${smisDir}/repository/schema.lock"
   snapFiles="${snapFiles} /home/embedded/library/ConsoleService/dat/console.log "
   snapFiles="${snapFiles} /home/embedded/library/HTTPService/apache/logs/error_log /home/embedded/library/HTTPService/apache/logs/error_log.bak"

   for file in `echo ${snapFiles}`
   do
      if [ -f "${file}" ]; then
         RunCommand "gzip -c ${file} > ${tmpSnapshotDirector}${file}.gz"
         if [ -f "${file}.bak.gz" ]; then
            RunCommand "cp ${file}.bak.gz ${tmpSnapshotDirector}${file}.bak.gz"
         fi
      else
         LogTrace "snapshot file [${file}] does not exist" 
      fi
   done
}

#########################################
# Get System information
#
#########################################
generateSystemInfo()
{
   echo "Collecting System Information....." > ${systemInfoFilename}
   echo "Current time: " `date` >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting Uptime info..... uptime" >> ${systemInfoFilename}
   uptime >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting process info..... ps -eflL" >> ${systemInfoFilename}
   ps -eflL >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting Ethernet Interface Info.... ifconfig" >> ${systemInfoFilename}
   ifconfig >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting network information..... netstat -a" >> ${systemInfoFilename}
   netstat -a >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting filesystem information..... df -k" >> ${systemInfoFilename}
   df -k >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting interfaceEth2 Info..... cat /etc/network/interfaceEth2" >> ${systemInfoFilename}
   cat /etc/network/interfaceEth2 >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting Last Firmware Update..... cat /etc/lastFirmwareUpdate" >> ${systemInfoFilename}
   cat /etc/lastFirmwareUpdate >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting Memory Usage..... free" >> ${systemInfoFilename}
   free >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting Msg Queue Usage..... cat /proc/sysvipc/msg" >> ${systemInfoFilename}
   cat /proc/sysvipc/msg >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting route info..... ip route list" >> ${systemInfoFilename}
   echo "Main routing table" >> ${systemInfoFilename}
   ip route list         >> ${systemInfoFilename}
   echo "Table 1 (eth0)" >> ${systemInfoFilename}
   ip route list table 1 >> ${systemInfoFilename}
   echo "Table 2 (eth1)" >> ${systemInfoFilename}
   ip route list table 2 >> ${systemInfoFilename}
   echo "Table 3 (eth3)" >> ${systemInfoFilename}
   ip route list table 3 >> ${systemInfoFilename}
   echo "Routing rules " >> ${systemInfoFilename}
   ip rule list          >> ${systemInfoFilename}
   echo "" >> ${systemInfoFilename}
   
   echo "Collecting MFG info..... /usr/bin/dumpmfg" >> ${systemInfoFilename}
   /usr/bin/dumpmfg >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting Directory Info..... ls -lR /home/embedded/library /tmp /root /etc" >> ${systemInfoFilename}
   ls -lR /home/embedded/library /tmp /root /etc >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}

   echo "Collecting Open File Info..... lsof" >> ${systemInfoFilename}
   lsof >> ${systemInfoFilename} 2>&1
   echo "" >> ${systemInfoFilename}
}

#########################################
# Create a list of log files and configuration files
#
#########################################
ProcessLogFiles()
{
   local logDir="/var/log"
   local snapFiles=""
   local output=""

   generateSystemInfo
   if [ -f "/proc/can_sja" ]; then
      RunCommand "cat /proc/can_sja > $logDir/canstatus.txt"
   fi
   
   snapFiles="$logDir/messages $logDir/messages.0 $logDir/AppMgrFDM.log $logDir/AppMgrFDM.log.0 $logDir/canstatus.txt $logDir/sysinfo.txt "
   snapFiles="${snapFiles} $logDir/bmm.log $logDir/dmesg"
   # BSP files
   output=`ls /var/log/BSP*`
   if [ $? = 0 ]; then
      snapFiles="${snapFiles} ${output}"
   fi

   for file in `echo ${snapFiles}`
   do
      if [ -f "${file}" ]; then
         RunCommand "gzip -c ${file} > ${tmpSnapshotDirector}${file}.gz"
         if [ -f "${file}.bak.gz" ]; then
            RunCommand "cp ${file}.bak.gz ${tmpSnapshotDirector}${file}.bak.gz"
         fi
      else
         LogTrace "snapshot file [${file}] does not exist" 
      fi
   done
}

#########################################
# Build a list of misc files we need to capture. Good place to add new files.
#
#########################################
ProcessSystemFiles()
{
   local output=""
   local snapFiles=""

   ## Process core files
   output=`ls /home/embedded/library/AppManager/dat/core* 2>/dev/null`
   if [ $? = 0 ]; then
      RunCommand "tar -Sczf ${tmpSnapshotDirector}/cores.tgz $output"
   else
      LogTrace "No core files" 
   fi

   snapFiles="/etc/extaccess.conf /var/log/appManager.log /etc/serial.num /etc/network/intefaceEth2 ${lastFirmwareUpdateFile}"
   snapFiles="${snapFiles} /etc/hosts /etc/nsswitch.conf /etc/resolv.conf /home/embedded/library/cts.log /home/embedded/library/INSTALLED.CA"
   snapFiles="${snapFiles} /etc/MANIFEST.ca /home/embedded/library/AppManager/bin/fixit.sh /etc/ntp.conf"

   for file in `echo ${snapFiles}`
   do
      if [ -f "${file}" ]; then
         RunCommand "gzip -c ${file} > ${tmpSnapshotDirector}${file}.gz"
      else
         LogTrace "snapshot file [${file}] does not exist" 
      fi
   done
   RunCommand "cp /home/embedded/library/branding/LCS_Personality.* ${tmpSnapshotDirector}/home/embedded/library/branding"
}

#########################################
# Process blade files
#
#########################################
ProcessBladeFiles()
{
   local file=""
   local files=""
   local dir=""
   if [ -f "/home/embedded/library/bladeSnap.sh" ]; then
      RunCommand "sh /home/embedded/library/bladeSnap.sh"
   fi
   files=`find /var/log/adic -name snap`
   for file in `echo ${files}`
   do
      dir=`echo $file | sed 's/snap//'`
      RunCommand "mkdir -p ${tmpSnapshotDirector}${dir}"
      RunCommand "gzip -c ${file} > ${tmpSnapshotDirector}${file}.gz"
      RunCommand "rm ${file}"
   done
   files=`find /tmp -name evl_*`
   for file in `echo ${files}`
   do
      RunCommand "gzip -c ${file} > ${tmpSnapshotDirector}${file}.gz"
      RunCommand "rm ${file}"
   done
   files=`find /var/log/adic -name cflog`
   for file in `echo ${files}`
   do
      dir=`echo $file | sed 's/cflog//'`
      RunCommand "mkdir -p ${tmpSnapshotDirector}${dir}"
      RunCommand "gzip -c ${file} > ${tmpSnapshotDirector}${file}.gz"
      RunCommand "rm ${file}"
   done
}

#########################################
# Display usage message
# Inputs: 
#
#########################################
Usage()
{
   echo "Usage: $0 [ -a ] [ -f \"filename\" ] [ -h ]

   -a append to existing snapshot file.
   -f the name of the snapshot file name. If this is not used the default filename is /tmp/snapshot.XXXXXX
   -h show this message

   ";
   exit 0
}

#################################################################################
####                               M A I N                                   ####
#################################################################################
snapshot=""
appendToSnapshotFile="false"

if [ $# -gt 0 ]; then
   ## Parce arguments
   while getopts "af:h" opt
   do
      case "$opt" in
         a)appendToSnapshotFile="true"
         ;;
         h)Usage
         ;;
         f) snapshot="$OPTARG"
         ;;
         *)Usage
         ;;
      esac
   done
fi

LogTrace "Create snapshot ${snapshot}" 

if [ "${snapshot}" = "" ]; then
   snapshot=`mktemp /tmp/snapshot.XXXXXX`
   LogTrace "No snapshot file specified, create default [$snapshot]" 
fi

tmpSnapshotDirector=`mktemp -d /tmp/$$.snapshotTmpDirectory.XXXXXX`
CreateTmpDirectories
if [ ! -d "${tmpSnapshotDirector}" ]; then
   LogError "Could not create directory ${tmpSnapshotDirector}" 
   exit 1
fi

ProcessCorePersistence
ProcessApplicationFiles
ProcessLogFiles
ProcessSystemFiles
ProcessBladeFiles

whereAmI=`pwd`
cd ${tmpSnapshotDirector}
files=`find . -type f`
if [ "$appendToSnapshotFile" = "true" ]; then
   RunCommand "tar rvf ${snapshot} $files" "1"
else
   RunCommand "tar cvf ${snapshot} $files" "1"
fi
LogTrace "Created snapshot ${snapshot}" 
cd $whereAmI

RunCommand "rm -rf ${tmpSnapshotDirector}"
RunCommand "rm -f ${applicationHome}/AppManager/dat/core*"
exit 0
