#!/bin/sh
#
# $Id: bin/changePersonality.sh 1.3 2007/03/09 07:23:19MST Jim Boyd (jboyd) dev  $
##

## Path to application home
applicationHome="/home/embedded/library"
## Source include file.
. ${applicationHome}/AppManager/bin/include.sh

#########################################
# Display usage message
# Inputs: 
#
#########################################
Usage()
{
   echo "Usage: $0 [ -h ] < -p \"new personality\" [ -r ] > 

   -h show this message
   -p where 'new personaility' is one of the following
      \"quantum\"
      \"adic\"
      \"ibm\"
      \"dell\"
      \"tandberg\"
   -r restart application
   ";
   exit 1
}

#########################################
# Create link to correct HTTP docs
# Inputs: 
#
#########################################
CreateHTTPdocsLink()
{
   local command=""
   LogTrace "Create HTTP docs link" 
   if [ "${branding}" = "ibm" ]; then
      command="ln -s ${applicationHome}/HTTPService/apache/htdocs.ibm ${applicationHome}/HTTPService/apache/htdocs"
      RunCommand "${command}" 
   else
      command="ln -s ${applicationHome}/HTTPService/apache/htdocs.adic ${applicationHome}/HTTPService/apache/htdocs"
      RunCommand "${command}" 
   fi
}

KillCoreApplication()
{
   local output=""
   local applicationStatus=0
   local timeout=0

   RunCommand "touch ${userChangeingLibraryPersonality}"

   LogTrace "Stopping Core application" 
   output=`ps -o pid --no-heading -C S101Core.exe | grep -v defunct`
   if [ $? -eq 0 ]; then
      set - $output
      RunCommand "kill -15 $1"
   else
      LogTrace "S101Core.exe is not running" 
   fi

   # Give it one minute to die, then kill it with signal 9
   while [ 1 ]; do
      sleep 1
      output=`ps -o pid --no-heading -C S101Core.exe`
      if [ $? -eq 0 ]; then
         applicationStatus=1
      else
         applicationStatus=0
      fi

      if [ ${applicationStatus} -eq 1 ]; then
         timeout=`expr $timeout + 1`
         if [ ${timeout} -gt 60 ]; then
            LogTrace "Need to terminate Core with signal 9" 
            output=`ps -o pid --no-heading -C S101Core.exe | grep -v defunct`
            if [ $? -eq 0 ]; then
               set - $output
               RunCommand "kill -9 $1"
            else
               LogTrace "S101Core.exe is not running" 
            fi

            break
         fi
      else
         break
      fi
   done

}

#################################################################################
####                               M A I N                                   ####
#################################################################################

personality=""
branding=""
restartApplication=""

if [ $# -eq 0 ]; then
   Usage
else
   while getopts "hp:r" opt
   do
      case "$opt" in
         h) Usage
         ;;
         p) branding="${OPTARG}"
         ;;
         r) restartApplication="true"
         ;;
         *) Usage
         ;;
      esac
   done
fi

if [ "${branding}" = "adic" ]; then
   personality="ADIC"
elif [ "${branding}" = "quantum" ]; then
   personality="QUANTUM"
elif [ "${branding}" = "ibm" ]; then
   personality="IBM"
elif [ "${branding}" = "dell" ]; then
   personality="DELL"
elif [ "${branding}" = "tandberg" ]; then
   personality="TANDBERG"
else
   Usage
fi

if [ -f "${applicationHome}/branding/LCS_Personality.${personality}" ]; then
   LogTrace "Personality is already \"${branding}\""
   exit 1
fi

if [ ! -d "${applicationHome}/branding" ]; then
   RunCommand "mkdir ${applicationHome}/branding"
fi

RunCommand "rm ${applicationHome}/branding/LCS_Personality.*"
RunCommand "touch ${applicationHome}/branding/LCS_Personality.${personality}"

LogTrace "Change Library Personaility to \"${personality}\""
## Edit /etc/profile
output=`grep ADICLIBRARY_BRAND /etc/profile`
if [ $? -eq 0 ]; then
   command="sed -i -r \"s/export ADICLIBRARY_BRAND=.*/export ADICLIBRARY_BRAND=${branding}/\" /etc/profile"
   RunCommand "${command}" 
else
   echo "export ADICLIBRARY_BRAND=${branding}" >> /etc/profile
fi

## Create htdocs link
if [ ! -L "${applicationHome}/HTTPService/apache/htdocs" -a -d "${applicationHome}/HTTPService/apache" ]; then
   CreateHTTPdocsLink
else
   if [ -L "${applicationHome}/HTTPService/apache/htdocs" ]; then
      RunCommand "rm ${applicationHome}/HTTPService/apache/htdocs"
      CreateHTTPdocsLink
   else
      CreateHTTPdocsLink
   fi
fi

if [ "${restartApplication}" = "true" ]; then
   LogTrace "Restart appManager from $0"
   RunCommand "chmod 755 ${applicationHome}/AppManager/bin/appManager.sh"
   RunCommand "/usr/bin/bmmcommands -D"
   KillCoreApplication
fi
LogTrace "Successfully changed personality"
exit 0
