#!/bin/sh
#
# $Id: bin/fmrManager.sh 1.18 2007/11/13 13:36:43MST Bob Ackerman (backerman) dev  $
##

applicationHome="/home/embedded/library"
## DEBUG applicationHome="/home/jboyd/shaggy/Code/Applications/Scalar101/S101Components"
distributionVersion=400

## Source include file.
. ${applicationHome}/AppManager/bin/include.sh

## DEBUG applicationHome="/home/embedded/library"

applicationSerialNumberFile="/etc/serial.num"

changeLibraryPersonality=""

#########################################
# Get the version of distribution
# Inputs: 
#     1. Distribution location
#########################################
GetDistributionVersion()
{
   local distribution="$1"
   local version=0

   LogTrace "Get distribution version" 
   if [ -f "/tmp/S101CoreVersion.cpp" ]; then
      RunCommand "rm /tmp/S101CoreVersion.cpp"
   fi
   RunCommand "tar xzf ${distribution} -C /tmp ./S101CoreVersion.cpp"
   version=`cat /tmp/S101CoreVersion.cpp  | grep FirmwareImageVersion | grep 'const char' | awk -F\" '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
   if [ "${version}" != "" ]; then
      LogTrace "Distribution $distribution has version $version" 
      distributionVersion=`echo ${version} | cut -c1,2,3`
   else
      LogError "Could not get distribution version." 
   fi
}

#########################################
# Extract bzip files
# Inputs: 
#     1. Message to log
#     2. bzip file to process
#
#########################################
ExtractBZipDistributionFile()
{
   local message="$1"
   local file="$2"
   local tmp=""
   local command=""

   LogTrace "${message}" 
   command="tar xvzf ${applicationDistributionName}  -C ${ramDiskMountPoint} ${file}"
   RunCommand "$command" "1"
   tmp=`echo $file | sed 's/^\.//'`
   if [ ${distributionVersion} -ge 320 ]; then
      LogTrace "Unzip $tmp" 
      echo $file | egrep -e '.*\.tar.bz2'  | grep -v 'ConsoleService' | grep -v 'ExternalLibs'
      if [ $? = 0 ]; then
	  RunCommand "tar -xjf ${ramDiskMountPoint}${tmp}  -C ${ramDiskMountPoint}" "1"
	  RunCommand "rm -f ${ramDiskMountPoint}${tmp}"
      else
	  RunCommand "bzip2 -df ${ramDiskMountPoint}${tmp}" "1"
	  ProccessUnzippedFile "${ramDiskMountPoint}${tmp}"
      fi
   else
      ProccessUnzippedFile "${ramDiskMountPoint}${tmp}"
   fi
}

#########################################
# Call the extract methods for all the components. After there extraced to RAM
# Disk, then unzip them and copy the contents of RAM Disk to
# /home/embedded/library. Then clear RAM Disk of all the files.
# Inputs: None
#
#########################################
ExtractZippedFiles()
{
   local command=""
   local output=""
   local file=""
   if [ ${distributionVersion} -ge 320 ]; then
      LogTrace "Extracting zip files." 
      ExtractBZipDistributionFile "Extract HTTP." "./HTTPService/HTTPService.tar.bz2"
      ExtractBZipDistributionFile "Extract Core." "./CoreService/bin/S101Core.exe.bz2"
      ExtractBZipDistributionFile "Extract Console." "./ConsoleService/ConsoleService.tar.bz2"
      ExtractBZipDistributionFile "Extract SNMP." "./SNMPService/SNMPService.tar.bz2"
      ## Lets copy repository (for indications) and blow away
      ## existing SMI-S directory
      RunCommand "cp -rf ${applicationHome}/SMI-SService/repository ${ramDiskMountPoint}/repository"
      RunCommand "rm -fr ${applicationHome}/SMI-SService/*"
      ExtractBZipDistributionFile "Extract SMIS." "./SMI-SService/SMISService.tar.bz2"
      # Port Indications to new repository
      LogTrace "Porting SMI-S indications."
      export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:${ramDiskMountPoint}/SMI-SService/lib
      `sh ${ramDiskMountPoint}/SMI-SService/bin/save_indications.sh -o ${ramDiskMountPoint}/repository -n ${ramDiskMountPoint}/SMI-SService/repository -p ${ramDiskMountPoint}/port_rep -t ${ramDiskMountPoint}/SMI-SService/bin`
      # If that was successful, move the ported repository to new 
      if [ $? = 0 ]; then
	  LogTrace "SMI-S indications ported successfully."
	  RunCommand "cp -rf ${ramDiskMountPoint}/port_rep/* ${ramDiskMountPoint}/SMI-SService/repository"
      fi
      RunCommand "rm -rf ${ramDiskMountPoint}/port_rep"
      RunCommand "rm -rf ${ramDiskMountPoint}/repository"

      ExtractBZipDistributionFile "Extract SSL." "./SSLService/SSLService.tar.bz2"
      if [ ${distributionVersion} -ge 500 ]; then
         ExtractBZipDistributionFile "Extract ExternalLibs." "./ExternalLibs/ExternalLibs.tar.bz2"
      fi
   else
      ## If we are downgrading from I2 and above to something lower that SP3( 320G.GS004 )
      LogTrace "Extracting tar files." 
      ExtractBZipDistributionFile "Extract HTTP." "./HTTPService/HTTPService.tar"
      ExtractBZipDistributionFile "Extract Console." "./ConsoleService/ConsoleService.tar"
      ExtractBZipDistributionFile "Extract SNMP." "./SNMPService/SNMPService.tgz"
      ExtractBZipDistributionFile "Extract SMIS." "./SMI-SService/SMISService.tgz"
      ExtractBZipDistributionFile "Extract SSL." "./SSLService/SSLService.tar"
   fi

   ## Move distrubition from RAM disk
   command="cp -fR ${ramDiskMountPoint}/* ${applicationHome}"
   RunCommand "${command}" "1"
   ## Lets remove everything from RAM dist
   output=`ls ${ramDiskMountPoint} | grep -v 'lost+found'`
   for file in `echo $output`
   do
      RunCommand "rm -fr ${ramDiskMountPoint}/${file}" 
   done
}

#########################################
# Untar the files on RAM Disk
# Inputs: None
#
#########################################
ProccessUnzippedFile()
{
   local file=""
   file="$1"

   if [ ${distributionVersion} -ge 320 ]; then
      file=`echo ${file} | sed 's/\.bz2$//'`
   fi
   echo $file | egrep -e '.*\.tar$' -e '.*\.tgz$'
   if [ $? = 0 ]; then ## Proccess the tar files
      echo $file | grep 'ConsoleService' ## Special case for console since the distribution format makes no sence
      if [ $? = 0 ]; then
         local consoleDistributionDirectory="${ramDiskMountPoint}/ConsoleService"
         if [ ! -d "${consoleDistributionDirectory}" ]; then
            RunCommand "mkdir ${consoleDistributionDirectory}" "1"
         fi
         RunCommand "tar xf $file -C ${consoleDistributionDirectory}" "1"
      else
         echo $file | grep 'ExternalLibs' ## Special case for ExternalLibs
         if [ $? = 0 ]; then
             RunCommand "tar -xpf $file -C /" "1"
         else
            echo $file | egrep -e '.*\.tar$'
            if [ $? -eq 0 ]; then
                RunCommand "tar xf $file -C ${ramDiskMountPoint}" "1"
            else
                RunCommand "tar xzf $file -C ${ramDiskMountPoint}" "1"
            fi
         fi
      fi
      RunCommand "rm -f $file" "1"      ## Remove the tar file 
   fi
}

#########################################
# Extract All the other files (NON bz2 and BSP files) that are in the firmware image.
# Inputs: None
#
#########################################
ExtractRegularFiles()
{
   local list=""
   local file=""
   local command=""
   local tmp=""
   local filesToExtract=""

   ## TODO: Can we speed this up????
   list=`tar tvzf ${applicationDistributionName} | awk '{print $6}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
   LogTrace "Extract regular files." 
   for file in $list
   do
      echo "$file" | grep -v '.*\.bz2$' | grep -v '.*-pkg.sh' | grep -v '.*\.tar$' | grep -v '.*\.tgz$' >/dev/null
      if [ $? = 0 ]; then
         echo "${file}" | grep -v '.*\/$' >/dev/null ## Don't include directories
         if [ $? = 0 ]; then
            filesToExtract="${filesToExtract} ${file}"
         fi
      fi
   done
   if [ "${filesToExtract}" != "" ]; then
      LogTrace "Extracting ${filesToExtract}" 
      command="tar xvzf ${applicationDistributionName} -C ${ramDiskMountPoint} ${filesToExtract}"
      RunCommand "$command" "1"
   fi

   ## Lets process blade distribution if it exists.
   file=`find ${ramDiskMountPoint} -name ${bladeDistribution}`
   if [ "${file}" != "" ]; then
      LogTrace "Move blade files to ${bladeDistributionDirectory}" 
      command="mv ${file} ${bladeDistributionDirectory}"
      RunCommand "${command}" "1"
      file=`basename ${file}`
      RunCommand "chmod 777 ${bladeDistributionDirectory}/${file}"
   fi
   file=`find ${ramDiskMountPoint} -name ${bladeImage}`
   if [ "${file}" != "" ]; then
      command="mv ${file} ${bladeDistributionDirectory}"
      RunCommand "${command}" "1"
   fi

   ## Copy distrubition from RAM disk to distribution home
   command="cp -fR ${ramDiskMountPoint}/* ${applicationHome}"
   RunCommand "${command}" "1"
   if [ -d "${applicationHome}/lost+found" ]; then
      RunCommand "rm -fr ${applicationHome}/lost+found"
   fi
}

#########################################
# Extract and install BSP scripts
# Inputs: None
#
#########################################
InstallBSPscripts()
{
   local output=""
   local command=""
   local scriptToRun=""
   output=`tar tzf ${currentFirmwareFileName} | grep '.*-pkg.sh$' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
   if [ "${output}" != "" ]; then
      ## If we downgrade below SP3 we will need to save a copy of busybox
      if [ ${distributionVersion} -lt 320 ]; then
         LogTrace "Copy busybox to /tmp, were downgrading beyond SP3" 
         RunCommand "cp /bin/busybox /tmp"
         RunCommand "chmod 6755 /tmp/busybox"
      fi

      if [ -f "${NO_REBOOT_BSP_NOT_INCLUDED}" ]; then
         RunCommand "rm ${NO_REBOOT_BSP_NOT_INCLUDED}" 
      fi
      ## CRITICAL CODE SEGMENT: if we screw up here, the LCB could become inoperable.
      ##
      LogTrace "Extract and run BSP scripts from RAM disk" 
      for file in `echo $output`
      do
         command="tar xf ${currentFirmwareFileName} -C ${ramDiskMountPoint} $file"
         RunCommand "$command" "1"
         basefilename=`basename $file`
         scriptToRun=`find ${ramDiskMountPoint} -name ${basefilename}`
         if [ "${scriptToRun}" != "" ]; then
            LogTrace "Execute script $scriptToRun" 
            RunCommand "sh ${scriptToRun}" "1"
            RunCommand "rm ${scriptToRun}" "1"
         fi
      done
      ##
      ## End of CRITICAL SEGMENT

      ## If we downgrade below SP3 we will loose awk, we need it to complete the downgrade.
      if [ ${distributionVersion} -lt 320 ]; then
         if [ -f "/tmp/busybox" -a -f "/usr/bin/awk" ]; then
            LogTrace "Create temperary link for awk" 
            RunCommand "rm /usr/bin/awk"
            RunCommand "ln -s /tmp/busybox /usr/bin/awk"
         fi
      fi
   else
      LogTrace "No BSP to install" 
      ## Lets indicate no reboot is required since no BSP included in distribution (engineering build)
      if [ -f "${NO_REBOOT_BSP_NOT_INCLUDED}" ]; then
         LogError "No reboot indicator file exist, should have been cleaned up." 
      else
         RunCommand "touch ${NO_REBOOT_BSP_NOT_INCLUDED}" "1"
      fi
   fi
}

#########################################
# Make a backup of the current system.
# Inputs: 
#     1. THe name of the save file.
#
#########################################
SaveCurrentState() 
{
   local saveRestore="$1"

   LogTrace "Create System Backup File ..." 
   local backUpFileName=""
   local persistentTarball="${applicationHome}/images/systemStateFiles"
   local corePersistence=""
   local bladePersistence=""
   local otherPersistence=""
   local totalPersistence=""
   local command=""

   if [ "${saveRestore}" = "" ]; then
      backUpFileName="${applicationHome}/images/OldSystemState"
   else
      ## output=`date +%m_%e_%y-%H%M%S`
      ## backUpFileName="/tmp/saveRestore-${output}"
      backUpFileName="${saveRestore}"
   fi

   ## Lets move the new library.tgz to RAM disk to make space for upgrade.
   if [ -f "${upgradeFirmwareFileName}" -a  ${saveRestore} != "" ]; then
      command="mv ${upgradeFirmwareFileName} ${ramDiskMountPoint}"
      RunCommand "${command}" "1"
   fi
   corePersistence=`find ${applicationHome}/CoreService/dat | egrep -v -e 'RASLog*' -e 'Discovery*' -e '\/dat$'`
   bladePersistence=`find /etc/adic/I* | grep config`
##   bladePersistence=`find /etc/adic/I*`
   totalPersistence="${corePersistence} ${bladePersistence}"

   ## List of other persistence files, add new files to this list.
   otherPersistence="${applicationHome}/HTTPService/apache/conf/httpd.conf "
   otherPersistence="${otherPersistence} ${applicationHome}/HTTPService/apache/conf/ssl.conf "
   otherPersistence="${otherPersistence} ${applicationHome}/SMI-SService/etc/openwbem/openwbem.conf "
   otherPersistence="${otherPersistence} ${applicationHome}/SNMPService/share/snmp/snmpd.conf "
   otherPersistence="${otherPersistence} ${applicationHome}/townsendCA/* "
   otherPersistence="${otherPersistence} /etc/extaccess.conf "
   otherPersistence="${otherPersistence} /etc/ntp.conf "
   otherPersistence="${otherPersistence} /etc/SMISenabled "
   otherPersistence="${otherPersistence} /etc/krb5.conf "
   otherPersistence="${otherPersistence} /etc/localtime "
   otherPersistence="${otherPersistence} /etc/default/TZ "
   otherPersistence="${otherPersistence} /etc/lcm.inf "

   if [ ${saveRestore} != "" ]; then
      otherPersistence="${otherPersistence} /home/embedded/library/SMI-SService/repository/*"
      otherPersistence="${otherPersistence} /home/embedded/library/SNMPService/var/net-snmp/snmpd.conf"
      otherPersistence="${otherPersistence} /etc/serial.num" 
      otherPersistence="${otherPersistence} ${applicationHome}/branding/LCS_Personality.*"
   fi

   ## Check if files exist before adding them.
   for file in `echo ${otherPersistence}`
   do
      if [ -f "$file" ]; then
         totalPersistence="${totalPersistence} ${file}"
      fi
   done

   ## If there is one hanging around, remove it.
   if [ -f "${persistentTarball}.gz" ]; then
      command="rm ${persistentTarball}.gz"
      RunCommand "${command}" "1"
   fi

   local manifestFile="/MANIFEST.MF"
   command="tar cf ${persistentTarball} ${totalPersistence}"
   RunCommand "$command" "1"
   if [ "$saveRestore" != "" ]; then
      `tar tvf ${persistentTarball} > $manifestFile 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
      if [ $? -eq 0 ]; then
         command="tar -uf ${persistentTarball} $manifestFile"
         RunCommand "$command" "1"
         ## Lets remove file
         command="rm $manifestFile"
         RunCommand "$command" "1"
      fi
   fi
   command="gzip -f ${persistentTarball}"
   RunCommand "$command" "1"
   if [ ${saveRestore} != "" ]; then
      systemBackupFile="${backUpFileName}"
   fi
   command="tar cf ${systemBackupFile} ${currentFirmwareFileName} ${persistentTarball}.gz"
   RunCommand "$command" "1"
   command="rm ${persistentTarball}.gz"
   RunCommand "$command" "1"
   LogTrace "Completed system backup" 
}

#########################################
# Check if restore is from Quantum to ADIC
# Inputs: 1. directory to look for branding in restore persistence file
#         2. tarball to look for branding
#
#########################################
CheckPersonality()
{
   local directoryToCheck="$1"
   local tarballName="$2"
   local personality=""

   LogTrace "Check to see if we need to change library personality"
   ## Check if restoring from Quantum to ADIC
   if [ -f "${applicationHome}/branding/LCS_Personality.QUANTUM" ]; then
      if [ "${directoryToCheck}" = "" ]; then
         ## WE need to check the tarball
         tar tf ${tarballName} | grep LCS_Personality.ADIC > /dev/null 2>&1
         if [ $? -ne 0 ]; then
            ## either its pre I2 or the restore is Quantum personality
            tar tf ${tarballName} | grep LCS_Personality.QUANTUM > /dev/null 2>&1
            if [ $? -ne 0 ]; then
               changeLibraryPersonality="adic"
            fi
         else
            changeLibraryPersonality="adic"
         fi
      else
         ## look in saved persistence for branding file.
         personality=`find ${directoryToCheck} -name LCS_Personality.*`
         if [ ${personality} = "" ]; then
            ## Restore file has no branding file must be ADIC (pre I2).
            ## We need to change personalit to ADIC.
            changeLibraryPersonality="adic"
         else
            ## We do have a branding file, is it ADIC
            if [ "${personality}" = "${applicationHome}/branding/LCS_Personality.ADIC" ]; then
               changeLibraryPersonality="adic"
            fi
         fi
      fi
   fi

   if [ -f "${applicationHome}/branding/LCS_Personality.ADIC" -a "${tarballName}" != "" ]; then
      tar tf ${tarballName} | grep LCS_Personality.QUANTUM > /dev/null 2>&1
      if [ $? -eq 0 ]; then
         changeLibraryPersonality="quantum"
      fi
   fi

   if [ "${changeLibraryPersonality}" != "" ]; then
      LogTrace "We need to change personality to ${changeLibraryPersonality}, lets do it after all files are successfully restored"
   fi

}

#########################################
# Check if this is a corrective action upgrade
#
#########################################
CheckForCorrectiveAction()
{
   local whereAmI=`pwd`
   command="tar xzf /home/embedded/library/upload/library.tgz -C /var/tmp etc/MANIFEST.ca 2>/dev/null"
   eval "$command"
   if [ $? -eq 0 ]; then
      LogTrace "++++++ Corrective Action Upgrade ++++++"
      ## TODO: We might in some cases have to reboot
      RunCommand "touch ${NO_REBOOT_BSP_NOT_INCLUDED}"
      RunCommand "tar xzf ${upgradeFirmwareFileName} -C ${ramDiskMountPoint}"
      cd ${ramDiskMountPoint}
      RunCommand "cp -R * /"
      LogTrace "Run Corrective Action script"
      RunCommand "/bin/sh /home/embedded/library/AppManager/bin/fixit.sh"
      if [ -f "${upgradeFirmwareFileName}" ]; then
         RunCommand "rm ${upgradeFirmwareFileName}"
      fi
      LogTrace "++++++ Corrective Action Completed ++++++"
      exit 0
   fi
}

#########################################
# Usage: Display usage
# Inputs: None
#
#########################################
Usage()
{
   echo "Usage: $0 [ -c ] [ -u < 'upgrade' | 'recover' | 'restore' > ] [ -f 'filename' ] [ -s ] [ -h ]

   -c indicates we are moving to new app manager.
   -f the name of the file to save to.
   -h show this message
   -s save a library image, can be used by restore
   -u upgrade library firmware, where \"upgrade\" assumes new firmware image, \"restore\" from
   saved image and \"recover\" assumes we need to rollback

   ";
   exit 0
}

#################################################################################
####                               M A I N                                   ####
#################################################################################

whatToUpgrade=""
saveFileName=""

## Parce arguments
while getopts "cf:hsu:" opt
do
   case "$opt" in
      c) converToNewApplicationManager="true"
      ;;
      f) saveFileName="$OPTARG"
      ;;
      h) Usage
      ;;
      s) whatToUpgrade="save"
      ;;
      u) whatToUpgrade="$OPTARG"
      if [ "${whatToUpgrade}" != "upgrade" -a  "${whatToUpgrade}" != "restore" -a  "${whatToUpgrade}" != "recover" -a  "${whatToUpgrade}" != "save" ]; then
         echo "ERROR: not a valid argument to -u option"
         Usage
      fi
      ;;
      *) Usage
      ;;
   esac
done

if [ $# -eq 0 ]; then
   ## default if no arguments
   whatToUpgrade="upgrade"
fi

InitLogFile "/var/log/appManager.log"

## Remove unwanted files.
filesToRemove=""
filesToRemove=`find /home/embedded/library/ -type f | egrep -e '\/core\.*' -e '\.dbg$'`
if [ "${filesToRemove}" != "" ]; then
   LogTrace "Removing files ${filesToRemove}" 
   RunCommand "rm ${filesToRemove}" "1"
fi

## Create RAM Disk
commandOut=`mount | grep '/dev/ram0 on' >/dev/null`
if [ $? -eq 0 ]; then
   LogTrace "RAM Disk already mounted, lets unmount it" 
   UnmountRAMdisk
fi

## Save requested, lets do a backup 
if [ "${whatToUpgrade}" = "save" ]; then
   LogTrace "Create a System recovery file ..." 
   SaveCurrentState "${saveFileName}"
   exit 0
fi

if [ "${whatToUpgrade}" = "upgrade" ]; then
   CreateRAMdisk "${ramDiskSize}"
   CheckForCorrectiveAction
   if [ "${converToNewApplicationManager}" != "true" ]; then
      ## Lets move the new library.tgz to RAM disk to make space for upgrade. We
      ## need the space so we can make a backup of the current state.
      if [ -f "${upgradeFirmwareFileName}" ]; then
         LogTrace "Move new firmware distribution to RAM disk" 
         command="mv ${upgradeFirmwareFileName} ${ramDiskMountPoint}"
         RunCommand "${command}" "1" 
      else
         LogError "No upgrade file was found." 
         exit 1
      fi

      ## Now lets do a backup of existing system configuration
      if [ -f "${currentFirmwareFileName}" ]; then
         SaveCurrentState
      else
         LogTrace "We are doing an upgrade but no \"${currentFirmwareFileName}\" was found" 
      fi

      ## We successfully saved the current system state, we will now move the new upgrade file over the current
      ## file and try to extract it. 
      if [ -f "${ramDiskMountPoint}/library.tgz" ]; then
         LogTrace "Overlay old firmware image with new firmware image from RAM disk" 
         command="mv ${ramDiskMountPoint}/library.tgz ${currentFirmwareFileName}"
         RunCommand "$command" "1"
      fi
   fi

   ## Now lets check if no images files can be found
   if [ ! -f "${currentFirmwareFileName}" ]; then
      LogError "No firmware image files found on library" 
      UnmountRAMdisk
      exit 1
   fi

   GetDistributionVersion "${currentFirmwareFileName}"
   if [ "${converToNewApplicationManager}" != "true" ]; then
      InstallBSPscripts
   else
      RunCommand "touch ${NO_REBOOT_BSP_NOT_INCLUDED}" 
   fi

   ## Remove old RAS logs
   if [ ${distributionVersion} -ge 500 ]; then
      RASlogfiles="${applicationHome}/CoreService/dat/RASLog.Hot ${applicationHome}/CoreService/dat/RASLog.Warm ${applicationHome}/CoreService/dat/RASLog.Cool" 
      for logFile in `echo $RASlogfiles`
      do
         if [ -f "$logFile" ]; then
            LogTrace "Removing old RAS log file $logFile"
            RunCommand "rm $logFile"
         fi
      done
   fi
   ExtractZippedFiles 
   ExtractRegularFiles
   LogTrace "Upgrade completed successfully" 
   UnmountRAMdisk
fi

## We are recovering from an attempted upgrade or restoring to an old version
if [ "${whatToUpgrade}" = "recover" -o "${whatToUpgrade}" = "restore" ]; then
   local command=""
   local tmpfilename=""
   local serialNumber=""
   local serialNumberCheck=""
   local message=""

   if [ "${whatToUpgrade}" = "restore" ]; then
      systemBackupFile="${restoreFilename}"
      message="restore"
      LogTrace "Restore system backup file ${systemBackupFile}" 
   else
      message="recovery"
      LogTrace "Start Recovery from botched upgrade" 
   fi
   systemBackupFileBaseName=`basename ${systemBackupFile}`

   CreateRAMdisk "${ramDiskSize}"
   if [ ! -f "${systemBackupFile}" ]; then
      LogError "No $message file found \"${systemBackupFile}\"." 
      exit 2
   fi

   ## Lets extract files, there should be a firmware image and persistence backup
   LogTrace "Extract the image to RAM Disk" 
   command="tar xf ${systemBackupFile} -C ${ramDiskMountPoint}"
   RunCommand "${command}" "1"

   ## Lets remove it from the file system.
   command="rm ${systemBackupFile}"
   RunCommand "$command" "1"

   LogTrace "Retrive $message persistent files" 
   tmpfilename=`find ${ramDiskMountPoint} -name ${persistentTarballBaseName}.gz`
   if [ "${tmpfilename}" != "" ]; then
      command="gzip -d ${tmpfilename}"
      RunCommand "${command}" "1"
   else
      LogInfo "No ${persistentTarballBaseName}.gz, restore must be pre 300G.GS0015"
      restoreIsPerVersion300G="true"
   fi

   if [ "${restoreIsPerVersion300G}" != "true" ]; then
      tmpfilename=`echo ${tmpfilename} | sed 's/\.gz$//'`
      CheckPersonality "" "${tmpfilename}"
      ## Lets install persistence.
      command="tar xf ${tmpfilename} -C /"
      RunCommand "${command}" "1"
   fi

   LogTrace "Move $message firmware image to application home images directory, overlay existing one." 
   tmpfilename=`find ${ramDiskMountPoint} -name ${applicationDistributionBasename}`
   if [ "${tmpfilename}" != "" ]; then
      command="mv -f ${tmpfilename} ${applicationHome}/images"
      RunCommand "${command}" "1"
   else
      LogError "Failed to find firmware image in $images file." 
      exit 1
   fi

   if [ "${restoreIsPerVersion300G}" = "true" ]; then
      CheckPersonality "${ramDiskMountPoint}/home/" ""
      LogTrace "copy persistence from RAM Disk"
      RunCommand "cp -R ${ramDiskMountPoint}/home/* /home" "1"
   fi


   local file=""
   ## Remve files from RAM Disk
   for file in `ls ${ramDiskMountPoint} | grep -v 'lost+found'`
   do
      command="rm -fr ${ramDiskMountPoint}/${file}"
      RunCommand "$command" 
   done

   GetDistributionVersion "${currentFirmwareFileName}"
   ## Do we need to do this.
   InstallBSPscripts
   ExtractZippedFiles 
   ExtractRegularFiles

   ## Restart NTP in case the servers changed ##
   LogTrace "Restarting NTP service"
   RunCommand "/etc/init.d/ntpd restart" "0"

   if [ "${whatToUpgrade}" = "recover" ]; then
      LogTrace "Recovery completed successfully" 
   fi
   if [ "${whatToUpgrade}" = "restore" ]; then
      LogTrace "Restore completed successfully" 
   fi
   UnmountRAMdisk

   if [ "${changeLibraryPersonality}" != "" ]; then
      if [ "${changeLibraryPersonality}" = "adic" ]; then
         if [ -f "/home/embedded/library/branding/LCS_Personality.ADIC" ]; then
            RunCommand "rm /home/embedded/library/branding/LCS_Personality.ADIC"
         fi
      else
         if [ -f "/home/embedded/library/branding/LCS_Personality.QUANTUM" ]; then
            RunCommand "rm /home/embedded/library/branding/LCS_Personality.QUANTUM"
         fi
      fi
      RunCommand "sh /home/embedded/library/AppManager/bin/changePersonality.sh -p ${changeLibraryPersonality}"
   fi
fi
exit 0
