#!/bin/sh
#
# $Id: bin/include.sh 1.13 2007/05/21 16:50:12MDT Jim Boyd (jboyd) dev  $
##

NO_REBOOT_BSP_NOT_INCLUDED="/tmp/noRebootRequiredBSPnotIncludedinDistribution"
I500_APPLICATION_MANAGER_LOG_FILE="/var/log/appManager.log"
COMMAND_EXIT_STATUS=0
COMMAND_OUTPUT=""
ramDiskSize=81920       
ramDiskMountPoint="/var/tmp"

## File indicator that user is changing personality.
userChangeingLibraryPersonality="/tmp/.userChangeingLibraryPersonality"

## Paths to application
applicationHome="/home/embedded/library"

## Firmware image names and locations
applicationDistributionName="/home/embedded/library/images/library.tgz"
applicationDistributionBasename=`basename ${applicationDistributionName}`
systemBackupFile="${applicationHome}/images/OldSystemState"
systemBackupFileBaseName=`basename ${systemBackupFile}`
restoreFilename="${applicationHome}/AppManager/dat/restore.tgz"
persistentTarballBaseName="systemStateFiles"
persistentTarball="${applicationHome}/images/${persistentTarballBaseName}"

currentFirmwareFileName="${applicationDistributionName}"
upgradeFirmwareFileName="${applicationHome}/upload/${applicationDistributionBasename}"
bladeDistributionDirectory="/tftpboot"
bladeImage="vxworks_7404_iob"
bladeDistribution="FCB_7404_IPENGUIN_All.bin"

lastFirmwareUpdateFile="/etc/lastFirmwareUpdate"

## Flag to determine that we have upgraded from old AppMgr to new appManager
converToNewApplicationManager="false"

restoreIsPerVersion300G="false"

## Files to communicate errors to web client
statusFirmwareUpgradeFailed="${applicationHome}/AppManager/status/statusFirmwareUpgradeFailed"
statusFirmwareRestoreFailed="${applicationHome}/AppManager/status/statusFirmwareRestoreFailed"
statusFirmwareRecoverFailed="${applicationHome}/AppManager/status/statusFirmwareRecoverFailed"
statusFirmwareCouldNotValidateVersion="${applicationHome}/AppManager/status/statusFirmwareCouldNotValidateVersion"
statusFirmwareRestoreFileCorrupted="${applicationHome}/AppManager/status/statusFirmwareRestoreFileCorrupted"
statusFirmwareRestoreSerialNumberMismatch="${applicationHome}/AppManager/status/statusFirmwareRestoreSerialNumberMismatch"
statusFirmwareRestoreVersionGreaterThanOld="${applicationHome}/AppManager/status/statusFirmwareRestoreVersionGreaterThanOld"

# Test if IPv6 is enabled
local IPv6_FLAG="/etc/ipv6"
if [ -e ${IPv6_FLAG} ]; then
    IPv6="true"
else
    IPv6="false"
fi

#########################################
# Closes all file descriptors connected
# to http sockets.
#
# Inputs: none
#
#########################################
CloseHttpFileDescriptors()
{
   # use lsof to get any fds to http
   for i in `lsof -i :http -Ff | grep -A1 "p$$" | grep "f" | cut -b 2-6`
   do
      eval "exec $i>&-";
   done
}

#########################################
# Wrapper for running cli commands
# Inputs: 
#  1. CLI to run
#  2. Indicator on whether to exit on error.
#
#  Example: RunCommand "ls -lrt" (do not exit application if error)
#           RunCommand "ls -lrt /FFF" "true" (exit application on error)
#
#########################################
RunCommand()
{
   local command="$1"
   local exitOnError="$2"

   COMMAND_OUTPUT=`eval $command 2>&1`
   COMMAND_EXIT_STATUS=$?
   if [ ${COMMAND_EXIT_STATUS} != 0 ]; then
      LogError "COMMAND \"$command\" : $COMMAND_OUTPUT"
      if [ "X${exitOnError}" != "X" ]; then
         exit ${exitOnError}
      fi
   fi
}

#########################################
# Create a RAM file system
# Inputs: 
#     1. The RAM disk size in Kb's. ( this is optional, the default size is 64Mb
#
#########################################
CreateRAMdisk()
{
   LogTrace "Creating RAM Disk /dev/ram0..." 
   local ramDiskDevice="/dev/ram0"
   local requestedSize="$1" ## Size in Kb

   LogTrace "Unmount RAM Disk /dev/ram0 if already exists..." 
   UnmountRAMdisk

   if [ "X${requestedSize}" != "X" ]; then
      ramDiskSize=${requestedSize}
   fi

   if [ ! -d "${ramDiskMountPoint}" ]; then
      RunCommand "mkdir ${ramDiskMountPoint}" "1"
   fi
   ## Format RAM Disk
   LogTrace "Format RAM Disk /dev/ram0" 
   RunCommand "dd if=/dev/zero of=${ramDiskDevice} bs=1k count=${ramDiskSize} >/dev/null" "1"
   ## Make RAM Disk File System
   LogTrace "Make RAM Disk file system" 
   RunCommand "mkfs.ext3 -vm0 ${ramDiskDevice} ${ramDiskSize} >/dev/null" "1"
   ## Mount RAM Disk
   LogTrace "Mount RAM Disk file system" 
   RunCommand "mount ${ramDiskDevice} ${ramDiskMountPoint}" "1"
}

#########################################
# Unmount the RAM Disk
# Inputs: None
#
#########################################
UnmountRAMdisk()
{
   local output=""
   output=`mount | grep '^/dev/ram0'`
   if [ $? -eq 0 ]; then
      LogTrace "Unmounting RAM Disk ${ramDiskMountPoint}..." 
      RunCommand "umount ${ramDiskMountPoint}" "1"
      RunCommand "freeramdisk /dev/ram0"
   else
      LogTrace "RAM Disk ${ramDiskMountPoint} does not exist." 
   fi
}

#########################################
# Log a message to a file
# Inputs: Message to log
#
#########################################
LogToFile()
{
   local input="$1"
   echo $input >> ${I500_APPLICATION_MANAGER_LOG_FILE}
   CheckLogFileSize
}

#########################################
# Initialize the log file
# Inputs: Name of the log file
#
#########################################
InitLogFile()
{
   local filename="$1"
   local command=""
   if [ "${filename}" != "" ]; then
      I500_APPLICATION_MANAGER_LOG_FILE="${filename}"
      if [ ! -f "${I500_APPLICATION_MANAGER_LOG_FILE}" ]; then
         touch ${I500_APPLICATION_MANAGER_LOG_FILE}
      fi
   fi
   CheckLogFileSize
}

#########################################
# Checks the log file size and if greater than max size it makes a backup and truncates 
# the current one
#
#########################################
CheckLogFileSize()
{
   local size=0
   size=`ls -l ${I500_APPLICATION_MANAGER_LOG_FILE} | awk '{print $5}'`
   if [ ${size} -gt 524288 ]; then
      command="cp ${I500_APPLICATION_MANAGER_LOG_FILE} ${I500_APPLICATION_MANAGER_LOG_FILE}.bak"
      eval $command
      if [ $? = 0 ]; then
         echo "Backed up old log file." > ${I500_APPLICATION_MANAGER_LOG_FILE}
         if [ -f "${I500_APPLICATION_MANAGER_LOG_FILE}.bak.gz" ]; then
            RunCommand "rm ${I500_APPLICATION_MANAGER_LOG_FILE}.bak.gz"
         fi
         RunCommand "gzip ${I500_APPLICATION_MANAGER_LOG_FILE}.bak"
      fi
   fi
}

#########################################
# Formats the message you want to log
# Inputs: 
#    1. Log level
#    2. Message to log
#
#########################################
LogMessage()
{
   local level="$1"
   local message="$2"
   local theTime=""
   local name=`basename $0`
   theTime=`date '+%b %e %H:%M:%S'`
   local output="$level [$theTime] [${name}] $message"

   ## echo "$output"
   LogToFile "$output"
}

#########################################
# Log INFO message
# Inputs: 
#    1. Message to log
#    2. Script name that called this function
#
#########################################
LogInfo()
{
   local message="$1"
   LogMessage "INFO" "${message}"
}

#########################################
# Log DEBUG message
# Inputs: 
#    1. Message to log
#    2. Script name that called this function
#
#########################################
LogDebug()
{
   local message="$1"
   LogMessage "DEBUG" "${message}"
}

#########################################
# Log TRACE message
# Inputs: 
#    1. Message to log
#    2. Script name that called this function
#
#########################################
LogTrace()
{
   local message="$1"
   LogMessage "TRACE" "${message}"
}

#########################################
# Log ERROR message
# Inputs: 
#    1. Message to log
#    2. Script name that called this function
#
#########################################
LogError()
{
   local message="$1"
   LogMessage "ERROR" "${message}"
}

#########################################
# Log FATAL message
# Inputs: 
#    1. Message to log
#    2. Script name that called this function
#
#########################################
LogFatal()
{
   local message="$1"
   LogMessage "FATAL" "${message}"
}
