#!/usr/bin/tclsh
#
# $Header: bin/krb5Config.tcl 1.5 2007/11/15 10:46:57MST Brian Szmyd (bszmyd) dev  $
#
# $Copyright: $
#
# $Log: bin/krb5Config.tcl  $
# Revision 1.5 2007/11/15 10:46:57MST Brian Szmyd (bszmyd) 
# Update /etc/hosts to add an entry of the libraries FQDN which is neccissary for Kerberos support.
# Append port number to KDC when getting current settings.
# Revision 1.4 2007/11/14 13:42:52MST Brian Szmyd (bszmyd) 
# Append KDC port to URI when getting settings.
# Revision 1.3 2007/09/27 14:21:41MDT Brian Szmyd (bszmyd) 
# Implemented getter for configuration info.
# Revision 1.2 2007/09/26 11:04:59MDT Brian Szmyd (bszmyd) 
# Fixed space at beginning of INI sections which libkrb5 couldn't handle.
# Support for mapping a domain to the realm.
# Revision 1.1 2007/09/19 17:13:51MDT Brian Szmyd (bszmyd) 
# Initial revision
# Member added to project e:/mks/projects/predator/ccDevModel/Code/Applications/Scalar101/S101Components/AppManager/project.pj
#

proc usage { command } {
    puts stderr "Usage: $command \[-g\] \[-s REALM KDC DOMAIN\]\n\n\
        actions:\n\
        \t-g     - Get current kerberos setup\n\
        \t-s     - Set current kerberos setup\n\n\
        options:\n\
        \tREALM  - The realm to authenticate principals against (e.g. QUANTUM.COM)\n\
        \tKDC    - The server responsible for authentication of the realm (e.g. kerberos.quantum.com:88)\n\
        \tDOMAIN - A Domain to map to the realm\n\n\
        return:\n\
        \tDuring a set operation, the return code of the shell will be set to '0' upon success.\n\n\
        \tDuring a get operation, if kerberos is configured you will receive a space delimited\n\
        \tstring identical to the ordering of the set operation's input parameters. Return code\n\
        \twill be '0' upon success.\n";
}

set krb5_conf "/etc/krb5.conf";

set realm_re {^([[:alnum:]][[:alnum:]\.]*)$};
set uri_re {^([[:alnum:]][[:alnum:]\-\.]*)(:[0-9]+)?$};
set map_re {^[[:alnum:]][[:alnum:]\-\.]*$}

set section_re {^\s*\[([[:alnum:]_]+)\]\s*$}
set configured_realm_re {^\s*([[:alnum:]][[:alnum:]\.]*)\s+=\s+\{\s*$};
set configured_uri_re {^\s*kdc\s+=\s+([[:alnum:]][[:alnum:]\-\.]*)(:[0-9]+)?\s*$};
set configured_map_re {^\s*([[:alnum:]][[:alnum:]\-\.]*)\s+=\s+.*$}
 

if { $argc < 1 } {
    usage $argv0;
    exit 2;
} elseif [string equal [lindex $argv 0] "-s" ] {

    if { $argc < 4 } {
        usage $argv0;
        exit 2;
    }

    set realm_raw [lindex $argv 1];
    set kdc_raw [lindex $argv 2];
    set mapping [lindex $argv 3];

    if [regexp $realm_re  $realm_raw  match  realm] {

        if [regexp $uri_re  $kdc_raw  match  kdc  kdc_port] {

            if [catch {open $krb5_conf w 0444} fileId] {
                puts stderr "Cannot open $krb5_conf: $fileId";
                exit 3;
            } else {
                puts $fileId "######## Auto-Generate Config File ########";
                puts $fileId "###       Do not modify by hand       ###\n";
                puts $fileId "\[libdefaults\]";
                puts $fileId "\tdefault_realm = $realm\n";
                puts $fileId "\[realms\]";
                puts $fileId "\t$realm = {";
                puts $fileId "\t\tkdc = $kdc$kdc_port";
                puts $fileId "\t}\n";
                puts $fileId "\[domain_realm\]";

                if [regexp $map_re $mapping  match] {

                    set io [open "|hostname" r];
                    if { [gets $io hostname] > 0 } {
                        file rename -force /etc/hosts /etc/hosts.bak;

                        set fin [open /etc/hosts.bak r];
                        set fout [open /etc/hosts w];

                        while {[gets $fin linein] >= 0} {
                            regsub "^((\[\[:digit:\]\]{1,3}\.){3}\[\[:digit:\]\]{1,3})\[\[:space:\]\]+$hostname.*$" $linein "\\1    $hostname.$mapping $hostname" lineout;
                            puts $fout $lineout;
                        }
                        close $fin;
                        close $fout;

                        file delete -force /etc/hosts.bak;

                        puts $fileId "\t$mapping = $realm";
                        puts $fileId "\t.$mapping = $realm";
                    } else {
                        exit 4;
                    }
                    close $io;
                }
                close $fileId;
            }
        } else {
            puts stderr "Invalid KDC";
            exit 1;
        }
    } else {
        puts stderr "Invalid Realm";
        exit 1;
    }

} elseif [string equal [lindex $argv 0] "-g"] {

    if [catch {open $krb5_conf r} fileId] {
        puts stderr "Cannot open $krb5_conf: $fileId";
        exit 3;
    }

    set in_realms 0;
    set in_maps 0;
    set maps [list]

    while {[gets $fileId line] >= 0} {

        if [ regexp $section_re $line match section ] {
            set in_realms 0;
            set in_maps 0;

            if [string equal $section "realms"] {
                set in_realms 1;
            } elseif [string equal $section "domain_realm"] {
                set in_maps 1;
            }
        } elseif { $in_realms == 1 } {
            regexp $configured_realm_re  $line  match  realm;
            regexp $configured_uri_re  $line match  kdc  kdc_port;

            if { [info exists realm]  && [info exists kdc] } {
                set in_realms 0;
            }
        } elseif { $in_maps == 1 } {
            if { [regexp $configured_map_re  $line  match map] } {
                lappend maps $map;
            }
        } 
    }

    if { [info exists realm] && [info exists kdc] } {
        set ret_val [concat $realm $kdc$kdc_port];
        for { set i 0 } { $i < [llength $maps] } { incr i 1 } {
            set ret_val [concat $ret_val [lindex $maps $i] ];
        }
        puts stdout $ret_val;
    } else {
        exit 1;
    }
} else {
    usage $argv0;
    exit 2;
}

exit 0;
