DNS_CONFIG_FILE="/etc/resolv.conf"

NTP_CONFIG_FILE="/etc/ntp.conf"
NTP_SERVICE_DAEMON="/etc/init.d/ntpd"

##Error codes
EXIT_STATUS_ADDRESS_CONFLICT=44

#########################################
# startNetwork
# Start the network interfaces
#
#########################################
startNetwork()
{
   LogTrace "Start network"
   RunCommand "/etc/init.d/networkingEth2 start"
}

#########################################
# stopNetwork
# Stop the network interface and DHCP
#
#########################################
stopNetwork()
{
   LogTrace "Stop network"
   RunCommand "/etc/init.d/networkingEth2 stop"
}

#########################################
# restartNetwork
# Restart the network interface
#
#########################################
restartNetwork()
{
   cnt=0
   while [ -f /tmp/netlock ]
   do
     LogTrace "Waiting to restart network"
     if [ $cnt -gt 120 ]; then
     	rm -f /tmp/netlock
     fi
     cnt=`expr $cnt + 1`
     sleep 1
   done
   LogTrace "Restart network"
   touch /tmp/netlock
   RunCommand "/etc/init.d/networkingEth2 restart"
   rm /tmp/netlock
}

#########################################
# changeLibraryHostname
# Change the library's host name. Updates the /etc/hosts and /etc/hostname files.
#
# Inputs: New hostname
#
#########################################
changeLibraryHostname()
{
   local newHostname="$1"
   local currentHostname=`hostname`

   RunCommand "sed -i \"s/$currentHostname/$newHostname/g\" /etc/hosts /etc/hostname"
   RunCommand "hostname $newHostname"
}

#########################################
# getLibraryHostname
# Get the library hostname
#
#########################################
getLibraryHostname()
{
   local hostname=`hostname`
   COMMAND_EXIT_STATUS=$?
   echo $hostname
}

#########################################
# configureDNS
#
# update the dns resolution servers
#########################################
configureDNS()
{
    local dnsServers="$1"
    local result=0

    for server in `echo $dnsServers`;
    do
        res=`/usr/bin/verifyAddress.sh ${server}`
        if [ $? -ne 0 ]; then
            result=1
        fi
    done
    
    if [ $result -eq 0 ]; then
        echo "##### Configured by services.sh #####" > $DNS_CONFIG_FILE
        for server in `echo $dnsServers`;
        do
            echo "nameserver ${server}" >> $DNS_CONFIG_FILE
        done
    else
        COMMAND_EXIT_STATUS=$EXIT_STATUS_ADDRESS_CONFLICT
    fi
}

#########################################
# getDomainServers
#
# 
#########################################
getDomainServers()
{
    servers=`cat $DNS_CONFIG_FILE | grep "^nameserver" | awk '{print $2}'`
    if [ -z "$servers" ]; then
        echo "disabled"
    else
        echo $servers
    fi
}

#########################################
# configureNTP
#
# 
#########################################
configureNTP()
{
    local result=0
    ntpServers="$1"

    if [ "$ntpServers" = "disable" ]; then
        echo "##### Configured by servicesManager.sh #####" > $NTP_CONFIG_FILE
        echo "#server disabled iburst" >> $NTP_CONFIG_FILE
    else
        numServers=`echo $ntpServers | awk '{print NF}'`

        if [ ${numServers} -gt 0 ]; then
            for server in `echo $ntpServers`;
            do
                res=`/usr/bin/verifyAddress.sh ${server}`
                if [ $? -ne 0 ]; then
                    result=1
                fi
            done

            if [ $result -eq 0 ]; then
                echo "##### Configured by servicesManager.sh #####" > $NTP_CONFIG_FILE
                for server in `echo $ntpServers`;
                do
                    echo "server ${server} iburst" >> $NTP_CONFIG_FILE
                done
            fi
        fi
    fi

    if [ $result -eq 0 ]; then
        echo "
driftfile /etc/ntp.drift
controlkey 1
requestkey 1
trustedkey 1
server 127.127.1.1
fudge  127.127.1.1 stratum 12
tinker panic 0
    " >> $NTP_CONFIG_FILE

        $NTP_SERVICE_DAEMON restart
    else
        COMMAND_EXIT_STATUS=$EXIT_STATUS_ADDRESS_CONFLICT
    fi
}

#########################################
# getNTPServers
#
# 
#########################################
getNTPServers()
{
    servers=`cat $NTP_CONFIG_FILE | grep "^server .* iburst$" | awk '{print $2}'`
    if [ -z "$servers" ]; then
        echo "disabled"
    else
        echo $servers
    fi
}

#########################################
# makeSSLCert
#
# 
#########################################
makeSSLCert()
{
   local hostName=`hostname`;
   local contact=`echo $1 | awk '{print $1}'`;
   local company=`echo $1 | awk '{print $2}'`;

   local isSSLenabled=`cat /etc/extaccess.conf | grep ssl_gui_flag | awk -F= '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
   local rand=`cat /etc/wwname`

   if [ ${isSSLenabled} -ne 0 ]; then
       RunCommand "/home/embedded/library/SSLService/bin/mkkey /home/embedded/library/SSLService NA NA NA $hostName \"$contact\" \"$company\" NA $rand";
       RunCommand "/home/embedded/library/HTTPService/apache/bin/apachectl stop";
   fi

}
