#!/bin/sh
#
# $Id: bin/validateDistribution.sh 1.5 2007/09/07 09:55:41MDT Jim Boyd (jboyd) dev  $
##

## Path to application home
applicationHome="/home/embedded/library"

## Source include file.
. ${applicationHome}/AppManager/bin/include.sh

Usage()
{
   echo "Usage: $0 < -f "restore file name" [-d] | [-r] > [ -h ] ]  

   -d check downgrade distribution
   -f the restore file name
   -h show this message
   -r check restore distribution
   ";
   exit 1
}

ValidateRestoreFile()
{
	local firmwareDistrubutionName="$1"
	local command=""
	local tmpfilename=""
	local restoreIsPerVersion300G="false"

	RunCommand "tar xf ${firmwareDistrubutionName} -C ${ramDiskMountPoint}" 

	if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
		LogTrace "Retrive recovery persistent files" 
		tmpfilename=`find ${ramDiskMountPoint} -name ${persistentTarballBaseName}.gz`
		if [ "${tmpfilename}" != "" ]; then
			RunCommand "gzip -d ${tmpfilename}"
		else
			LogInfo "No ${persistentTarballBaseName}.gz, restore must be pre 300G.GS0015"
			restoreIsPerVersion300G="true"
		fi
	fi

	if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
		if [ "${restoreIsPerVersion300G}" = "false" ]; then
			tmpfilename=`echo ${tmpfilename} | sed 's/\.gz$//'`
			## Lets check that the restore file is compatable, check serial numbers are the same
			RunCommand "tar xf ${tmpfilename} -C ${ramDiskMountPoint}" 
		fi
		LogTrace "Checking serial numbers"
		tmp1=`cat /etc/serial.num`
		if [ -f "${ramDiskMountPoint}/etc/serial.num" ]; then
			tmp2=`cat ${ramDiskMountPoint}/etc/serial.num`
			if [ "${tmp1}" != "${tmp2}" ]; then
				LogError "Serial numbers do not match, current: $tmp1, distribution: $tmp2"
				COMMAND_EXIT_STATUS=1
			else
				LogTrace "Serial numbers match, current: $tmp1, distribution: $tmp2"
			fi
		else
			LogTrace "Did not find /etc/serial.num in distribution lets check for SERIAL_NUM file"
			if [ -f "${ramDiskMountPoint}/SERIAL_NUM" ]; then
				tmp2=`cat ${ramDiskMountPoint}/SERIAL_NUM`
				if [ "${tmp1}" != "${tmp2}" ]; then
					LogError "Serial numbers do not match, current: $tmp1, distribution: $tmp2"
					COMMAND_EXIT_STATUS=1
				else
					LogTrace "Serial numbers match, current: $tmp1, distribution: $tmp2"
				fi
			else
				LogTrace "Did not find serial number file in distribution."
				COMMAND_EXIT_STATUS=1
			fi
		fi
	fi

	if [ ${COMMAND_EXIT_STATUS} = 0 -a  ${EXIT_STATUS} = 0 ]; then
		## Lets report versions
		tmpfilename=`find ${ramDiskMountPoint} -name ${applicationDistributionBasename}`
		if [ "${tmpfilename}" != "" ]; then
			RunCommand "tar xf ${tmpfilename} -C ${ramDiskMountPoint} ./S101CoreVersion.cpp"
			if [ -f "${ramDiskMountPoint}/S101CoreVersion.cpp" ]; then
				newVersion=`cat ${ramDiskMountPoint}/S101CoreVersion.cpp  | grep FirmwareImageVersion | grep 'const char' | awk -F\" '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
			fi
			if [ -f "${applicationHome}/S101CoreVersion.cpp" ]; then
				currentVersion=`cat ${applicationHome}/S101CoreVersion.cpp  | grep FirmwareImageVersion | grep 'const char' | awk -F\" '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
			fi
			LogTrace "Current version: ${currentVersion}, restore version: ${newVersion}" 

			LogTrace "Checking for valid restore version" 
			old=`echo $currentVersion | cut -c1,2,3`
			new=`echo $newVersion | cut -c1,2,3`
			## Validate we can restore filmware.
			if [ $new -gt $old ]; then
				LogTrace "The restore distribution is not valid, restore version $new is more recent than current $old." 
				COMMAND_EXIT_STATUS=2
			fi
		else
			COMMAND_EXIT_STATUS=2
		fi
	fi
}

ValidateUpgradeDowngradeFile()
{
   local firmwareDistrubutionName="$1"
   local personality=""
   local newVersion=""
   local hasBlades="false"
   local fileSize=0

   if [ -f "${applicationHome}/branding/LCS_Personality.QUANTUM" ]; then
      personality="quantum"
   fi
   if [ -f "${applicationHome}/branding/LCS_Personality.ADIC" ]; then
      personality="adic"
   fi
   if [ -f "${applicationHome}/branding/LCS_Personality.DELL" ]; then
      personality="dell"
   fi
   LogTrace "Library Personality [${personality}]"

   if [ -f "/home/embedded/library/CoreService/dat/bmsbids" ]; then
      fileSize=`ls -l /home/embedded/library/CoreService/dat/bmsbids | awk '{print $5}'`
      if [ ${fileSize} -gt 0 ]; then
         LogTrace "Library has blade[s]"
         hasBlades="true"
      fi
   fi

   command="tar xzf ${firmwareDistrubutionName} -C /var/tmp etc/MANIFEST.ca 2>/dev/null"
   eval "$command"
   if [ $? -eq 0 ]; then
      LogTrace "Corrective Action distribution..."
   else
      RunCommand "tar xf ${firmwareDistrubutionName} -C ${ramDiskMountPoint} ./S101CoreVersion.cpp" 
      if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
         if [ -f "${ramDiskMountPoint}/S101CoreVersion.cpp" ]; then
            newVersion=`cat ${ramDiskMountPoint}/S101CoreVersion.cpp  | grep FirmwareImageVersion | grep 'const char' | awk -F\" '{print $2}' 2>>${I500_APPLICATION_MANAGER_LOG_FILE}`
            majorNumber=`echo $newVersion | cut -c1,2,3`
         fi
      else
         LogTrace "Failed to find S101CoreVersion.cpp in distribution"
         COMMAND_EXIT_STATUS=2
      fi

      if [ ${COMMAND_EXIT_STATUS} = 0 ]; then
         ## We are either ADIC, Quantum or DELL and we have blades and we trying to downgrade to pre 400
         if [ "X${hasBlades}" = "Xtrue" -a "X${personality}" != "X" -a ${majorNumber} -lt 400 ]; then 
            LogTrace "Library has blades and trying to downgrade to version ${majorNumber}"
            COMMAND_EXIT_STATUS=3
         fi
         ## if we are quantum and trying to downgrade to pre 400
         if [ ${COMMAND_EXIT_STATUS} -eq 0 -a "X${personality}" = "Xquantum" -a ${majorNumber} -lt 400  ]; then
            LogTrace "Library has personality quantum trying to downgrade to version ${majorNumber}"
            COMMAND_EXIT_STATUS=4
         fi
         ## The library has an apollo robot and we are trying to downgrade below 520 (I4).
         if [ ${COMMAND_EXIT_STATUS} -eq 0 -a ${majorNumber} -lt 520 -a -f "/etc/ApolloRobot"  ]; then
            LogTrace "Library has an Apollo Robot and we are trying to downgrade to version ${majorNumber}"
            COMMAND_EXIT_STATUS=5
         fi
      fi
   fi
}

#################################################################################
####                               M A I N                                   ####
#################################################################################

EXIT_STATUS=0
isRestorOperation="false"
isDowngradeOperation="false"

if [ $# -gt 0 ]; then
   ## Parce arguments
   while getopts "df:hr" opt
   do
      case "$opt" in
         d) isDowngradeOperation="true"
         ;;
         f) distributionFileName="$OPTARG"
         ;;
         h)Usage
         ;;
         r) isRestorOperation="true"
         ;;
         *)Usage
         ;;
      esac
   done
fi

if [ "X${distributionFileName}" = "X" ]; then
   Usage
fi

if [ "X${isRestorOperation}" != "Xtrue" -a "X${isDowngradeOperation}" != "Xtrue" ]; then
   Usage
fi

if [ "X${isRestorOperation}" = "Xtrue" ]; then
   CreateRAMdisk "51200"
   LogTrace "Validate Restore File"
   ValidateRestoreFile "${distributionFileName}"
fi

if [ "X${isDowngradeOperation}" = "Xtrue" ]; then
   CreateRAMdisk "4096"
   LogTrace "Validate Upgrade/Downgrade File"
   ValidateUpgradeDowngradeFile "${distributionFileName}"
fi

EXIT_STATUS=${COMMAND_EXIT_STATUS}
LogTrace "Exit Status: $EXIT_STATUS"
UnmountRAMdisk
exit ${EXIT_STATUS}
