#!/bin/sh

ROOT_INSTALL_DIR="/home/embedded/library"
RETURN_STATUS=0
FACTORY_DEFAULTS=0
CLEAN_DONT_RESTART=0
RUN_TIME=`date`
TMP_LICENSE_KEY_DIR="/tmp/licenseKeyDir/$$"
REMOVE_LICENSE_KEYS=0
IPv6_GUI_ENABLED_FILE="/home/embedded/library/CoreService/dat/ipv6Enabled"
EXTERNAL_INTERFACE_PERSISTENT_SETTINGS="/etc/externalNetworkInterfaceSettings"

. ${ROOT_INSTALL_DIR}/AppManager/bin/include.sh

Usage()
{
   echo "Usage: $0 [-f [-c] -l ]

   -c: used with -f option to indicate we do not want to restart library firmware
   -f: reset to factory defaults (does no remove system log files or reset network) and restarts, restarts Local Console last
   -l: remove license keys
   -h: print usage message

   ";
   exit 1
}

KillI500Aplications()
{
   LogTrace "Kill I500 Applications"
   RunCommand "/etc/init.d/AppManager stop"
}

KillHttp()
{
   apachectl="/home/embedded/library/HTTPService/apache/bin/apachectl"
   if [ -f "${apachectl}" ]; then
      LogTrace "Stopping Application http"
      RunCommand "${apachectl} stop"
   fi
}

KillConsoleApp()
{
   set -- `ps -ef | grep ConsoleMgmtApplication | grep -v grep`
   processId=$2
   if [ "$processId" != "" ]; then
      LogTrace "Stopping ConsoleMgmtApplication"
      RunCommand "kill -9 $processId"
   else
      LogTrace "Application ConsoleMgmtApplication not running"
   fi
   ## Remove the log files
   RunCommand "rm -f ${ROOT_INSTALL_DIR}/ConsoleService/dat/console*"
}

ResetNetworkDefaluts()
{
   LogTrace "Resetting network configuration"
   RunCommand "cp -f /etc/network/interfaceEth2.dhcp /etc/network/interfaceEth2"
   RunCommand "cp -f /etc/hosts.orig /etc/hosts"
   RunCommand "cp -f /etc/hostname.orig /etc/hostname"
   RunCommand "cp -f /etc/resolv.conf.orig /etc/resolv.conf"
   RunCommand "cp -f /etc/extaccess.conf.orig /etc/extaccess.conf"
   if [ -f "/etc/disableStatelessAutoConfiguration" ]; then
      RunCommand "rm /etc/disableStatelessAutoConfiguration"
   fi
   if [ -f "${EXTERNAL_INTERFACE_PERSISTENT_SETTINGS}" ]; then
      RunCommand "rm ${EXTERNAL_INTERFACE_PERSISTENT_SETTINGS}"
   fi
}

ReinitializeLibrary()
{
   LogTrace "Removing persistent and application log files"
   if [ ! -f "$IPv6_GUI_ENABLED_FILE" ]; then
      IPv6_GUI_ENABLED_FILE=""
   fi
   if [ $REMOVE_LICENSE_KEYS -eq 1 ]; then
      RunCommand "rm ${ROOT_INSTALL_DIR}/CoreService/dat/[CcDIiLMNPRSTmsuvb]*"
   else
      RunCommand "rm ${ROOT_INSTALL_DIR}/CoreService/dat/[CcDIiMNPRSTmsuvb]*"
   fi
   RunCommand "rm ${ROOT_INSTALL_DIR}/CoreService/dat/uds_ekm*"
   RunCommand "rm ${ROOT_INSTALL_DIR}/CoreService/dat/ekm_data*"
   RunCommand "rm ${ROOT_INSTALL_DIR}/CoreService/dat/akm_servers*"
   if [ ! -d ${ROOT_INSTALL_DIR}/upload ]; then
      RunCommand "mkdir ${ROOT_INSTALL_DIR}/upload"
   fi

   ## Remove any autolevel images that exist
   RunCommand "rm -f ${ROOT_INSTALL_DIR}/UDS/autolevel/*"

   ## Remove core files
   RunCommand "rm -f ${ROOT_INSTALL_DIR}/AppManager/dat/*"

   ## If the CalibrationTool.exe exists in ~root, remove it
   RunCommand "rm -f /root/CalibrationTool.exe"

   ## Remove log files
   RunCommand "rm -f ${ROOT_INSTALL_DIR}/SMI-SService/logs/*"
   RunCommand "rm -f ${ROOT_INSTALL_DIR}/SNMPService/dat/*"
   RunCommand "rm -f ${ROOT_INSTALL_DIR}/HTTPService/apache/logs/*"
   RunCommand "rm -f ${ROOT_INSTALL_DIR}/SNMPService/var/net-snmp/snmpd.conf"
   RunCommand "rm -f ${ROOT_INSTALL_DIR}/SNMPService/share/snmp/snmpd.conf"
   if [ -f "${ROOT_INSTALL_DIR}/SMI-SService/repository.bk" ]; then
       RunCommand "rm -rf ${ROOT_INSTALL_DIR}/SMI-SService/repository"
       RunCommand "tar Pxzf ${ROOT_INSTALL_DIR}/SMI-SService/repository.bk"
   fi

   ## Disable NTP
   RunCommand "sh /home/embedded/library/AppManager/bin/ipv6Manager.sh -t 'disable'"

   ## Clear Kerberos Configuration
   RunCommand "rm -f /etc/krb5.conf"

   ## Disable SMI-S
   if [ -f "/etc/SMISenabled" ]; then
      RunCommand "rm /etc/SMISenabled"
   fi
}

CleanSystemLogs()
{
   LogTrace "Removing system log files"
   systemLogfiles=`ls /var/log/ | grep -v 'lost+found' | grep -v messages | grep -v 'appManager.log'`

   for i in $systemLogfiles
   do
      RunCommand "rm -f /var/log/$i"
   done
}

ResetSecurity()
{
   LogTrace "Reset Security"
   RunCommand "/usr/bin/eam -R"
}

ManufacturingTestProtection()
{
   sleep 3
   /sbin/ifconfig eth2 down
   sleep 2
   set -- `ps -ef | grep /usr/bin/eam | grep -v grep`
   processId=$2
   if [ "$processId" != "" ]; then
      kill -9 $processId
   fi
}

##########
## MAIN ##

## Get command line arguments
while getopts "cfhl" opt
do
   case "$opt" in
      c) CLEAN_DONT_RESTART=1
      ;;
      f) FACTORY_DEFAULTS=1
      ;;
      l) REMOVE_LICENSE_KEYS=1
      ;;
      h) Usage
      ;;
      ?) Usage
      ;;
   esac
done

if [ $FACTORY_DEFAULTS -eq 1 ]; then
   sleep 2
   RunCommand "cp /home/embedded/library/AppManager/images/factorDefaultsImage /dev/fb0"
   LogTrace "Executing Factory Defaults ClearToShip"
else
   RunCommand "cp /home/embedded/library/AppManager/images/clearToShipImage /dev/fb0"
   LogTrace "Executing Manafacturing ClearToShip"
fi

## ReInitialize DAB before S101Core goes down
LogTrace "Run clearToShipDABs"
RunCommand "sh ${ROOT_INSTALL_DIR}/clearToShipDABs"
LogTrace "Completed clearToShipDABs"

# echo > /etc/wwname
# echo > /etc/serial.num
cat /etc/lcm.inf.bak > /etc/lcm.inf

KillI500Aplications

if [ $FACTORY_DEFAULTS -eq 0 ]; then
   ManufacturingTestProtection
   ResetNetworkDefaluts
   ReinitializeLibrary
   CleanSystemLogs
   ResetSecurity
   RunCommand "/bin/sync"
   RunCommand "/sbin/halt"
else
   ReinitializeLibrary
   ResetSecurity
   if [ "X${IPv6_GUI_ENABLED_FILE}" != "X" ]; then
      LogTrace "Restore ${IPv6_GUI_ENABLED_FILE} state."
      RunCommand "touch ${IPv6_GUI_ENABLED_FILE}"
   fi
   if [ $CLEAN_DONT_RESTART -eq 0 ]; then
      CleanSystemLogs
      LogTrace "Restarting applications ..."
      LogTrace "Rebooting..."
      RunCommand "/bin/sync"
      RunCommand "/sbin/reboot &"
   else
      LogTrace "In Clean Dont Restart"
   fi
fi
LogTrace "Completed Clear to Ship"
exit $RETURN_STATUS
