# Script to clean up disk space of core files .tgz etc...
# This script will delete files that are used during a Firmware update.
# DO NOT RUN THIS SCRIPT IF A FIRMWARE UPDATE IS IN PROGRESS

APPROOTDIR=/home/embedded/library

if [ ! -d ${APPROOTDIR} ]
then
	echo "Application home directory not found"
	exit 0
else
	echo "Executing Disk Space clean"
   echo "The current Disk space usage is:"
   df -h
   echo "====================================================================="
fi

# Delete anything in the uploads directory
echo "Removing all files in ${APPROOTDIR}/upload..."
rm -rf ${APPROOTDIR}/upload/*

# Delete all previous upload attemps in /tmp
echo "Removing all upload files in /tmp..."
rm -rf /tmp/php*

# Remove the Extracter directory & contents if it exists
echo "Removing the Extractor ..."
rm -rf ${APPROOTDIR}/Extractor

# Remove the calibration tool
echo "Removing the CalibrationTool ..."
rm -f ${APPROOTDIR}/ConsoleService/bin/CalibrationTool.exe

# Remove any BSP scripts in the images directory
echo "Removing any BSP scripts ..."
rm -f ${APPROOTDIR}/images/*BSP*
# More BSP scripts, CR 62526
rm -f ${APPROOTDIR}/images/images/*BSP*

# Remove the OldSystemState file from the images directory
echo "Removing any OldSystemState files ..."
rm -f ${APPROOTDIR}/images/OldSystemState

# Delete all save config archive create attempts
echo "Removing all save config archive create attempts ..."
rm -f /tmp/saveRestore*

# Find *.tgz and deleted them if they are not in the images directory 
# (By deleteing *.tgz from upload we get rid of any failed attemps.)
echo "Removing all .tgz files that are not in images"
rm -f `find /home/embedded/library/ -name '*.tgz' | grep -v images`

# Find all *.dbg down the embedded tree and delete all of them.
echo "Removing all Debug executables files from library..."
rm -f `find ${APPROOTDIR} -name "*.dbg" -type f`

# Delete all core files
echo "Removing all core files from library..."
rm -f ${APPROOTDIR}/AppManager/dat/core*

# Delete all files under /var/tmp 
echo "Removing all files from /var/tmp..."
rm -rf /var/tmp/*

# Delete sled logs, drive dumps, plus drive brick firmware images 
echo "Removing Sled logs and Drive dumps"
rm -f ${APPROOTDIR}/UDS/drive_dumps/*
rm -f ${APPROOTDIR}/UDS/drive_firmware/*

# CR 62150 - CKB 
# Delete old, statically linked, CalibrationTool
echo "Removing CalibrationTool..."
rm -f /root/CalibrationTool

# CR 62150 - CKB 
# Delete gdb
echo "Removing gdb binaries..."
rm -f /usr/bin/gdb
rm -f /usr/bin/gdbserver

# CR 62150 - CKB 
# Remove snapshots from /tmp
echo "Removing snapshot files..."
rm -f /tmp/TicketSnapshot*
rm -f /tmp/snapshot*
rm -f /tmp/S101Core.SignalSnapShot*
rm -f /tmp/snopshat.tar

# CR 62150 - CKB 
# Remove unnecessary branding information/help text
# Remove help text and unneeded remote UI based on brand
echo "Removing unnecessary Remote UI and help text..."
case $ADICLIBRARY_BRAND in
	'adic')
		# Remove IBM Remote UI and unneeded help text
		rm -rf /home/embedded/library/HTTPService/apache/htdocs.ibm
		rm -rf /home/embedded/library/HTTPService/apache/htdocs.adic/RoboHelpDell
		rm -rf /home/embedded/library/HTTPService/apache/htdocs.adic/RoboHelpTandberg
		;;
	'dell')
		# Remove IBM Remote UI and unneeded help text
		rm -rf /home/embedded/library/HTTPService/apache/htdocs.ibm
		rm -rf /home/embedded/library/HTTPService/apache/htdocs.adic/RoboHelpTandberg
		;;
	'ibm')
		# Remove ADIC Remote UI
		rm -rf /home/embedded/library/HTTPService/apache/htdocs.adic
		;;
	'tandberg')
		# Remove IBM Remote UI and unneeded help text
		rm -rf /home/embedded/library/HTTPService/apache/htdocs.ibm
		rm -rf /home/embedded/library/HTTPService/apache/htdocs.adic/RoboHelpDell
		;;
	*)
		echo -e "\tUnable to match branding information! (${ADICLIBRARY_BRAND})"
		;;
esac

# CR 62150 - CKB 
# Remove deprecated Apache binaries
echo "Removing deprecated Apache binaries..."
rm -f /home/embedded/library/HTTPService/apache/bin/ab
rm -f /home/embedded/library/HTTPService/apache/bin/apxs
rm -f /home/embedded/library/HTTPService/apache/bin/checkgid
rm -f /home/embedded/library/HTTPService/apache/bin/gcache
rm -f /home/embedded/library/HTTPService/apache/bin/httpsd
rm -f /home/embedded/library/HTTPService/apache/bin/setssl.sh

# CR 62150 - CKB 
# Remove temporary email files
echo "Removing temporary email files..."
rm -f /tmp/.EM*

# CR 62150 - CKB 
# Remove statically linked insmod binary
echo "Removing statically linked insmod binary..."
rm -f /sbin/insmod.static

#Remove unused smis shared libraries
rm -f /home/embedded/library/SMI-SService/lib/requesthandlers/libowrequesthandlerbinary.so
rm -f /home/embedded/library/SMI-SService/lib/cmpiproviders/libcppindicationcommon.so
rm -f /home/embedded/library/SMI-SService/lib/cmpiproviders/libcmpifcbcomputersystem.so

#PCR 19029 - RUW
# Remove the old "CellPositionLog.csv", which is now called "SlotPositionLog.csv"
echo "Removing old CellPositionLog.csv (replaced with SlotPositionLog.csv)"
rm -f /home/embedded/library/CoreService/dat/CellPositionLog.csv

# Remove akm export & import files
rm -rf /tmp/exptmpdir
rm -rf /tmp/akmImportTmp
rm -f /tmp/*.pem
rm -f /tmp/*.tmp

echo "Disk cleanup complete"
echo "===================================================================="
echo "The new disk space usage is:"
df -h
echo ""

