/******************************************************************************
*
* Copyright 2004 by ADIC, Inc.  All rights reserved.
* No part of this work may be reproduced or transmitted in any
* form or by any means, electronic or mechanical, including
* photocopying and recording, or by any information storage
* or retrieval system, except as may be expressly permitted by
* the 17 U.S.C. section 101, et. seq., or in writing by
* ADIC, Inc.
*******************************************************************************/
/******************************************************************************
* Name: setupLibraryPartitions.js
* Purpose: Javascript functions creating, modifying, and deleting library partitions
*******************************************************************************/

// ===================================================================================================
/*
 * Set the selectedLibrary
 */
function setSelectedLibrary(libInfo)
{
   document.forms['selectionForm'].selectedLibrary.value = libInfo;
}

// ===================================================================================================
/*
 * Select/deselect all drives in the given frame based on
 * state of the given checkbox
 */
function selectAllDrives(frameName, checkbox)
{
   var inputTagList = window.frames[frameName].document.getElementsByTagName('input');
   for (i=0; i<inputTagList.length; i++)
   {
      if (inputTagList[i].type == "checkbox")
      {
         if (!inputTagList[i].disabled) inputTagList[i].checked = checkbox.checked;
      }
   }
}

// ===================================================================================================
/*
 * Create or update the partition defined in the given form and frame
 */
function submitLibraryInfo(formName, frameName, actionMessage)
{
   var driveCnt = 0;
   var inputTagList = window.frames[frameName].document.getElementsByTagName('input');
   var drives = new Array();
   for (i=0; i<inputTagList.length; i++)
   {
      if (inputTagList[i].type == "checkbox" && inputTagList[i].checked)
      {
         drives.push(inputTagList[i].value);
			driveCnt++;
      }
   }
   if (drives.length == 0)
   {
      alert("You must select at least one drive");
      return;
   }

   document.forms[formName].selectedDrives.value = drives.join(":");
   if (checkLibraryName(formName) && checkSlotCount(formName) && checkEKMCounts(formName, frameName, driveCnt) )
   {
         openProgressWindow(actionMessage, formName);
   }
}

// ===================================================================================================
/*
 * Check the library name in the given form, this will check
 * against currently known library names
 */
function checkLibraryName(formName)
{
   var f = document.forms[formName];
   if (f.libName.value == "")
   {
      alert("Please enter a library name");
      return false;
   }
   if (f.libNames.value != "")
   {
      libraryListValue = f.libNames.value;
      libraryListArray = libraryListValue.split(":");
      for (i=0; i<libraryListArray.length; i++)
      {
         if (libraryListArray[i] == f.libName.value)
         {
            alert("The library name you have entered [" + f.libName.value + "] already exists.");
            return false;
         }
      }
   }
   return true;
}

// ===================================================================================================
/*
 * Check the slot count in the given form
 */
function checkSlotCount(formName)
{
   var f = document.forms[formName];
   if (f.slotCount.value == "")
   {
      alert("Please enter the number of slots");
      return false;
   }
   if (eval(f.slotCount.value) > eval(f.maxSlotCount.value) )
   {
      alert("The number of slots selected must less than or equal to " + f.maxSlotCount.value);
      return false;
   }
   if (f.minSlotCount)
   {
      if (eval(f.slotCount.value) < eval(f.minSlotCount.value) )
      {
         alert("The number of slots selected cannot be less than the number of media: " + f.minSlotCount.value);
         return false;
      }
   }
   if (eval(f.slotCount.value) == 0 )
   {
      alert("The number of slots selected must be greater than zero.");
      return false;
   }
   return true;
}

// ===================================================================================================
/*
 * Check the library's EKM Licenses against the number trying to be used.
 */
function checkEKMCounts(formName, frameName, driveCnt)
{
    // Checking licenses applies only to modification of partitions.  This is because a new 
    // partition cannot have LME enabled without subsequent operator action.
      
    if(frameName != 'modifyDrivesList') 
        return true;  // Checking licenses applies only to modification of partitions

    var libEKM = window.frames[frameName].document.getElementById('ekmEnabled').value;
    if(libEKM == 0 || !libEKM) 
        return true; // If EKM is not enabled on this partition, then exit

	// Number of drives in the system that have encryption enabled
	var encryptedDriveCnt = window.frames[frameName].document.getElementById('ekmUsed').value;
	
	// Array of drives in the system that support encryption
	var libEKMDrives = window.frames[frameName].document.getElementById('ekmDrives').value;
	// Remove the last character if its a colon
	if(libEKMDrives[libEKMDrives.length - 1] == ":") 
        libEKMDrives = libEKMDrives.substring(0, libEKMDrives.length - 1);
    var libEKMDrivesArray = libEKMDrives.split(":");
    
    // Array of drives in the partition that were initially selected
	var libOrigCheckedDrives = window.frames[frameName].document.getElementById('origCheckedDrives').value;
	// Remove the last character if its a colon
	if(libOrigCheckedDrives[libOrigCheckedDrives.length - 1] == ":") 
        libOrigCheckedDrives = libOrigCheckedDrives.substring(0, libOrigCheckedDrives.length - 1);
    var origCheckedDrivesArray = libOrigCheckedDrives.split(":");
    
    // Determine how many of the originally selected drives supported
    // encryption
    var originalLibEncryptedCount = 0;
    for(i = 0; i < origCheckedDrivesArray.length; i++)
    {
        driveGuid = origCheckedDrivesArray[i];
        if(libEKMDrivesArray.indexOf(driveGuid) >= 0) 
            originalLibEncryptedCount++;
    }

    //alert( encryptedDriveCnt +" | "+ originalLibEncryptedCount );
    // Calculate the number of encrypted drives in use in other partitions
    var encryptedCountInOtherPartitions = encryptedDriveCnt - originalLibEncryptedCount;
    
    // Count of the number of encryptable drives selected
    var iEKMDriveSelCount=0;

    // Iterate through the checkboxes that represent the drives
    var inputTagList = window.frames[frameName].document.getElementsByTagName('input');
    for (i=0; i<inputTagList.length; i++)
    {
      if (inputTagList[i].type == "checkbox" && inputTagList[i].checked)
      {
         // If the drive represented by this checkbox is in the encrypted drives
         // list, increment the selection count.
         var driveGuid = inputTagList[i].value;
         if(libEKMDrivesArray.indexOf(driveGuid) >= 0) 
             iEKMDriveSelCount++;
      }
    }

    // Calculate what the new total encrypted drives would be if we went ahead
    // with this operation.  If its greater than the number of licenses, then
    // display an error.
    var newPotentialTotalEncryptedDriveCount = iEKMDriveSelCount + encryptedCountInOtherPartitions;
 
    // Get the number of installed licenses
    var f 	   = document.forms[formName];
	var ekmMax = f.ekmMax.value;
	
	/*alert(
	    "EKM Enabled: " + libEKM + "\n" + 
	    "Total encrypted drives in library: " + encryptedDriveCnt + "\n" +
	    "Drives that support encryption: " + libEKMDrives + "\n" +
	    "Original Checked Drives: " + libOrigCheckedDrives + "\n" +
	    "Number of encryptable drives checked originally: " + originalLibEncryptedCount + "\n" +
	    "Number of encryptable drives in OTHER partitions: " + encryptedCountInOtherPartitions + "\n" +
	    "Encryptable drives selected now: " + iEKMDriveSelCount + "\n" +
	    "Potential total encrypted count: " + newPotentialTotalEncryptedDriveCount + "\n" +
	    "Total EKM licenses: " + ekmMax);
    */

    if(newPotentialTotalEncryptedDriveCount > ekmMax)
	{
	   //alert("There "+(ekmLeft==1?'is':"are")+" "+(ekmLeft==0?'no':ekmLeft)+" EKM "+(ekmLeft==1?'license':"licenses")+" available. Please reconfigure the partition so you do not exceed your EKM license count.");
	   alert("There are not enough EKM licenses available to support this configuration.\nPlease reconfigure the partition so you do not exceed your EKM license count.");
	   return false;
	}
    return true;	
}

// ===================================================================================================
/*
 * Check the available resources for create and make the div visible
 */
function createLib(currentTag)
{
   if (eval(document.forms['selectionForm'].unassignedDriveCount.value) == 0)
   {
      alert("No unassigned drives are available for partition create.");
      return;
   }
   if (eval(document.forms['selectionForm'].unassignedSlotCount.value) == 0)
   {
      alert("No unassigned slots are available for partition create.");
      return;
   }
   document.getElementById('createDriveList').src = "/setupLibDriveList.htm";
   switchDiv(currentTag, 'simpleCreateLib');
}

// ===================================================================================================
/*
 * Update the modify form with the currently selected library information
 * and make the div visible
 */
function modifyLib()
{
   var libInfo = document.forms['selectionForm'].selectedLibrary.value;
   if (libInfo == "")
   {
      alert("No libraries to modify!");
      return;
   }

   var libInfoList = libInfo.split(":");
   var modForm = document.forms['modifyForm'];
   var currentLibraryName = libInfoList[0];

   // See if there are any tapes in the drives that belong to the partition being modified.
   var libDriveList = document.forms['selectionForm'].libDrives.value.split(":");
   for( i=0; i < libDriveList.length; i++ )
   {
     if( libDriveList[i] == libInfoList[1] )
     {
       alert("There is media present in one of the drives for this partition.\nPlease remove the media and try again.");
       return;
     }
   }

   modForm.libName.value = currentLibraryName;
   document.getElementById('modLibNameHeader').innerHTML = currentLibraryName;
   document.getElementById('modLibName').innerHTML = currentLibraryName;

   // Remove the current name from the list of all library names
   var libNames = document.forms['selectionForm'].libNames.value;
   modForm.libNames.value = "";
   if (libNames != "")
   {
      var modifyLibNamesList = new Array();
      var allLibNamesList = libNames.split(":");
      for (i = 0; i < allLibNamesList.length; i++)
      {
         if (allLibNamesList[i] != currentLibraryName) modifyLibNamesList.push(allLibNamesList[i]);
      }
      modForm.libNames.value = modifyLibNamesList.join(":");
   }

   modForm.libGUID.value = libInfoList[1];
   for (i = 0; i < modForm.emulationType.length; i++)
   {
     if (modForm.emulationType.options[i].value == libInfoList[2])
     {
        modForm.emulationType.options[i].selected = true;
        document.getElementById('modEmulationType').innerHTML =
              modForm.emulationType.options[i].text;
   break;
     }
   }
   for (i = 0; i < modForm.barcodePolicy.length; i++)
   {
     if (modForm.barcodePolicy.options[i].value == libInfoList[3])
     {
        modForm.barcodePolicy.options[i].selected = true;
        document.getElementById('modBarcodePolicy').innerHTML =
              modForm.barcodePolicy.options[i].text;
   break;
     }
   }

   var slotCount = libInfoList[5];
   modForm.slotCount.value = slotCount;
   document.getElementById('modSlotCount').innerHTML = slotCount;
   var maxSlots = eval(modForm.slotCount.value) + eval(document.forms['selectionForm'].unassignedSlotCount.value);
   document.getElementById('modAvailableSlotCount').innerHTML = maxSlots;
   modForm.maxSlotCount.value = maxSlots;
   modForm.driveSelect.checked = false;
   document.getElementById('modifyDrivesList').src =
        "/setupLibModDriveList.htm?libGUID=" + libInfoList[1];
   modForm.minSlotCount.value = libInfoList[6];
   switchDiv('selectOperation', 'modifyLib');
}

// ===================================================================================================
/*
 * Delete the currently selected library
 */
function deleteLib()
{
	var libInfo = document.forms['selectionForm'].selectedLibrary.value;
	if (libInfo == "")
	{
		alert("No Libraries exist to delete.");
		return;
	}
	var libInfoList = libInfo.split(":");
	var libName = libInfoList[0];
	var libGUID = libInfoList[1];
	var libMediaCount = libInfoList[6];
	
	// See if there are any tapes in the drives that belong to the partition being modifyed.
	var libDriveList = document.forms['selectionForm'].libDrives.value.split(":");
	var bDriveIsLoaded = false;
	for( i=0; i < libDriveList.length; i++ )
	{
		if( libDriveList[i] == libInfoList[1] )
		{
			bDriveIsLoaded = true;
			break;
		}
	}

	if(bDriveIsLoaded)
	{
			var sMsg =	"There is media present in one or more of the drives in this partition.\n" +
							"When you delete a partition, all drives, media and slots will be unassigned.\n" +
							"You will need to create a new partition to use these resources.\n" +
							"Are you sure you want to delete library [ " + libName + " ]?";
			if( confirm( sMsg ) )
         {
			   openProgressWindow("Deleting Logical Library", 'selectionForm');
         }
	}
	else	if( libMediaCount > 0 )
	{
		var message = libMediaCount + " media " + (libMediaCount>1 ? "are" : "is") + " defined for the selected partition." +
							"It is recommended all media\n" +
							"from the library partition and associated drives be removed before deletion.\n" +
							"Are you sure you want to delete library [ " + libName + " ]?";
		if( confirm(message) )
			openProgressWindow("Deleting Logical Library", 'selectionForm');
	}
	else
	{
		if (confirm("Are you sure you want to delete library [ " + libName + " ]?"))
			openProgressWindow("Deleting Logical Library", 'selectionForm');
	}
}
// ===================================================================================================
