/****************************************************************************** 
 * Copyright 1991-2008 by Quantum, Inc.  All rights reserved.
 * No part of this work may be reproduced or transmitted in any
 * form or by any means, electronic or mechanical, including
 * photocopying and recording, or by any information storage
 * or retrieval system, except as may be expressly permitted by
 * the 17 U.S.C. section 101, et. seq., or in writing by
 * Quantum, Inc.
 *******************************************************************************/
/****************************************************************************** 
 * Name:  setupNetwork.js
 * Purpose: Javascript functions for setting up the system network 
 * $Id: setupNetwork.js 1.24 2007/08/15 16:47:59MDT Bill Gray (bgray) dev  $
 * $ADICCopyright: Copyright 2007 Quantum Corp. $
 *******************************************************************************/
var focusField;
var okToApply = true;

/****************************************************************
 * Apply the changes to the network setup
 ****************************************************************/
function applyChanges() 
{
   var sMsg =  "Caution: Changing the network settings may interrupt the library connection and library communication. " +
               "Your current browser session may become invalid, requiring your browser to be closed and then restarted " +
               "to reconnect to the library with your new configuration settings.";
   // First see if there were any changes.
   var f = document.forms[0];

   var bChange = false;
   
   if(   f.currentDHCP.value           != f.useDHCP.checked ||
         f.currentHostname.value       != f.hostname.value ||
         f.currentIPAddr.value         != f.ipAddress.value ||
         f.currentSubnetMask.value     != f.netmask.value ||
         f.currentGatewayAddr.value    != f.gateway.value )
      bChange = true;

   // If IPv6 is not enabled we need to test for changes in the DNS
   if(   !IPv6Enabled && 
         (f.currentDnsPrimary.value    != f.dnsPrimary.value ||
         f.currentDnsSecondary.value   != f.dnsSecondary.value ))
      bChange = true;
   
   if( ! bChange )
   {
      alert("No Changes were made.");
      return;
   }
   
   if( verifyDomainHostname( f.hostname ) )
   {
      if( f.useDHCP.checked == false )
      {
      // Validate the IP fields before progressing
      if( verifyIP( f.ipAddress,0 ) )
        if( verifyIP( f.gateway,0 ) )
          if( verifySubnetMask( f.netmask,0, 0 ) )
          {
               if( okToApply == false )
               {
                  alert("The data in one or more fields is incorrect - please re-enter it");
                  return false;
               }
               if( confirm(sMsg) )
                  openProgressWindow('Configuring Network With Static Address\'s.', '0');
          }
      }
      else if( confirm(sMsg) )
       openProgressWindow('Configuring Network with DHCP.', '0');
   }
}

/****************************************************************
 * Check for a Valid Domain Host name in the INPUT field passed.
 * MAX length of 63
 * Only A-Z, a-z, 0-9, "-"
 ****************************************************************/
function checkDomainHostname( aField )
{
   //var pattern = /[^a-zA-Z0-9-]/;   // To spec
   var pattern = /[^a-z0-9-]/;   // To match the local
   var pattern2 = /^[0-9]/;
   var pattern3    = /^[-]/;
   var string = aField.value;
   
   if( (string.match(pattern)) || (string.match(pattern2)) || (string.match(pattern3)) )
   {
   /*##############################################################
    # NOTE: !?! You can not POP up a dialog here or the OnChange
    # or OnBlur will also get ran.
    ##############################################################*/
      //alert("Only numbers, and periods are allowed in the IP Address string.");
      window.status = "Only lower case letters, numbers, and '-' are allowed in the host name string.  It must start with a letter!"
      
      // remove the invalid character and put focus on the input field
      string = string.slice(0,-1);
      aField.value = string;
      beep();
      return false;
   }
   else
    window.status = "";  // Clear any OLD errors

   return true;
}

/****************************************************************
 * Verisfy the Domain Host name in the INPUT field passed.
 * MAX length of 63
 * Only A-Z, a-z, 0-9, "-"
 ****************************************************************/
function verifyDomainHostname( aField )
{
   //var pattern = /[^a-zA-Z0-9-]/;   // To spec
   var pattern    = /[^a-z0-9-]/;   // To match the local
   var pattern2    = /^[0-9]/;
   var pattern3    = /^[-]/;
   var string       = aField.value;
   
   if( (string.match(pattern)) || (string.match(pattern2)) || (string.match(pattern3)) )
   {
   /*##############################################################
    # NOTE: !?! You can not POP up a dialog here or the OnChange
    # or OnBlur will also get ran.
    ##############################################################*/
      //alert("Only numbers, and periods are allowed in the IP Address string.");
      msg = "Only lower case letters, numbers, and '-' are allowed in the host name string. It must start with a letter!";
      alert( msg );
      
      // remove the invalid character and put focus on the input field
//      string = string.slice(0,-1);
//      aField.value = string;
//      beep();
      /********************************************************
       * Note: the setting of the focus DOES NOT WORK like
       * aField.focus();
       * You must call the focus function after the KEY
       * that made the user leave the field gets processed, 
       * so you MUST use a timer.
       ********************************************************/
      focusField = aField;
      setTimeout( setTheFocus, 10 );
      okToApply = false;
      return false;
   }
   else
      okToApply = true;
      
   return true;
}

/****************************************************************
 * Check for a Valid IP Address in the INPUT field passed.
 ****************************************************************/
function checkIPAddrChar( aField )
{
   var pattern = /[^\.0-9_]/;
   var string = aField.value;
   
   if( string.match(pattern) )
   {
   /*##############################################################
    # NOTE: !?! You can not POP up a dialog here or the OnChange
    # or OnBlur will also get ran.
    ##############################################################*/
      //alert("Only numbers, and periods are allowed in the IP Address string.");
      window.status = "Only numbers, and periods are allowed in the IP Address string!"
      
      // remove the invalid character and put focus on the input field
      string = string.slice(0,-1);
      aField.value = string;
      beep();
      return false;
   }
   else
    window.status = "";  // Clear any OLD errors

   return true;
}

/****************************************************************
 * Verify that the data in the input field passed is a valid 
 * IP subnet mask address.
 * If aBlankOK is true then Blank IPs are OK
 * aField    - The Subnet mask field
 * aBlankOK  - Is a blank Subnet mask OK?
 * aUpdateGW - If true the gateway filed may be changed to a valid
 *             one.
 ****************************************************************/
function verifySubnetMask( aField, aBlankOK, aUpdateGW ) 
{
   var IPvalue = aField.value;
   
   errorString = "";
   theName = "Subnet Mask";
   
   var ipPattern = /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/;
   if (IPvalue == "0.0.0.0" && aBlankOK != "1")
      errorString = errorString + theName + ': '+IPvalue+' is a special IP address and cannot be used here.';
   if (IPvalue == "255.255.255.255")
      errorString = errorString + theName + ': '+IPvalue+' is a special IP address and cannot be used here.';
   if( IPvalue == ""  && aBlankOK != "1" )
      errorString = errorString + theName + ' BLANK cannot be used here.';
   
   if( errorString == "" )
   {
      var ipArray = IPvalue.match(ipPattern); 
      if (ipArray == null)
         errorString = errorString + theName + ': '+IPvalue+' is not a valid IP address.';
      else 
      {
         var bFound0=false;
         for (i = 1; i < 5; i++) 
         {
            thisSegment = ipArray[i];
            if (thisSegment > 255) 
            {
               errorString = errorString + theName + ': '+IPvalue+' is not a valid IP address.';
               break;
            }
            if (thisSegment < 255 && thisSegment != 0 )
            {
               if( bFound0 )
               {
                  errorString = errorString + theName + ': '+IPvalue+' is not a valid subnet mask.';
                  break;
               }
               else
                  bFound0 = true;
            }
         }
      }
   }
   
   if( errorString != "" )
   {
      // Display the error and return the focus to the field.
      alert (errorString);
      window.status = errorString;
      
      /********************************************************
      * Note: the setting of the focus DOES NOT WORK like
      * aField.focus();
      * You must call the focus function after the KEY
      * that made the user leave the field gets processed, 
      * so you MUST use a timer.
      ********************************************************/
      okToApply = false;
      focusField = aField;
      setTimeout( setTheFocus, 10 );
      return false;
   }
   else
      okToApply = true;

   if( aUpdateGW )
   {
      // If the gateway is invalid or emptry - fill it out with our best guess.
      var sGateway   = document.forms[0].gateway.value;
      var sIp        = document.forms[0].ipAddress.value;
      var sIpOctet   = sIp.match(ipPattern);
      var gateway;
      var gw;
      var addr         = OctetToInt(sIpOctet, 1);
      var mask         = OctetToInt(IPvalue.match(ipPattern), 1);
   
      if(sGateway != "")
      {
         gateway  = sGateway.match(ipPattern);
         gw         = OctetToInt(gateway, 1);
         if ((parseInt(addr) & parseInt(mask)) == (parseInt(gw) & parseInt(mask)))
         {
            return; // current gateway is ok
         }
      }
   
      gw = parseInt(addr) & parseInt(mask);
      var gateway = new Array;
      // Note: Becasue all but the last octet can be negative when converting from octets to int.
      // We need to & them with 255
      gateway[4] = (parseInt(gw) & 0x000000FF);
      gateway[3] = (((parseInt(gw) & 0x0000FF00) >> 8)  & 0xff);
      gateway[2] = (((parseInt(gw) & 0x00FF0000) >> 16) & 0xff) ;
      gateway[1] = (((parseInt(gw) & 0xFF000000) >> 24) & 0xff);
      if (gateway[4] < 255) 
         gateway[4]++;
      thegateway = gateway[1] + "." + gateway[2] + "." + gateway[3] + "." + gateway[4];
      document.forms[0].gateway.value = thegateway;
   }
   return true;
}

/****************************************************************
 * You miust set the var focusFiled before setting the timer to 
 * this function.
 ****************************************************************/
function setTheFocus()
{
   focusField.focus();
   focusField.select();
   
   return true;
}


