/****************************************************************************** 
 * Copyright 1991-2008 by Quantum Corporation.  All rights reserved.
 * No part of this work may be reproduced or transmitted in any
 * form or by any means, electronic or mechanical, including
 * photocopying and recording, or by any information storage
 * or retrieval system, except as may be expressly permitted by
 * the 17 U.S.C. section 101, et. seq., or in writing by
 * Quantum Corporation.
 *******************************************************************************/
/****************************************************************************** 
* Name:  setupPortConfig.js
* Purpose: Javascript functions for setting up FC ports on the I/O Blade
* $Id: setupPortConfig.js 1.3 2006/12/20 09:47:40MST Bill Gray (bgray) dev  $
*******************************************************************************/


// This function gets all of the info for the defined FC Hosts
// With this information the host info is populated on configure
// and passed to the rescan and reset operations
function getFibreChannelInfo(task) {
   var headerform = document.forms[0];
   var configureform = document.forms[1];
   var currentValues;
   var name;
   var hostType;
   var loopID;
   var speed;
   var frameSize;
   var portMode;
   var connType;
   
   // check to see if 0 or 1 channels exist
   // Goofy fix for radio buttons that only have one element in the array.
   // input type radio will return undefined if only one element in 
   // the array.
   // below: no channels in the list

   if (document.forms[0].currentValues.value == "") {
      alert("Please select a fibre channel port.");
      return;
   }

   if (task == "Configure") {
      currentValues = document.forms[0].currentValues.value;
      currentValues = currentValues.split(":");
      name          = currentValues[2];
      hostType      = currentValues[3];
      loopID        = currentValues[4];
      speed         = currentValues[5];
      frameSize     = currentValues[6];
      portMode      = currentValues[7];
      connType      = currentValues[8];
      pendingHost   = currentValues[9];
      pendingLoop   = currentValues[10];
      pendingSpeed  = currentValues[11];
      pendingFrame  = currentValues[12];
      pendingPort   = currentValues[13];
      pendingConn   = currentValues[14];

      document.getElementById('currentName').innerHTML = "<b>" + name + "</b>";

      if (loopID == -1)
         document.getElementById('currentLoopID').innerHTML = "Soft";
      else if (loopID == 126)
         document.getElementById('currentLoopID').innerHTML = "None";
      else 
         document.getElementById('currentLoopID').innerHTML = loopID;

      selectLoopID(pendingLoop);

      if (speed > 0)
         speed = speed + " Gb/s";
      if (pendingSpeed > 0)
         pendingSpeed = pendingSpeed + " Gb/s";

      document.getElementById('currentSpeed').innerHTML = speed;
      selectSpeed(pendingSpeed);

      document.getElementById('currentFrameSize').innerHTML = frameSize;
      selectFrameSize(pendingFrame);

      if (portMode == "publicTargetOnly")
         document.getElementById('currentPortMode').innerHTML = "Target";
      else
         document.getElementById('currentPortMode').innerHTML = "Target & Initiator";

      if (pendingPort == "publicTargetOnly")
         selectPortMode("Target");
      else
         selectPortMode("Target & Initiator");

      if (connType == "loopOnly")
      {
         document.getElementById('currentConnType').innerHTML = "Loop";
      }
      else if (connType == "pointToPointOnly")
      {
         document.getElementById('currentConnType').innerHTML = "Point to Point";
         document.getElementById('currentLoopID').innerHTML = "None";
      }
      else if (connType == "loopPreferred")
      {
         document.getElementById('currentConnType').innerHTML = "Loop Preferred";
      }
      else if (connType == "pointToPointPreferred")
      {
         document.getElementById('currentConnType').innerHTML = "Point to Point Preferred";
         document.getElementById('currentLoopID').innerHTML = "None";
      }

      if (pendingConn == "loopOnly")
      {
         selectConnType("Loop");
         document.forms[1].loop.disabled = false;
      }
      else if (pendingConn == "pointToPointOnly")
      {
         selectConnType("Point to Point");
         document.forms[1].loop.options[0].selected = true;
         document.forms[1].loop.disabled = true;
      }
      else if (pendingConn == "loopPreferred")
      {
         selectConnType("Loop Preferred");
         document.forms[1].loop.disabled = false;
      }
      else if (connType == "pointToPointPreferred")
      {
         selectConnType("Point to Point Preferred");
         document.forms[1].loop.options[0].selected = true;
         document.forms[1].loop.disabled = true;
      }

      document.forms[1].opType.value = document.forms[0].opType.value;
      document.forms[1].currentValues.value = document.forms[0].currentValues.value;
      switchDiv('header', 'config'); // goto configure page
   }
   else if (task == "Rescan") {
      if (confirm("Rescanning will search for new devices.  Rescan now?")) {
         openProgressWindow(task, '0');
      }
   }
   else if (task == "Reset") {
      if (confirm("Resetting may disrupt other user activity. Reset now?")) { 
         openProgressWindow(task, '0');
      }
   }
}

function selectLoopID(loopID) {
   var f = document.forms[1];
   // for Soft
   if (loopID == -1)
      f.loop.options[1].selected = true;
   for (i=1; i<f.loop.options.length; i++) {
      if (f.loop.options[i].value == loopID) {
         f.loop.options[i].selected = true;
         break;
      }
   }
}

function selectSpeed(pSpeed) {
   var f = document.forms[1];
   for (i=0; i<f.speed.options.length; i++) {
      if (f.speed.options[i].value == pSpeed) {
         f.speed.options[i].selected = true;
         break;
      }
   }
}

function selectFrameSize(frameSize) {
   var f = document.forms[1];
   for (i=0; i<f.frame.options.length; i++) {
      if (f.frame.options[i].value == frameSize) {
         f.frame.options[i].selected = true;
         break;
      }
   }
}

function selectPortMode(portMode) {
   var f = document.forms[1];
   for (i=0; i<f.port.options.length; i++) {
      if (f.port.options[i].value == portMode) {
         f.port.options[i].selected = true;
         break;
      }
   }
}

function selectConnType(connType) {
   var f = document.forms[1];
   for (i=0; i<f.conn.options.length; i++) {
      if (f.conn.options[i].value == connType) {
         f.conn.options[i].selected = true;
         break;
      }
   }
}

function confirmReset()
{
   changeObjectVisibility('checkForReset','visible');
}

function saveAndReset()
{
   changeObjectVisibility('checkForReset','hidden');
   document.forms[1].reset.value = "true";
   openProgressWindow('Configuring Fibre Channel', 1);
}

function saveWithoutReset()
{
   changeObjectVisibility('checkForReset','hidden');
   openProgressWindow('Configuring Fibre Channel', 1);
}

function maybeDisableLoopID() {
   var f = document.forms[1];
   if (f.conn.value == 'Loop')
   {
      if (f.loop.options[0].selected)
         f.loop.options[1].selected = true;
      f.loop.disabled = false;
   }
   if (f.conn.value == 'Loop Preferred')
   {
      if (f.loop.options[0].selected)
         f.loop.options[1].selected = true;
      f.loop.disabled = false;
   }
   if (f.conn.value == 'Point to Point')
   {
      f.loop.options[0].selected = true;
      f.loop.disabled = true;
   }
   if (f.conn.value == 'Point to Point Preferred')
   {
      f.loop.options[0].selected = true;
      f.loop.disabled = true;
   }
}

function maybeSetSoft() {
   var f = document.forms[1];
   if (f.loop.options[0].selected)
      f.loop.options[1].selected = true;
}

