<?
/*
 * Quantum Confidential
 *
 * Automated Tape Library Software Source
 * (C) Copyright Quantum 2009. All Rights Reserved.
 * Non-published
 * 
 * $Id: $
 */
class BladeInfoXML extends InfoXML
{
   var $list = array(); // List of Blade objects
   var $user = null;
   var $EX_bladeList = null;

   function BladeInfoXML( $user = null )
   {
      parent::InfoXML();
      $this->user = $user;
      $this->getData();
   }

   function getData()
   {
      $start = gettimeofday();
      $this->EX_bladeList = @get_blades( $this->user );
      @$this->checkError( $this->EX_bladeList, "get_blades", $start );
      if( isset( $this->EX_bladeList ) )
      {
         foreach( $this->EX_bladeList as $item )
         {
            $blade = new Blade( $this->user, $item );
            $this->list[] = $blade;
         }
      }
      else
      {
         $blade = new Blade( $this->user, null );
         $this->list[] = $blade;
      }
   }

   function toXML()
   {
      $count = count( $this->list );
      $out = "";
      if( $count > 0 )
      {
         $out = $this->tab . "<IOBladeInfo>\n";
         for( $i = 0; $i < $count; $i++ )
            $out .= $this->list[$i]->toXML();
         $out .= $this->tab . "</IOBladeInfo>\n";
      }
      return $out;
   }
}

class Blade extends InfoXML
{
   var $user = null;
   var $blade = null;
   var $location;
   var $type;
   var $firmwareVersion;
   var $serialNumber;
   var $status;
   var $HPF;
   var $bladeStatus = array( "Unknown", "Not Ready", "Booting", "Auto Leveling", "Auto Leveling Complete", "Auto Level Failed", "Ready", "Powered Down" );
   var $list = array();
   var $EX_portList = null;
   var $EX_hpf = null;

   function Blade( $user = null, $blade = null )
   {
      parent::InfoXML();
      $this->blade = $blade;
      $this->user = $user;
      $this->initializeVariables();
      $this->getData();
   }

   function initializeVariables()
   {
      $this->location = $this->INTERFACE_ERROR_STRING;
      $this->type = $this->INTERFACE_ERROR_STRING;
      $this->firmwareVersion = $this->INTERFACE_ERROR_STRING;
      $this->serialNumber = $this->INTERFACE_ERROR_STRING;
      $this->status = $this->INTERFACE_ERROR_STRING;
      $this->HPF = $this->INTERFACE_ERROR_STRING;
   }

   function getData()
   {
      if( isset( $this->blade ) )
      {
         $this->location = $this->blade->location;
         $this->type = "FC IOB 7404";
         $this->firmwareVersion = $this->blade->fw_ver;
         $this->serialNumber = trim( $this->blade->serial );
         $this->status = $this->bladeStatus[ $this->blade->status % 8 ];

         // H P F
         $start = gettimeofday();
         $this->EX_hpf = @get_host_port_failover($this->user, $this->blade->id );
         @$this->checkError( $this->EX_hpf, "get_host_port_failover", $start );
         if( isset($this->EX_hpf) )
         {
            if( $this->EX_hpf->enabled == 1 )
               $this->HPF = "Enabled";
            else
               $this->HPF = "Disabled";
         }

         // Get Port Information
         $start = gettimeofday();
         $this->EX_portList = @get_blade_ports($this->user, $this->blade->id );
         @$this->checkError( $this->EX_portList, "get_blade_ports", $start );
         if( isset( $this->EX_portList ) )
         {
            foreach( $this->EX_portList as $item )
            {
               $port = new Port( $item );
               $this->list[] = $port;
            }
         }
         else
         {
            $port = new Port( null );
            $this->list[] = $port;
         }
      }
   }

   function toXML()
   {
      $out = $this->tab1 . "<Blade>\n";
      $this->createTag( $out, $this->tab2, "Location", $this->location );
      $this->createTag( $out, $this->tab2, "Type", $this->type );
      $this->createTag( $out, $this->tab2, "FirmwareVersion", $this->firmwareVersion );
      $this->createTag( $out, $this->tab2, "SerialNumber", $this->serialNumber );
      $this->createTag( $out, $this->tab2, "Status", $this->status );
      $this->createTag( $out, $this->tab2, "HPF", $this->HPF );
      $count = count( $this->list );
      if( $count > 0 )
      {
         $out .= $this->tab2 . "<PortInfo>\n";
         for( $i = 0; $i < $count; $i++ )
            $out .= $this->list[$i]->toXML();
         $out .= $this->tab2 . "</PortInfo>\n";
      }

      $out .= $this->tab1 . "</Blade>\n";
      return $out;
   }
}

class Port extends InfoXML
{
   var $number;
   var $wwpn;
   var $type;
   var $status;
   var $connectionType;
   var $connectionSpeed;
   var $loopID;

   var $port = null;
   var $portStatus = array( "Configure Wait", "Loop Initialize", "Login", "Ready", "Lost Sync", "Error", "Reinitialize", "Non Participate", "Failed" );
   var $connectTypeEnum = array('Loop','Point to Point', 'Loop Preferred');
   var $connectSpeedEnum = array( 0 => "Auto", 1 => "1Gb/s", 2 => "2Gb/s", 4 => "4Gb/s" );
   
   function Port( $port = null )
   {
      parent::InfoXML();
      $this->port = $port;
      $this->initializeVariables();
      $this->getData();
   }

   function initializeVariables()
   {
      $this->number = $this->INTERFACE_ERROR_INT;
      $this->wwpn = $this->INTERFACE_ERROR_STRING;
      $this->type = $this->INTERFACE_ERROR_STRING;
      $this->status = $this->INTERFACE_ERROR_STRING;
      $this->connectionType = $this->INTERFACE_ERROR_STRING;
      $this->connectionSpeed = $this->INTERFACE_ERROR_STRING;
      $this->loopID = $this->INTERFACE_ERROR_INT;
   }

   function getData()
   {
      if( isset( $this->port ) )
      {
         $this->number = $this->port->portNum;
         $this->wwpn = $this->port->wwnn;
         if( $this->port->portNum <= 2 )
            $this->type = "Target";
         else
            $this->type = "Initiator";
         $this->status = $this->portStatus[ $this->port->status % 9 ];
         $this->connectionType = $this->connectTypeEnum[ $this->port->connection % 3 ];
         $this->connectionSpeed = $this->connectSpeedEnum[ $this->port->speed % 5 ];
         $this->loopID = ( ($this->port->loopId >= 0 && $this->port->loopId < 126) && $this->port->loopIdMode == 1) ? $this->port->loopId : "Auto";
      }
   }

   function toXML()
   {
      $out = $this->tab3 . "<Port>\n";
      $this->createTag( $out, $this->tab4, "Number", $this->number );
      $this->createTag( $out, $this->tab4, "WWPN", $this->wwpn );
      $this->createTag( $out, $this->tab4, "Type", $this->type );
      $this->createTag( $out, $this->tab4, "Status", $this->status );
      $this->createTag( $out, $this->tab4, "ConnectionType", $this->connectionType );
      $this->createTag( $out, $this->tab4, "ConnectionSpeed", $this->connectionSpeed );
      $this->createTag( $out, $this->tab4, "LoopID", $this->loopID );
      $out .= $this->tab3 . "</Port>\n";
      return $out;
   }
}
?>
