<?
/*
 * Quantum Confidential
 *
 * Automated Tape Library Software Source
 * (C) Copyright Quantum 2009. All Rights Reserved.
 * Non-published
 * 
 * $Id: $
 */
class DriveInfoXML extends InfoXML
{
   var $list = array(); // List of Drive objects
   var $user = null;
   var $EX_driveList = null;
   var $EX_driveList1 = null;

   function DriveInfoXML( $user = null )
   {
      parent::InfoXML();
      $this->user = $user;
      $this->getData();
   }

   function getData()
   {
      $sortCriteria = new SortCriteria();
      $sortCriteria->sortColumn = 3;
      $sortCriteria->ascending = 1;
      $sortCriteria->filterPattern = "*";
      $start = gettimeofday();
      $this->EX_driveList = @get_all_drives( $this->user, $sortCriteria );
      @$this->checkError( $this->EX_driveList, "get_all_drives", $start );

      if( isset( $this->EX_driveList ) )
      {
         foreach( $this->EX_driveList as $item )
         {
            $drive = new Drive( $item );
            $this->list[] = $drive;
         }
      }
      else
      {
         $drive = new Drive( null );
         $this->list[] = $drive;
      }
   }

   function toXML()
   {
      $count = count( $this->list );
      if( $count > 0 )
      {
         $out = $this->tab . "<DriveInfo>\n";
         for( $i = 0; $i < $count; $i++ )
         {
            $out .= $this->list[$i]->toXML();
         }
         $out .= $this->tab . "</DriveInfo>\n";
         return $out;
      }
   }
}

class Drive extends InfoXML
{
   var $drive;
   var $owner;
   var $location;
   var $serialNumber;
   var $reportedSerialNumber;
   var $status;
   var $encryption;
   var $mountStatus;
   var $vendor;
   var $model;
   var $firmwareVersion;
   var $type;
   var $interfaceType;
   var $interfaceAddress;
   var $requestedConnectionType;
   var $actualConnectionType;
   var $requestedConnectionSpeed;
   var $actualConnectionSpeed;
   var $loopID;

   var $topology = array( "Unknown", "Auto", "Loop", "Point to Point", "Auto" );
   var $speed = array( "Auto", "1Gb/s", "2Gb/s", "4Gb/s" );
   var $actualSpeed = array( "Unknown", "1Gb/s", "2Gb/s", "4Gb/s" );
   var $ekm = array( "Not Supported", "Not Supported", "Not Supported", "Application Managed Encryption", "Library Managed Encryption" );

   function Drive( $drive = null )
   {
      parent::InfoXML();
      $this->drive = $drive;
      $this->initializeVariables();
      $this->getData();
   }

   function initializeVariables()
   {
      $this->owner = $this->INTERFACE_ERROR_STRING;
      $this->location = $this->INTERFACE_ERROR_STRING;
      $this->serialNumber = $this->INTERFACE_ERROR_STRING;
      $this->reportedSerialNumber = $this->INTERFACE_ERROR_STRING;
      $this->status = $this->INTERFACE_ERROR_STRING;
      $this->encryption = $this->INTERFACE_ERROR_STRING;
      $this->mountStatus = $this->INTERFACE_ERROR_STRING;
      $this->vendor = $this->INTERFACE_ERROR_STRING;
      $this->model = $this->INTERFACE_ERROR_STRING;
      $this->firmwareVersion = $this->INTERFACE_ERROR_STRING;
      $this->type = $this->INTERFACE_ERROR_STRING;
      $this->interfaceType = $this->INTERFACE_ERROR_STRING;
      $this->interfaceAddress = $this->INTERFACE_ERROR_STRING;
      $this->requestedConnectionType = $this->INTERFACE_ERROR_STRING;
      $this->actualConnectionType = $this->INTERFACE_ERROR_STRING;
      $this->requestedConnectionSpeed = $this->INTERFACE_ERROR_STRING;
      $this->actualConnectionSpeed = $this->INTERFACE_ERROR_STRING;
      $this->loopID = $this->INTERFACE_ERROR_INT;
   }

   function getData()
   {
      if( isset($this->drive) )
      {
         if( $this->drive->name == "Physical Library" )
            $this->owner = "Unassigned";
         else
            $this->owner = $this->drive->name;
         $this->location = $this->drive->location;
         $this->serialNumber = trim($this->drive->serialNumber);
         $this->reportedSerialNumber = trim($this->drive->logicalSerialNumber);
         $this->status = $this->drive->status;
         $this->encryption = $this->ekm[ $this->drive->encryptionMethod % 5 ];
         $this->mountStatus = $this->drive->barcode == "" ? "EMPTY" : $this->drive->barcode;
         $this->vendor = $this->drive->vendor;
         $this->model = trim($this->drive->model);
         $this->firmwareVersion = trim($this->drive->firmwareLevel);
         $this->type = $this->drive->driveType;
         // $this->interfaceType = $this->drive->interfaceType;
         if( $this->drive->interfaceTypeEnum == 1 ) // Fibre
         {
            $this->interfaceAddress = $this->drive->wwpn;
            $this->interfaceType = "FIBRE";
         }
         else if( $this->drive->interfaceTypeEnum == 2 ) // SCSI
         {
            $this->interfaceAddress = $this->drive->scsiId;
            $this->interfaceType = "SCSI";
         }
         else
         {
            $this->interfaceAddress = $this->drive->wwpn;
            $this->interfaceType = "SAS";
         }

         if( $this->drive->interfaceTypeEnum != 2 )
         {
            $this->requestedConnectionType = $this->topology[ $this->drive->topology % 5 ];
            $this->actualConnectionType = $this->topology[ $this->drive->topologyAct % 5 ];
            $this->requestedConnectionSpeed = $this->speed[ $this->drive->speed % 4 ];
            $this->actualConnectionSpeed = $this->actualSpeed[ $this->drive->speedAct ];
            $this->loopID = $this->drive->loopId;
         }
      }
   }

   function toXML()
   {
      $out = $this->tab1 . "<Drive>\n";
      $this->createTag( $out, $this->tab2, "Owner", $this->owner );
      $this->createTag( $out, $this->tab2, "Location", $this->location );
      $this->createTag( $out, $this->tab2, "SerialNumber", $this->serialNumber );
      $this->createTag( $out, $this->tab2, "ReportedSerialNumber", $this->reportedSerialNumber );
      $this->createTag( $out, $this->tab2, "Status", $this->status );
      $this->createTag( $out, $this->tab2, "Encryption", $this->encryption );
      $this->createTag( $out, $this->tab2, "MountStatus", $this->mountStatus );
      $this->createTag( $out, $this->tab2, "Vendor", $this->vendor );
      $this->createTag( $out, $this->tab2, "Model", $this->model );
      $this->createTag( $out, $this->tab2, "FirmwareVersion", $this->firmwareVersion );
      $this->createTag( $out, $this->tab2, "Type", $this->type );
      $this->createTag( $out, $this->tab2, "InterfaceType", $this->interfaceType );
      $this->createTag( $out, $this->tab2, "InterfaceAddress", $this->interfaceAddress );
      if( isset( $this->drive ) && $this->drive->interfaceTypeEnum == 1 )
      {
         $out .= $this->tab2 . "<Connection>\n";
         $this->createTag( $out, $this->tab3, "RequestedConnectionType", $this->requestedConnectionType );
         $this->createTag( $out, $this->tab3, "ActualConnectionType", $this->actualConnectionType );
         $this->createTag( $out, $this->tab3, "RequestedConnectionSpeed", $this->requestedConnectionSpeed );
         $this->createTag( $out, $this->tab3, "ActualConnectionSpeed", $this->actualConnectionSpeed );
         $this->createTag( $out, $this->tab3, "LoopID", $this->loopID );
         $out .= $this->tab2 . "</Connection>\n";
      }

      $out .= $this->tab1 . "</Drive>\n";
      return $out;
   }
}
?>
