<?
/*
 * Quantum Confidential
 *
 * Automated Tape Library Software Source
 * (C) Copyright Quantum 2009. All Rights Reserved.
 * Non-published
 * 
 * $Id: $
 */
class NetworkInfoXML extends InfoXML
{
   var $list = array();
   var $user = null;
   var $EX_networkInfoIpv4 = null;
   var $EX_networkInfoIpv6 = null;

   function NetworkInfoXML( $user = null )
   {
      parent::InfoXML();
      $this->user = $user;
      $this->getData();
   }

   function getData()
   {
      $start = gettimeofday();
      $this->EX_networkInfoIpv4 = @get_network_configuration_ipv4( $this->user );
      @$this->checkError( $this->EX_networkInfoIpv4, "get_network_configuration_ipv4", $start );
      $start = gettimeofday();                                                                                                                                      
      $ipv6Enabled = @is_ipv6_enabled();                                                                                                                            
      @$this->checkError( $ipv6Enabled, "is_ipv6_enabled", $start );                                                                                                
      $start = gettimeofday();                                                                                                                                      
      if( isset( $ipv6Enabled ) && $ipv6Enabled )                                                                                                                   
      {                                                                                                                                                             
         $this->EX_networkInfoIpv6 = @get_all_ipv6_addresses( $this->user );                                                                                        
         @$this->checkError( $this->EX_networkInfoIpv6, "get_all_ipv6_addresses", $start );                                                                         
      }

      if( isset( $this->EX_networkInfoIpv4 ) )
      {
         $net = new Network( "IPv4", $this->EX_networkInfoIpv4->ip );
         $this->list[] = $net;
      }

      if( isset( $this->EX_networkInfoIpv6 ) )
      {
         foreach( $this->EX_networkInfoIpv6 as $item )
         {
            $net = new Network( "IPv6", $item );
            $this->list[] = $net;
         }
      }
   }

   function toXML()
   {
      $out = $this->tab1 . "<NetworkInfo>\n";
      $count = count($this->list);
      for($i = 0; $i < $count; $i++ )
         $out .= $this->list[$i]->toXML();
      $dns = new DNSinfoXML( $this->user );
      $out .= $dns->toXML();
      $out .= $this->tab1 . "</NetworkInfo>\n";
      return $out;
   }
}

class Network extends InfoXML
{
   var $protocol = "";
   var $address = "";

   function Network( $protocol="IPv4", $address="" )
   {
      parent::InfoXML();
      $this->protocol = $protocol;
      $this->address = $address;
   }

   function toXML()
   {
      $out = $this->tab2 . "<Network>\n";
      $this->createTag( $out, $this->tab3, "Protocol", $this->protocol );
      $this->createTag( $out, $this->tab3, "IPAddress", $this->address );
      $out .= $this->tab2 . "</Network>\n";
      return $out;
   }
}

class DNSinfoXML extends InfoXML
{
   var $list = array();
   var $user = null;
   var $isEnabled = false;
   var $EX_dnsInfo = null;

   function DNSinfoXML( $user = null )
   {
      parent::InfoXML();
      $this->user = $user;
      $this->getData();
   }

   function getData()
   {
      $start = gettimeofday();
      $this->EX_dnsInfo = @get_network_dns( $this->user );
      @$this->checkError( $this->EX_dnsInfo, "get_network_dns()", $start );
      if( isset( $this->EX_dnsInfo ) && $this->EX_dnsInfo->dnsPrimary != "" )
         $this->isEnabled = true;
   }

   function toXML()
   {
      $out = $this->tab2 . "<DNS>\n";
      if( $this->isEnabled )
      {
         $out .= $this->tab3 . "<State>Enabled</State>\n";
         if( $this->EX_dnsInfo->dnsPrimary != "" )
            $out .= $this->tab3 . "<IPAddress>" . $this->EX_dnsInfo->dnsPrimary . "</IPAddress>\n";
         if( $this->EX_dnsInfo->dnsSecondary != "" )
            $out .= $this->tab3 . "<IPAddress>" . $this->EX_dnsInfo->dnsSecondary . "</IPAddress>\n";
      }
      else if( !isset( $this->EX_dnsInfo ) )
      {
         $this->createTag( $out, $this->tab3, "State", $this->INTERFACE_ERROR_STRING );
      }
      else
      {
         $out .= $this->tab3 . "<State>Disabled</State>\n";
      }

      $out .= $this->tab2 . "</DNS>\n";
      return $out;
   }
}
?>
