<?
/*
 * Quantum Confidential
 *
 * Automated Tape Library Software Source
 * (C) Copyright Quantum 2009. All Rights Reserved.
 * Non-published
 * 
 * $Id: $
 */
class PartitionInfoXML extends InfoXML
{
   var $list = array();
   var $user = null;
   var $EX_partitionList = null;
   var $EX_productInfo = null;
   var $EX_ekmInfo = null;

   function PartitionInfoXML( $user = null, $productInfo = null )
   {
      parent::InfoXML();
      $this->user = $user;
      $this->EX_productInfo = $productInfo;
      $this->getData();
   }

   function getData()
   {
      $sortCriteria = new SortCriteria( 1, "ASC", "", "*" );
      $start = gettimeofday();
      $this->EX_partitionList = @get_logical_libraries($this->user, $sortCriteria);
      @$this->checkError( $this->EX_partitionList, "get_logical_libraries", $start );
      if( isset( $this->EX_partitionList ) )
      {
         foreach( $this->EX_partitionList as $item )
         {
            $part = new Partition( $this->user, $item, $this->EX_productInfo );
            $this->list[] = $part;
         }
      }
   }

   function toXML()
   {
      $count = count( $this->list );
      if( $count > 0 )
      {
         $out = $this->tab . "<PartitionInfo>\n";
         for( $i = 0; $i < $count; $i++ )
         {
            $out .= $this->list[$i]->toXML();
         }
         $out .= $this->tab . "</PartitionInfo>\n";
         return $out;
      }
   }
}

class Partition extends InfoXML
{
   var $list = array();
   var $p_name;
   var $domain;
   var $mode;
   var $encryption;
   var $autoClean;
   var $serialNumber;
   var $storageSlotCount;
   var $driveCount;
   var $ieSlotCount;
   var $mediaCount;

   var $user = null;
   var $partitionInfo = null;
   var $productInfo = null;
   var $EX_driveList = null;
   var $methods  = array( 0 => "Not Supported", 1 => "Not Supported", 2 => "Not Supported", 3 =>'Application Managed Encryption', 4 =>'Library Managed Encryption' );

   function Partition( $user = null, $partitionInfo = null, $productInfo = null )
   {
      parent::InfoXML();
      $this->user = $user;
      $this->initializeVariables();
      $this->partitionInfo = $partitionInfo;
      $this->productInfo = $productInfo;
      $this->getData();
   }

   function initializeVariables()
   {
      $this->p_name = $this->INTERFACE_ERROR_STRING;
      $this->domain = $this->INTERFACE_ERROR_STRING; 
      $this->mode = $this->INTERFACE_ERROR_STRING; 
      $this->encryption = $this->INTERFACE_ERROR_STRING; 
      $this->autoClean = $this->INTERFACE_ERROR_STRING; 
      $this->serialNumber = $this->INTERFACE_ERROR_STRING; 
      $this->storageSlotCount = $this->INTERFACE_ERROR_INT; 
      $this->driveCount = $this->INTERFACE_ERROR_INT; 
      $this->ieSlotCount = $this->INTERFACE_ERROR_INT; 
      $this->mediaCount = $this->INTERFACE_ERROR_INT; 

      $this->accessType = $this->INTERFACE_ERROR_STRING; 
      $this->accessLocation = $this->INTERFACE_ERROR_STRING; 
   }

   function getData()
   {
      if( $this->partitionInfo != null )
      {
         $start = gettimeofday();
         $ekm = @get_partition_enryption( $this->user, $this->partitionInfo->guid );
         @$this->checkError( $ekm, "get_partition_enryption", $start );

         $this->domain = "LTO";
         if( isset( $ekm ) )
            $this->encryption = $this->methods[ $this->getEncryptMethod( $ekm ) % 5 ];

         if( isset( $this->productInfo ) )
         {
            if( $this->productInfo->cleaningSlotCount > 0 )
               $this->autoClean = "Enabled";
            else
               $this->autoClean = "Disabled";
         }

         if( isset( $this->productInfo ) )
         {
            $this->p_name = $this->partitionInfo->name;
            $this->mode = $this->partitionInfo->status;
            $this->serialNumber = $this->partitionInfo->serialNumber;
            $this->storageSlotCount = $this->partitionInfo->totalSlots;
            $this->driveCount = $this->partitionInfo->totalDrives;
            $this->ieSlotCount = $this->partitionInfo->totalMailbox;
            $this->mediaCount = $this->partitionInfo->totalMedia;
         }
         $this->getAccessType();
      }
   }

   function setAutoClean( $enable = false )
   {
      if( $enable )
         $this->autoClean = "Enabled";
      else
         $this->autoClean = "Disabled";
   }

   function getEncryptMethod( $ekm = null )
   {
      $method = $ekm->method;
      $sortCriteria = new SortCriteria();
      $start = gettimeofday();
      $this->EX_driveList = @get_all_drives_by_library($this->user, $this->partitionInfo->guid, $sortCriteria, 0, 255);
      @$this->checkError( $this->EX_driveList, "get_all_drives_by_library", $start );
      $start = gettimeofday();
      if( isset( $this->EX_driveList ) )
      {
         $lto4IBMdrive = false;
         foreach( $this->EX_driveList as $drive )
         {
            if( $drive->driveType == 'LTO-4' 
               && $drive->vendor == "IBM" 
               && $drive->name == $this->partitionInfo->name )
            {
               if( $drive->interfaceType == "Fibre" )
               {
                  $lto4IBMdrive = true;
               }
            }
         }

         if( ! $lto4IBMdrive )
            $method = 1;
         return $method;
      }
   }

   function getAccessType()
   {
      $start = gettimeofday();
      $cpd = @get_command_path_drives( $this->user, $this->partitionInfo->guid );
      @$this->checkError( $cpd, "get_command_path_drives", $start );
      if( isset( $cpd ) )
      {
         if( count( $cpd ) > 0 )
         {
            $accessType = "Drive";
            $accessLocation = $this->INTERFACE_ERROR_STRING;
            if( isset( $this->EX_driveList ) )
            {
               foreach( $this->EX_driveList as $drive )
               {
                  if( $drive->guid == $cpd[0] )
                  {
                     $accessLocation = $drive->location;
                  }
               }
            }
            $access = new AccessPath( $accessType, $accessLocation );
            $this->list[] = $access;
         }
         else
         {
            $start = gettimeofday();
            $EX_bladeList = @get_blades( $this->user );
            @$this->checkError( $EX_bladeList, "get_blades", $start );
            $accessType = "I/O Blade";
            if( isset( $EX_bladeList ) )
            {
               foreach( $EX_bladeList as $item )
               {
                  $access = new AccessPath( $accessType, $item->location );
                  $this->list[] = $access;
               }
            }
         }
      }
      else
      {
         $access = new AccessPath( $this->INTERFACE_ERROR_STRING, $this->INTERFACE_ERROR_STRING );
         $this->list[] = $access;
      }
   }

   function toXML()
   {
      $out = $this->tab1 . "<Partition>\n";
      $this->createTag( $out, $this->tab2, "Name", $this->p_name );
      $this->createTag( $out, $this->tab2, "Domain", $this->domain );
      $this->createTag( $out, $this->tab2, "Mode", $this->mode );
      $this->createTag( $out, $this->tab2, "Encryption", $this->encryption );
      $this->createTag( $out, $this->tab2, "AutoClean", $this->autoClean );
      $this->createTag( $out, $this->tab2, "SerialNumber", $this->serialNumber );
      $this->createTag( $out, $this->tab2, "StorageSlotCount", $this->storageSlotCount );
      $this->createTag( $out, $this->tab2, "DriveCount", $this->driveCount );
      $this->createTag( $out, $this->tab2, "IESlotCount", $this->ieSlotCount );
      $this->createTag( $out, $this->tab2, "MediaCount", $this->mediaCount );
      $count = count( $this->list );
      if( $count > 0 )
      {
         $out .= $this->tab2 . "<AccessInfo>\n";
         for( $i = 0; $i < $count; $i++ )
         {
            $out .= $this->list[$i]->toXML();
         }
         $out .= $this->tab2 . "</AccessInfo>\n";
      }
      $out .= $this->tab1 . "</Partition>\n";
      return $out;
   }
}

class AccessPath extends InfoXML
{
   var $accessType = "";
   var $accessLocation = "";
   var $user = null;

   function AccessPath( $accessType, $accessLocation )
   {
      parent::InfoXML();
      $this->accessType = $accessType;
      $this->accessLocation = $accessLocation;
   }

   function toXML()
   {
      $out .= $this->tab3 . "<Path>\n";
      $this->createTag( $out, $this->tab4, "Type", $this->accessType );
      $this->createTag( $out, $this->tab4, "Location", $this->accessLocation );
      $out .= $this->tab3 . "</Path>\n";
      return $out;
   }
}
?>
