
//------------------------------------------------------------------------------
//  Licensed Materials - Property of IBM
//  (c) Copyright IBM Corporation 2001,2002 All Rights Reserved.
//  US Government Users Restricted Rights - Use, duplication or disclosure 
//  restricted by GSA ADP Schedule Contract with IBM Corp.
//------------------------------------------------------------------------------
//  Utilities or Commom js functions
//  
//  Filename:  utils.js
//
//  This file contains common javascript code for the gui.  See functions
//  for details.
//
//      
//  Change History:
//      
//  Date        Defect  Changed By  Description of Change
//  ----        ------  ----------  ---------------------
//  07/21/2005          rwinston    File Created
//
//------------------------------------------------------------------------------  

//------------------------------------------------------------------------------
// setAllCheckboxes
//
// This function checks all the checkboxes within the specified form.
//
//------------------------------------------------------------------------------           
function setAllCheckboxes(form, value) 
{
    var inputElements = form.elements;
    var numberCheckboxes = 0;
    
    for (iCount=0; iCount<inputElements.length; iCount++)
    {
        var inputEl = inputElements[iCount];
        
        if (inputEl)
        {
            if ((inputEl.type=='checkbox') && (!inputEl.disabled))
            {
                inputEl.checked = value;
                inputEl.defaultChecked = value;

                numberCheckboxes++;
            }
        }
    }
    if (value == false)
    {
        document.getElementById('numberSelected').value=0;
    }
    else
    {
        document.getElementById('numberSelected').value=numberCheckboxes;
    }

    document.getElementById('table_select').innerHTML = document.getElementById('numberSelected').value;
    
    return;
}


//------------------------------------------------------------------------------
// getSelectedRadioButton
//
// This function returns the selected radio within the specified form.
//
//------------------------------------------------------------------------------                                  
function getSelectedRadioButton(form) 
{
    var inputElements = form.elements;

    for (iCount=0; iCount<inputElements.length; iCount++)
    {
        var inputEl = inputElements[iCount];    
        
        if (inputEl)
        {
            if ((inputEl.type=='radio') && (inputEl.checked))
            {
                return inputEl;
            }
        }
    }

    return 0;
}                         

//------------------------------------------------------------------------------
// closeMessage
//
// This function hides the div containing the message box within 
// the specified form.
//
//------------------------------------------------------------------------------                                  
function closeMessage()
{
    var divObj = document.getElementById("msg");

    //form.Message.value=0;
    //form.submit();

    divObj.className = "div-hide";
    
    return;
}

//------------------------------------------------------------------------------
// colWork
//
// This function builds an outgoing form to POST to self.  Initiates a 
// column sort for the main table in specified form.
//
// This function is set to be depreciated.
// To be replaced by goJsort, Awaiting testing and approval.
// bbryan@us.ibm.com 06/10/08
//------------------------------------------------------------------------------                                  
function colWork(form, sortby)
{

    //alert(form.name+" "+form.sortBy.value+" "+sortby);
    form.sortBy.value=sortby;    
    form.sortIn.value="ascending";

    document.body.style.cursor = 'wait';
    form.submit();

    
    return;
}



//------------------------------------------------------------------------------
// imagePreload()
//
// This function will preload all images into a clients browser cache.
// This is useful if you have rollover images so that it will rollover immediatly
// instead of having to perform a server request for the image.
//
// Call the function as imagePreload("image 1 url", "image 2 url", "..." , "image n url")
//------------------------------------------------------------------------------ 
function imagePreload()
{
    if (typeof(arguments) != 'undefined')
    {
        for (i = 0; i < arguments.length; i++ )
        {
            if (typeof(arguments[i]) == "object")
            {
                for (k = 0; k < arguments[i].length; k++)
                {
                    var oImage = new Image;
                    oImage.src = arguments[i][k];
                }
            }

            if (typeof(arguments[i]) == "string")
            {
                var oImage = new Image;
                oImage.src = arguments[i];
            }
        }
    }
}



//------------------------------------------------------------------------------
// goJsort(tableId, colNum, sortType)
// 
// This function is the new client side js sort implemented for table sorting.
// It has been written using DOM standards so the code is not the easiest and
// was designed to work with the standard orion tables.  The table to be sorted
// must have a uniqe ID to the page.  Also the carat images must have a uniqe ID
// to the table being sorted and must match their location in the column.
// This function not only was also designed to sort a table (which is easy to do)
// but also keeps the HTML code that is attached to each cell and row in the table.
// If you simply sort the text in the table the original tags will remain where they
// are when the page first loaded or be destroyed.
// If the sort function did not keep the embedded HTML with the cells/rows when
// performing an operation on the table like move/delete/change a(n) cartridge/library/slot
// after a sort a different item would change or there would be broken code.
//
// This function is a bi-directional sort but always does an ascending sort first.
//
// It is necessary to inlucde two variables in the main document before the table
// headers are constructed. currentCol is always set to zero and previousCol needs
// to be set to the column that $sortBy is set to in the PHP code.
// currentCol = 0;
// previousCol = <PHP default sort column>;
//
// tableId = The id of the table you want to sort
// colNum = the column number you want to sort on index 0 for the first column
// sortType = the data type you are sorting, 'alphaNum' for sorting columns that
//              contain both alpha and numeric chars
//------------------------------------------------------------------------------
function goJsort(tableId, colNum, sortType)
{
    // declare variables
    var tableObj = document.getElementById(tableId);
    var tableRows = tableObj.getElementsByTagName('tr');
    var sortableRows = new Array();

    // update currentCol
    currentCol = colNum;

    // get the text of the column we want to sort by and get the entire row object
    // and pack into a row object called rowObj, and pack each rowObj into a new
    // array, this makes it easy to sort.  We can ignore the first row because it
    // is the table headers
    for (i = 1; i < tableRows.length; i++)
    {
        sortableRows[i - 1] = new rowObj(tableRows[i].getElementsByTagName('td')[colNum].getElementsByTagName('span')[0].firstChild.data, tableRows[i].cloneNode(true));
    }

    // if we sort on the same column a second time all we need to do is reverse the
    // array or if the column was sorted by the php POST
    if (colNum == previousCol)
    {
        sortableRows.reverse();
        flipCarat(tableId, colNum)
    }
    else
    {
        // I used a switch here for future expansion, in case there are new cases
        // that require a special breakdown in order to sort
        switch (sortType)
        {
            // sort columns with alphanumeric chars in them
            case 'alphaNum':
                sortableRows.sort(compareAlphaNum);
            break;
        }
        changeCarat(tableId, colNum);
    }

    // now write the sorted array of rowObjs back into the table
    for (i = 1; i < tableRows.length; i++)
    {
        tableRows[i].parentNode.replaceChild(sortableRows[i - 1].getRowNode(), tableRows[i]);
    }

    // remove all check boxes and reset counters
    if (document.getElementById('table_select'))
    {
        setAllCheckboxes(document.form_main, false);
    }

    // set the clicked column to the previousCol
    previousCol = colNum;
    return;
}



//------------------------------------------------------------------------------
// changeCarat(tableId, colNum)
//
// This function will change the carat in the table header when doing a goJsort.
//
// tableId = is the table id where the carat is
// colNum = is the column carat to change
//------------------------------------------------------------------------------
function changeCarat(tableId, colNum)
{
    // declair variables
    var tableObj = document.getElementById(tableId);
    var caratImages = tableObj.getElementsByTagName('img');

    // cycle through all of the carats to find the right one
    for (i = 0; i < caratImages.length; i++)
    {
        // change all carats to the thin down arrow
        caratImages[i].src = "images/tbl48.gif";
        caratImages[i].setAttribute("alt", "Click to Sort Ascending");
        caratImages[i].setAttribute("title", "Click to Sort Ascending");
        // get the right carat and change that to the full up arrow
        if (caratImages[i].id == colNum)
        {
            caratImages[i].src = "images/tbl49.gif";
            caratImages[i].setAttribute("alt", "Click to Sort Descending");
            caratImages[i].setAttribute("title", "Click to Sort Descending");
        }
    }
}



//------------------------------------------------------------------------------
// flipCarat(tableId, colNum)
//
// This function will simply switch the carat of the column
//
// tableId = the table id where the carat is
// colNum = is the column number
//------------------------------------------------------------------------------
function flipCarat(tableId, colNum)
{
    // declair variables
    var tableObj = document.getElementById(tableId);
    var caratImages = tableObj.getElementsByTagName('img');

    // cycle through all the carats
    for (i = 0; i < caratImages.length; i++)
    {
        // find the right carat
        if (caratImages[i].id == colNum)
        {
            // split the url of the image up
            var textarr = caratImages[i].src.split("/");
            var text = textarr[textarr.length - 1];

            // compare url and change
            if (text == "tbl49.gif")
            {
                caratImages[i].src = "images/tbl50.gif";
                caratImages[i].setAttribute("alt", "Click to Sort Ascending");
                caratImages[i].setAttribute("title", "Click to Sort Ascending");
            }
            else
            {
                caratImages[i].src = "images/tbl49.gif";
                caratImages[i].setAttribute("alt", "Click to Sort Descending");
                caratImages[i].setAttribute("title", "Click to Sort Descending");
            }
        }
    }
}



//------------------------------------------------------------------------------
// rowObj(text, rowNode)
//
// This function is the constructor for the row object.  It contains the text
// that is to be sorted on and the entire row of the table.
//
// Contains the getters for the text and for the row.
//
// text = the text that you will sorting by
// rowNode = is the entire row node of the table
//------------------------------------------------------------------------------
function rowObj(text, rowNode)
{
    this.text = text;
    this.rowNode = rowNode;

    if (typeof(prototype_called) == 'undefined')
    {
        prototype_called = true;
        rowObj.prototype.getText = getText;
        rowObj.prototype.getRowNode = getRowNode;
    }
}



//------------------------------------------------------------------------------
// getText()
//
// This function is the getter for the rowObj
//
// returns the text for the given rowObj
//------------------------------------------------------------------------------
function getText()
{
    return this.text;
}



//------------------------------------------------------------------------------
// getRowNode()
//
// This function is the getter for the rowObj
//
// returns the rowNode for the given rowObj
//------------------------------------------------------------------------------
function getRowNode()
{
    return this.rowNode;
}



//------------------------------------------------------------------------------
// compareAlphaNum(a, b)
//
// This is the sort function that is called in order to sort a list of rowObj's
//
// returns -1, 0, 1
//------------------------------------------------------------------------------
function compareAlphaNum(a, b)
{
    var x = a.getText();
    var y = b.getText();
    return ((x < y) ? -1 : ((x > y) ? 1 : 0));
}



//------------------------------------------------------------------------------
// toggleSelect
//
// This function toggles the checked attribute for radio and checkboxes.
//
//------------------------------------------------------------------------------                                  
function toggleSelect(elementId)
{
    var inEl = document.getElementById(elementId);

    if (inEl.checked==true)
    {
        inEl.checked=false;
        // document.getElementById('numberSelected').value--;
        // document.getElementById('table_select').innerHTML=document.getElementById('numberSelected').value;
    }
    else
    {
        inEl.checked=true;
        //document.getElementById('numberSelected').value++;
        //document.getElementById('table_select').innerHTML=document.getElementById('numberSelected').value;
    }
    
    return;
}

function updateNumberSelected(elementId)
{
    var inEl = document.getElementById(elementId);

    if (inEl.checked==false)
    {
        document.getElementById('numberSelected').value--;
        document.getElementById('table_select').innerHTML = document.getElementById('numberSelected').value;
    }
    else
    {
        document.getElementById('numberSelected').value++;
        document.getElementById('table_select').innerHTML = document.getElementById('numberSelected').value;
    }
    return;
}


//------------------------------------------------------------------------------
// toggleSelect
//
// This function toggles the checked attribute for radio and checkboxes.
//
//------------------------------------------------------------------------------                                  
function refreshPage()
{
    document.form_main.submit();

    return;
}

//------------------------------------------------------------------------------                                  
function toggleSort(elementId, newHREF)
{
    inEl = document.getElementById(elementId);

    inEl.href.value = newHREF;
    
    return;
}


function validateEmailAddr(str)
{
    var at="@";
    var dot=".";
    var lat=str.indexOf(at); //position of @
    var lstr=str.length; //string size
    var ldot=str.indexOf(dot); //position of .

    //there is no @ in string or there is at position 0 or last position in string       
    if (str.indexOf(at)==-1 || str.indexOf(at)==0 || str.indexOf(at)==lstr)
    {
        return false;
    }

    //same for . (as for @)
    if (str.indexOf(dot)==-1 || str.indexOf(dot)==0 || str.indexOf(dot)==lstr)
    {
        return false;
    }

    //check for extra @ after the first @    
    if (str.indexOf(at,(lat+1))!=-1)
    {
        return false;
    }

    //check for . right after or before the @
    if (str.substring(lat-1,lat)==dot || str.substring(lat+1,lat+2)==dot)
    {
        return false;
    }

    //check for spaces 
    if (str.indexOf(" ")!=-1)
    {
        return false;
    }

    return true;           

}


function DisplayHelpWindow()
{
    window.open( "help/help_index.htm", "helpWindow",
                 "width=700,height=550,status=yes,resizable=yes,toolbar=yes,scrollbars=yes,location=no,menubar=no");
    return;
}

// Looking for either IP or hostname
function checkHostOrIP( aField, aBlankOk )
{
    var pattern = /[^\.:0-9_A-Za-z-]/;
    var string = aField.value;
    var errorString = "";
    var ret;
    
    // If it's BLANK and blanks are ok Just Exit
    if (aField.value == "" && aBlankOk == "1")
        return true;

    if (aField.value == ""  && aBlankOk != "1")
        errorString = errorString + theName + ' BLANK cannot be used here. Please enter a valid IP address or hostname.';
    else if (string.match(pattern))
        errorString = "Please enter a valid IP address or host name";

    if (errorString == "")
    {
        // determine if we need to validate IPv6 or IPv4
        if (aField.value.search(":") != -1)
        {
            ret = verifyIPv6( aField, aBlankOk );
        }
        else
        {
            tmp = aField.value.split(".");
            if (tmp.length == 4)   // Is it a HOSTNAME or IPv4 address
            {
                ret = verifyIPv4( aField, aBlankOk );
            }
        }
    }

    if (errorString != "" || ret == false)
    {
        if (errorString != "")
        {
            alert (errorString);
            // aField.focus();
        }

        //  window.status = errorString;
        //  focusField = aField;
        //  setTimeout( setTheFocus, 10 );
        return false;
    }
    return true;
}



/****************************************************************
 * Verify that the data in the input field passed is a valid 
 * IPv4 address.
 * If aBlankOK is true then Blank IPs are OK
 * Also setes the Global Var okToApply according to the 
 * validation of the field.
 ****************************************************************/
function verifyIPv4( aField, aBlankOK ) 
{
    var IPvalue = aField.value;

    var errorString = "";
    theName = "IPv4 address";

    var ipPattern = /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/;
    
    //alert(aBlankOK);
    if ((aBlankOK) && (IPvalue == ""))
    {
        return true;
    }

    if (IPvalue == "0.0.0.0" && aBlankOK != "1")
    {
        errorString = errorString + theName + ': '+IPvalue+' is a special IP address and cannot be used here. Please enter a valid IP address.';
    }
    if (IPvalue == "255.255.255.255")
    {
        errorString = errorString + theName + ': '+IPvalue+' is a special IP address and cannot be used here. Please enter a valid IP address.';
    }
    if (IPvalue == ""  && aBlankOK != "1")
    {
        errorString = errorString + ' BLANK cannot be used here. Please enter a valid IP address.';
    }


    if (errorString == "")
    {
        var ipArray = IPvalue.match(ipPattern); 
        if (ipArray == null)
        {
            errorString = errorString + theName + ': '+IPvalue+' is not a valid IP address. Please enter a valid IP address.';
            //aField.focus();
        }
        else
        {
            for (i = 1; i < 5; i++)
            {
                thisSegment = ipArray[i];
                if (thisSegment > 255)
                {
                    errorString = errorString + theName + ': '+IPvalue+' is not a valid IP address. Please enter a valid IP address.';
                    //aField.focus();
                    break;
                }
                if ((i == 0) && (thisSegment > 255))
                {
                    errorString = errorString + theName + ': '+IPvalue+' is a special IP address and cannot be used here. Please enter a valid IP address.';
                    // aField.focus();
                    break;
                }
            }
        }
    }

    if (errorString != "")
    {
        // Display the error and return the focus to the field.
        alert (errorString);
        //window.status = errorString;

        /********************************************************
         * Note: the setting of the focus DOES NOT WORK like
         * aField.focus();
         * You must call the focus function after the KEY
         * that made the user leave the field gets processed, 
         * so you MUST use a timer.
         ********************************************************/
        //okToApply = false;
        // focusField = aField;
        // setTimeout( setTheFocus, 10 );
        // aField.focus();
        return false;
    }
    // else
    // okToApply = true;

    return true;
}




/****************************************************************
 * Verify that the data in the input field passed is a valid 
 * IPv6 address.
 * If aBlankOK is true then Blank IPs are OK
 * Also sets the Global Var okToApply according to the 
 * validation of the field.
 ****************************************************************/
function verifyIPv6( aField, aBlankOK ) 
{
    var IPvalue = aField.value;
    var errorString = "";
    var ret;
    
    theName = "IPv6 address";
    if (IPvalue == "0:0:0:0" && aBlankOK != "1")
        errorString = errorString + theName + ': '+IPvalue+' is a special IP address and cannot be used here.';
    if (IPvalue == ""  && aBlankOK != "1")
        errorString = errorString + theName + ' BLANK cannot be used here. Please enter a valid IP address.';

    if (errorString == "")
    {
        // The validateIPv6 function will display any errors.
        ret = validateIPv6( IPvalue )
    }

    if (errorString != "" || ret == false)
    {
        // Display the error and return the focus to the field.
        if (errorString != "")
        {
            alert (errorString);
            //    aField.focus();
            //     window.status = errorString;
        }

        /********************************************************
         * Note: the setting of the focus DOES NOT WORK like
         * aField.focus();
         * You must call the focus function after the KEY
         * that made the user leave the field gets processed, 
         * so you MUST use a timer.
         ********************************************************/
        // okToApply = false;
        // focusField = aField;
        // setTimeout( setTheFocus, 10 );
        return false;
    }
    // else
    //  okToApply = true;

    return true;

}


function verifyPrefixIPv6( aPrefixObj )
{
    number = Number(aPrefixObj.value);
    if (number < 1 || number > 128)
    {
        alert("Invalid network prefix. It must be between 1 and 128.  Please enter a valid prefix.");
        /********************************************************
        * Note: the setting of the focus DOES NOT WORK like
        * aField.focus();
        * You must call the focus function after the KEY
        * that made the user leave the field gets processed, 
        * so you MUST use a timer.
        ********************************************************/
        //  okToApply = false;
        //  focusField = aPrefixObj;
        //  setTimeout( setTheFocus, 10 );
        //aField.focus();
        return false;
    }

    return true;
}


function verifyGatewayIPv6( aGwObj, aIp, aPrefix )
{
    resultMsgIpv6 = "";   // Clear out each call.
    if (!verifyGatewayIPv6_( aGwObj.value, aIp, aPrefix ))
    {
        alert( resultMsgIpv6 );
        //aField.focus();
        
        /********************************************************
        * Note: the setting of the focus DOES NOT WORK like
        * aField.focus();
        * You must call the focus function after the KEY
        * that made the user leave the field gets processed, 
        * so you MUST use a timer.
        ********************************************************/
        //  okToApply = false;
        //  focusField = aGwObj;
        //  setTimeout( setTheFocus, 10 );
        return false;
    }
    return true;
}






/**************************************************************************
   IPv6 Address Space
   ---------------------------------------------------------------------
    IPv6 Prefix           Allocation              Reference      Note
   -----------           ----------              ---------      ----
   0000::/8              Reserved by IETF        [RFC4291]      [1] [5] 
   0100::/8              Reserved by IETF        [RFC4291]
   0200::/7              Reserved by IETF        [RFC4048]      [2]
   0400::/6              Reserved by IETF        [RFC4291]
   0800::/5              Reserved by IETF        [RFC4291]
   1000::/4              Reserved by IETF        [RFC4291]
   2000::/3              Global Unicast          [RFC4291]      [3]
   4000::/3              Reserved by IETF        [RFC4291]
   6000::/3              Reserved by IETF        [RFC4291]
   8000::/3              Reserved by IETF        [RFC4291]
   A000::/3              Reserved by IETF        [RFC4291]
   C000::/3              Reserved by IETF        [RFC4291]
   E000::/4              Reserved by IETF        [RFC4291]
   F000::/5              Reserved by IETF        [RFC4291]
   F800::/6              Reserved by IETF        [RFC4291]
   FC00::/7              Unique Local Unicast    [RFC4193]
   FE00::/9              Reserved by IETF        [RFC4291]
   FE80::/10             Link Local Unicast      [RFC4291]
   FEC0::/10             Reserved by IETF        [RFC3879]      [4]
   FF00::/8              Multicast               [RFC4291]
   
   Notes:

   [0]  The IPv6 address management function was formally delegated to
        IANA in December 1995 [RFC1881].
   
   [1]  The "unspecified address", the "loopback address", and the IPv6
        Addresses with Embedded IPv4 Addresses are assigned out of the
        0000::/8 address block.
   
   [2]  0200::/7 was previously defined as an OSI NSAP-mapped prefix set
        [RFC-gray-rfc1888bis-03.txt]. This definition has been deprecated
        as of December 2004 [RFC4048].
   
   [3]  The IPv6 Unicast space encompasses the entire IPv6 address range
        with the exception of FF00::/8. [RFC4291] IANA unicast address
        assignments are currently limited to the IPv6 unicast address
        range of 2000::/3. IANA assignments from this block are registered
        in the IANA registry: iana-ipv6-unicast-address-assignments.
   
   [4]  FEC0::/10 was previously defined as a Site-Local scoped address
        prefix. This definition has been deprecated as of September 2004
        [RFC3879].
   
   [5]  0000::/96 was previously defined as the "IPv4-compatible IPv6
        address" prefix.  This definition has been deprecated by [RFC4291].
*****************************************************************************/

var resultMsgIpv6 = ""; // Global used in the IPv6 validation functions.

function validateIPv6( aAddr )
{
    var prefix;
    var tmp;
    var addr;
    var ret;
    
    resultMsgIpv6 = "";   // Clear out each call.
    
    // First was there a net prefix on the Address
    tmp = aAddr.split("/");
    if (tmp.length == 2)
    {
        addr   = tmp[0];
        prefix = tmp[1];
    }
    else
    {
        addr   = tmp[0];
        prefix = "";
    }

    if (checkFormatIPv6( addr ))
    {
        var pattern = /[^a-fA-F0-9]/
        // Now Check each hextet
        hexList = addr.split(":");
        for (i = 0; i < hexList.length; ++i)
        {
            if (hexList[i].length > 4)
            {
                //if ((i==(hexList.length-1)))// && verifyIPv4(hexList[i], 0))
                {
                    //alert("this is recognized");
                }
                //else
                {
                    resultMsgIpv6 = ("Segment is to large in address segment " + (i+1) + " '" + hexList[i] + "'. Please enter a valid IP address.");
                }
            }
            else if (hexList[i].match(pattern))
                resultMsgIpv6 = ("Invalid character found in address segment " + (i+1) + " '" + hexList[i] + "'. Please enter a valid IP address.");
            //else
            //resultMsgIpv6 = ("Match succedded " + hexList[i]);
        }
        if (resultMsgIpv6 == "")
            ret = checkAddressTypeIPv6( addr );
    }

    // Now check the prefix if one was passed in.
    if (ret != false)
        if (prefix.length > 0)
            ret = verifyPrefixIPv6( prefix );


    if (resultMsgIpv6 != "")
    {
        alert( resultMsgIpv6 );
        //aField.focus();
        return false;
    }

    if (ret == false)
        return false;

    return true;
}

function checkFormatIPv6( aAddr )
{
    var hexList;
    var blank = 0;
    
    hexList = aAddr.split(":");
    
    if (hexList.length > 8)
    {
        resultMsgIpv6 = ("Too many segments in address " + aAddr + ".  Please enter a valid IP address.");
        return false;
    }
    else if (hexList < 2)
    {
        resultMsgIpv6 = ("Not enough segments in address " + aAddr + ". Please enter a valid IP address.");
        return false;
    }

    if (aAddr[0] == ":")
    {
        // !?! Not sure about this
        // The spec stated IPv4 address can be writtin as ::ffff:0:0/96
        // and ::1/128 is the loopback address - would this be valid anywhere??????
        resultMsgIpv6 = ("IP Address started with the delimiter in address " + aAddr + ". Please enter a valid IP address.");
        return false;
    }

    for (i = 0; i < hexList.length; ++i)
    {
        if (hexList[i] == "")
        {
            blank++;
            if (blank > 1)
            {
                resultMsgIpv6 = ("Address contains more than one BLANK segment. Please enter a valid IP address.");
                return false;
            }
        }
    }
    return true;
}

function checkAddressTypeIPv6( aAddr )
{
    var unicastMin   = 0x2000;
    var unicastMax   = 0x3FFF;
    var linkMin      = 0xFE80;
    var linkMax      = 0xFEBF;
    var siteMin      = 0xFEC0;
    var siteMax      = 0xFEFF;
    var num          = 0;
    
    hexList = aAddr.split(":");
    
    tmp = "0x" + hexList[0];
    number = Number(tmp);
    //alert("Hextet 0 = " + num + " | unicastMin = " + unicastMin);
    
    if (number >= unicastMin && number <= unicastMax)
        AddrType = "global unicast";
    else if (number >= linkMin && number <= linkMax)
        AddrType = "link-local unicast";
    else if (number >= siteMin && number <= siteMax)
        AddrType = "site-local unicast";
    else
    {
        resultMsgIpv6 = "Address is not a valid type of either:\n'Global unicast', 'Link-local unicast', or 'Site-local unicast'.  Please enter a valid IP address.";
        AddrType = "";
        return false;
    }
    return true;
}


function verifyGatewayIPv6_( aGw, aIp, aPrefix )
{
    var prefixNumber;
    var shiftbits;
    var mask;
    var network16bitFields;
    var ip16bitField;
    var gw16bitField;

    prefixNumber = Number(aPrefix);
    network16bitFields = prefixNumber / 16;
    shiftbits = prefixNumber % 16;

    GW = aGw.split(":");
    IP = aIp.split(":");
    // The piece of the network address that fits nicely into the 16bit fields
    for (i=0; i< network16bitFields; i++)
    {
        gw16bitField = Number(GW[i]);
        ip16bitField = Number(IP[i]);
        if (ip16bitField != gw16bitField)
        {
            resultMsgIpv6 = "Gateway and static IP address with current network prefix is not valid.  Please enter a valid Gateway and IP address.";
            return false;
        }
    }

    // The network piece of addres that does not fit nicely into 16bit field bounds(16,32,48,64,80,96,112,128)
    if (shiftbits > 0)
    {
        // The part of the address that somewhere inside the 16bit field.
        ip16bitField = Number(IP[network16bitFields]);
        gw16bitField = Number(GW[network16bitFields]);
        
        // Create mask to use to compare gateway address with ip address. If
        // prefix was 37, this would be 5 bits past 32, so we shift 5, and do
        // an exclusive-or to create the following mask "1111100000000000"
        mask = 0xffff >> shiftbits;
        mask = mask ^ 0xFFFF;
        
        if ((ip16bitField & mask) != (gw16bitField & mask))
        {
            resultMsgIpv6 = "Static IP address and Gateway with current network prefix is not valid.  Please enter a valid Gateway and IP address.";
            return false;
        }
    }
    return true;
}

/****************************************************************
    Determin which verify function to call, IPv4 or IPv6
 ****************************************************************/
function verifyIP( aField, aBlankOK ) 
{
    var ret = false;
    // First detemin if the address is IPv4 or IPv6
    // Remember IPv6 can have "." but not ":" and IPv6 can have both, but must have a :
    if (aField.value.search(":") == -1)
    {
        // Must be an IPv4 address
        ret =  verifyIPv4( aField, aBlankOK );
    }
    else    // Default to IPv6 if it had any :'s in it
    {
        ret = verifyIPv6( aField, aBlankOK );
    }
    if (ret == false)
    {
        // window.status = errorString;
        // focusField = aField;
        // setTimeout( setTheFocus, 10 );
        return false;
    }

    return ret;
}


